%
% BDIAG.M
%
%
% Creates a block diagonal, given a block (matrix) and the
% number of times it must be replicated.
%
% b_diag = bdiag(block,P,'type')
%
% IN:
% block - block to be made into a block diagonal (L*K)

% P - no. of times block must be replicated
% type - (optional). If no third arg. is given, bdiag creates a
% block diagonal with the blocks on the main diagonal.
% 'main' does the same. type='anti' results in the blocks
% being placed on the antidiagonal.
%
% OUT:
% b_diag = PL x PK block diagonal
%
%
% NOTE: Try replacing the ELSEIF by ELSE for some good fun!!! Try explaining
% the program's behaviour when the ELSEIFs are replaced by ELSEs.
%
%
%----------------------------------------------------------------

function b_diag = bdiag(block,P,varargin);

[L,K] = size(block);
b_diag = zeros(P*L,P*K); % initializing b_diag

if nargin > 3
error([ 'bdiag: ' int2str(nargin) ' args given to bdiag, only 2 or 3 reqd.']);
end

if nargin==2 | (nargin==3 & varargin{1}=='main'),
dtype='main';
elseif (nargin==3 & varargin{1}=='anti'),
dtype='anti';
end

if dtype=='main'
for i = 1:P,
b_diag([(i-1)*L+1:i*L],[1:P*K]) =...
[repmat(zeros(L,K),1,i-1), block, repmat(zeros(L,K),1,P-i) ];
end
elseif dtype=='anti'
for i = 1:P,
b_diag([(i-1)*L+1:i*L],[1:P*K]) =...
[repmat(zeros(L,K),1,P-i), block, repmat(zeros(L,K),1,i-1)];
end
end

return

