function [N, NNstar, M, MMstar, NMI, R]=calcflippingratios(genpar,as,bs,cs,aa,bb,cc,ion_cell,spgsym_cell,spgtrans_cell,p,singlecrys)
%
% function calculates flipping ratio
% for a list of HKL
%

%--- hkl-list
nr_hkl=singlecrys{2}+2;
hkl_crys_AA=[[singlecrys{1,3:nr_hkl}].hhkl_AA];
hkl_crys_AA_meas=reshape(hkl_crys_AA,3,columns(hkl_crys_AA)/3);

%---- nuclear structure factor
[N]=nucfac(as,bs,cs,aa,bb,cc,ion_cell,spgsym_cell,spgtrans_cell,hkl_crys_AA_meas');

%---- mag. inter. vec., |F|^2 and magnetic interaction vector
%---- in cartesian coordinates

%---- cell information
lat_cell.ll=[as,bs,cs]';
lat_cell.angl=[aa,bb,cc]';

%---- propagation vector is (0,0,0)
propvec_cell{1,1} = 1;
propvec_cell{1,2}.label = 'k01';
propvec_cell{1,2}.kx = 0;
propvec_cell{1,2}.ky = 0;
propvec_cell{1,2}.kz = 0;
flag_k0=1;

%---- magnetic moment along z
moment_cell{1,1}=0;  %--- set value later
for i=2:ion_cell{1,1}+1
   if (ion_cell{1,i+1}.sx != 0 || ion_cell{1,i+1}.sy || ion_cell{1,i+1}.sz)
      moment_cell{1,i}.fq=['MF_',ion_cell{1,i+1}.ion];
      moment_cell{1,i}.ion=ion_cell{1,i+1}.ion;
      moment_cell{1,i}.mx=0;
      moment_cell{1,i}.my=0;
      moment_cell{1,i}.mz=ion_cell{1,i+1}.sz;
      moment_cell{1,i}.imx=0;
      moment_cell{1,i}.imy=0;
      moment_cell{1,i}.imz=0;
      moment_cell{1,i}.phase=0;
      moment_cell{1,i}.pmx=0;
      moment_cell{1,i}.pmy=0;
      moment_cell{1,i}.pmz=0;
      moment_cell{1,i}.pimx=0;
      moment_cell{1,i}.pimy=0;
      moment_cell{1,i}.pimz=0;
      moment_cell{1,i}.pphase=0;
   else
      %--- remove non magnetic ions
      ion_cell{1,i+1}=[];   
   endif
endfor

%--- remove empty elements from cell
moment_cell=moment_cell(~cellfun('isempty',moment_cell));
moment_cell{1,1}=columns(moment_cell)-1;
moment_cell{1,columns(moment_cell)+1}.type=0;

ion_cell{1,2}=[];
ion_cell=ion_cell(~cellfun('isempty', ion_cell));
ion_cell{1,1}=columns(ion_cell)-2;

%--- in mufit symmetry operations have to be given expliclitely
%--- for each magnetic moment
%--- hence it is necessary to augment the number of symmetry operations
%--- if there is more than one magnetic ion the cell
nsym=spgsym_cell{1,1};
tmp=spgsym_cell(1,[2:end]);
if ion_cell{1} > 1
    for i=1:ion_cell{1}-1;
        tmp=[tmp tmp];
    endfor
endif

spgsym_cell={};
spgsym_cell{1}=nsym;
spgsym_cell=[spgsym_cell tmp];

%--- calculate magnetic interaction factor
[M_cell,spins_cart_cell]=moment(lat_cell,ion_cell,moment_cell,propvec_cell,spgsym_cell,spgtrans_cell);
%--- no magnetic domain with field along z
domains_cell{1,1}=1;
domains_cell{1,2}.rot=[1 0 0; 0 1 0; 0 0 1];
domains_cell{1,2}.pop=1;
domains_cell{1,2}.det=1;
[QQ,fstar,formfactor,popul,llor]=hklprep(as,bs,cs,aa,bb,cc,genpar.lambda,singlecrys,domains_cell,spins_cart_cell,propvec_cell);
[Mperpfsqr,fsqr,mag_vec_cart]=magfac2(spins_cart_cell,fstar,QQ,flag_k0,formfactor);

NNstar=zeros(singlecrys{1,2},1);
M=zeros(singlecrys{1,2},1);
MMstar=zeros(singlecrys{1,2},1);
NMI=zeros(singlecrys{1,2},1);
R=zeros(singlecrys{1,2},1);

for i = 1:singlecrys{1,2}
   %--- correction for out-of-plane reflections
   %--- normal beam geometry
   polf=sind(90-singlecrys{1,i+2}.nu);
   polfsqr=polf^2;

   %--- calculation of flipping ratios
   %--- 1.- nuclear part
   NNstar(i)=N(i).*conj(N(i));
   %--- 2.- magnetic part
   M(i)=mag_vec_cart(i,3);
   Mstar=conj(M(i));
   MMstar(i)=M(i)*Mstar;
   %---  M_perp*M_perp^* = MM^*sin(alfa)^2
   MMstar(i)=MMstar(i).*polfsqr;
   %--- 3.- nuclear-magnetic interaction part
   NMI(i)=N(i)*conj(M(i))+conj(N(i))*M(i);
   NMI(i)=NMI(i).*polfsqr; %--- correction for out-of-plane scattering angle
   NMI(i)=p.*NMI(i);  %--- correct for finite polarisation (n{^+}-n{^-})/(n{^+}+n{^-})

   %--- Flipping ratio R
   R(i)=(NNstar(i).+MMstar(i).+NMI(i))./(NNstar(i).+MMstar(i).-NMI(i));
endfor
endfunction
