## Copyright (C) 2007,2008,2009 Bertrand Roessli <bertrand.roessli@psi.ch>
##
##
## MuFit is free software; you can redistribute it and/or
## modify it under the terms of the GNU General Public
## License as published by the Free Software Foundation;
## either version 2, or (at your option) any later version.
##
## Mufit is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied
## warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
## PURPOSE.  See the GNU General Public License for more
## details.
##
## You should have received a copy of the GNU General Public
## License along with Octave; see the file COPYING.  If not,
## write to the Free Software Foundation, Inc., 51 Franklin Street,
## Fifth Floor, Boston, MA 02110-1301, USA.
##
## usage: mufit
##
## Author: Bertrand Roessli <bertrand.roessli@psi.ch>

## -*- texinfo -*-
## @deftypefn{Function File}  chi (@var{p},@var{varargin})
## @deftypefnx{Function File} {[@var{chisq},@var{f},@var{p},@var{R_Bragg}]}= chi (@var{p},@var{dp},@var{varargin})
## Calculate ChiSquare and R_Bragg;
## @var{p}: vector containing the parameters;
## @var{dp}: vector containg the parameter steps;
## @var{varargin} is a cell array containing additional variables necessary
## to calculate the function defined in @var{varargin(1)};
## the function returns ChiSquare, the function value, the parameters and
## R_Bragg
## @end deftypefn


function [chisq,fsave,p,R_Bragg,R_w,R_exp]=chi(p,dp,varargin)

func=varargin{1};
y=varargin{2};
wt=varargin{3};
[f,p]=feval(func,p,varargin{4:length(varargin)});

y=y(:);
wt=wt(:);
%if the errors are zeros remove from list
nzerr = find(finite(wt));
fsave=f(:);
%---- ChiSq
y = y(nzerr);
wt = wt(nzerr);
f = fsave(nzerr);
v=length(y)-length(find(dp));
chisq=sum(((y.-real(f)).*wt).^2 )/v;

%---- R_Bragg
R_Bragg=100.*sum(abs(y.-f))./sum(abs(y));

%---- R_weighted
R_w = sum(wt.^2.*abs(y.-f).^2./(wt.*y).^2);

%---- R_exp
R_exp = (rows (f) /sum( (wt.*y).^2))^0.5;

endfunction

