## Copyright (C) 2007,2008,2009 Bertrand Roessli <bertrand.roessli@psi.ch>
##
##
## MuFit is free software; you can redistribute it and/or
## modify it under the terms of the GNU General Public
## License as published by the Free Software Foundation;
## either version 2, or (at your option) any later version.
##
## Mufit is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied
## warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
## PURPOSE.  See the GNU General Public License for more
## details.
##
## You should have received a copy of the GNU General Public
## License along with Octave; see the file COPYING.  If not,
## write to the Free Software Foundation, Inc., 51 Franklin Street,
## Fifth Floor, Boston, MA 02110-1301, USA.
##

## -*- texinfo -*-
## @deftypefn{Function File}  readmu
## @deftypefnx{Function File}
## {[@genpar,@singlecrys, 
## @var{as}, @var{bs}, @var{cs}, @var{aa}, @var{bb}, @var{cc},
## @var{parameters},
## @var{dparameters},
## @var{flag_k0},
## @var{flag_nmi},
## @var{k0},
## @var{moment_type},
## @var{mfit_type},
## @var{simplexnuc_options},
## @var{lmfitnuc_options},
## @var{nsim_ann_par},
## @var{lat_cell},
## @var{ion_cell},
## @var{moment_cell},
## @var{propvec_cell_rlu},
## @var{propvec_cell_AA},
## @var{spgsym_cell},
## @var{spgtrans_cell},
## @var{domains_cell}]}]= readmu
## @* Read parameters from file 'crys.inp' fir magnetic fit (see
## ./examples/crys.inp)
##
## @*output:
##        @*1.-: @var{genpar}:lambda, extinction, scale factor
##        @*2.-: @var{singlecrys}: flag Intensities (0) / structure factor^2 (1)
##        @*3.-: @var{as},@var{bs},{...}, @var{cc}: cell parameters
##        @*4.-: @var{parameters}: parameters
##        @*5.-: @var{dparameters}: parameter steps
##        @*6.-: @var{flag_k0}: flag k_0=-k_0
##        @*7.-: @var{flag_nmi}: flag nuclear-magnetic interaction vector
##        @*8.-: @var{k0}: propagation vector (rlu)
##        @*9.-: @var{moment_type}: switch polar/ cartesian coordinate
##        @*10.-: @var{mfit_type}: switch to select fit algorithm (0/1/2)
##        @*11.-: @var{simplexnuc_options}: simplex options
##        @*12.-: @var{lmfitnuc_options}: levenberg-marquard options
##        @*13.-: @var{nsim_ann_par}: simulated annealing options  
##        @*14.-: @var{lat_cell}: lattice parameters
##        @*15.-: @var{ion_cell}: atomic positions
##        @*16.-: @var{moment_cell}: magnetic model
##        @*17.-: @var{propvec_cell_rlu}: propagation vector in rlu
##        @*18.-: @var{propvec_cell_AA}: propagation vector in AA^-1
##        @*19.-: @var{spgsym_cell}: symmetry operations
##        @*20.-: @var{spgtrans_cell}: translations (used to propagate spins)
##        @*21.-: @var{domains_cell}: symmetry operations for magnetic domains 
##        @*22.-: @var{draw_mag_opt}: drawing options  

%last modified 27.11.09


function [genpar,singlecrys,...
          as,bs,cs,aa,bb,cc,...
          parameters,dparameters,...
          flag_k0,flag_nmi,k0,moment_type,...
          fit_type,simplex_options,lmfit_options,sim_ann_par,...
          lat_cell,ion_cell,moment_cell,propvec_cell_rlu,propvec_cell_AA,spgsym_cell,spgtrans_cell,domains_cell,...
          draw_mag_opt]=...
          crys_inp


%---- import parameters from file
[fid,msg]=fopen('crys.inp',"rt");
if (fid == -1) fprintf(stdout,'--->crys.inp not found: %s\n',msg); endif

%---- lambda and extinction 
[dummy]=fgetl(fid);
[lambda,zach,scale,count]=fscanf(fid,'%f %f %s\n',"C");
genpar.lambda=lambda;
genpar.zach=zach;
genpar.scale=scale;

%---- flag Int (0)/ fsqr (1) 
[dummy]=fgetl(fid);
[intfsqr,count]=fscanf(fid,'%f\n',"C");
singlecrys{1}.intfsqr=intfsqr;

%---- title line
dummy=fgetl(fid);

%---- lattice parameters and angles
dummy='!';
while (dummy(1) == '!')
		[dummy]=fgetl(fid);
endwhile
[as,bs,cs,aa,bb,cc,count]=sscanf(dummy,'%f %f %f %f %f %f',"C");
if (count != 6)
	cmd = sprintf('zenity --error --text="Wrong input for lattice parameters"');  
        [status, output] = system(cmd);
	system('vi crys.inp +/lattice');
	error('wrong input for lattice\n');
endif

[Mcryst2cart,Mrec2cart]=lat2cart(as,bs,cs,aa,bb,cc);

%---- number atoms

dummy='!';
while (dummy(1) == '!')
		[dummy]=fgetl(fid);
endwhile
[nr_at, count]=sscanf(dummy,'%f\n',"C");
if (count != 1)
        cmd = sprintf('zenity --error --text="Wrong input for number of atoms"');
        [status, output] = system(cmd);
        system('vi crys.inp +/atoms');
	error('wrong input for number of atoms');
endif

ion_cell{1}=nr_at;

%---- ion type, position, B-factor and occupation
[dummy]=fgetl(fid);
for i=1:nr_at
	text=fgetl(fid);
	[ion , x, y, z, B, occ, radius, red, green, blue,  count]=sscanf(text,'%s %f %f %f %f %f %f %f %f %f \n',"C");
	if (count != 10)
		cmd = sprintf('zenity --info --text="entry should be: AT   x         y        z        B   occ"');  
                [status, output] = system(cmd);
        	system('vi crys.inp +/AT');
		error("please restart!\n");
	endif
	atpos=[x; y; z]; %atomic positions in rlu
	at.ion=ion;at.x=atpos(1);at.y=atpos(2);at.z=atpos(3);at.B=B;at.occ=occ;
	at.radius=radius;
	at.red=red; at.blue=blue;at.green=green;	
	ion_cell{i+1}=at;
endfor

%---- put lattice also in ion_cell
lattice.ll=[as;bs;cs];
lattice.angl=[aa;bb;cc];
lat_cell=lattice;

%---- read switch for input-type of magnetic moments
[dummy]=fgetl(fid);
[moment_type, count]=fscanf(fid,'%f\n',"C");
if (count != 1)
                cmd = sprintf('zenity --error --text="Wrong input for switch between polar and cartesian coordinates"');
                [status, output] = system(cmd);
		error("\nplease restart\n");
endif
if (moment_type ~=0 && moment_type ~=1)
                cmd = sprintf('zenity --info --text="entry should be:0/1 for cartesian/polar coordinates"');
                [status, output] = system(cmd);
                system('vi crys.inp +/switch');
                error("\nplease restart mufit!\n");

endif
switch moment_type

	case 0
		%read wave-functions for magnetic moments
		[dummy]=fgetl(fid);
		[nr_moment, count]=fscanf(fid,'%f\n',"C");
		if (count != 1)
                	cmd = sprintf('zenity --info --text="entry should be: # of magnetic ions"');
                	[status, output] = system(cmd);
                	system('vi crys.inp +/2*pi');
			error("\nit is necessary to restart mufit\n");
		endif
		moment_cell{1}=nr_moment;
		for i=1:nr_moment

%----- read magnetic moments
			text=fgetl(fid);
			[ion,fq,mx,my,mz,imx,imy,imz,phi,count]=sscanf(text,'%s %s %f %f %f %f %f %f %f',"C");
			if (count != 9)
 		        	cmd = sprintf('zenity --info --text="entry: Label MF_Label  RMx RMy Rmz  Imx Imy Imz Phase"');
                		[status, output] = system(cmd);
                		system('vi crys.inp +/2*pi');
                		error("\nplease restart!\n");
			endif
			mom.fq=fq;
			mom.ion=ion;

%----- will apply transformation to cartesian coordinates later in function moment.m
			mom.mx=mx;mom.my=my;mom.mz=mz;
			mom.imx=imx;mom.imy=imy;mom.imz=imz;
			mom.phase=phi;
%----- !read parameters
			text=fgetl(fid);
			[pmx,pmy,pmz,pimx,pimy,pimz,pphi,count]=sscanf(text,'%s %s %s %s %s %s %s',"C");
			if (count != 7)
				disp(text);
				error('wrong input in parameters line');
			endif
			mom.pmx=pmx;mom.pmy=pmy;mom.pmz=pmz;
			mom.pimx=pimx;mom.pimy=pimy;mom.pimz=pimz;
			mom.pphase=pphi;
			moment_cell{i+1}=mom;
		endfor
	case 1
		%read wave-functions for magnetic moments
		[dummy]=fgetl(fid);
		[nr_moment, count]=fscanf(fid,'%f\n',"C");
		if (count != 1)
			error('wrong input');
		endif
		moment_cell{1}=nr_moment;
		for i=1:nr_moment
			text=fgetl(fid);
			[ion,fq,rm,rphi,rtheta,im,iphi,itheta,phi,count]=sscanf(text,'%s %s %f %f %f %f %f %f %f',"C");
			if (count != 9)
				disp(text);
				error('wrong input');
			endif
			mom.fq=fq;
			mom.ion=ion;
			mom.rm=rm;
			mom.rphi=rphi;
			mom.rtheta=rtheta;
			mom.im=im;
			mom.iphi=iphi;
			mom.itheta=itheta;
			mom.phase=phi;
%---- !read parameters
			text=fgetl(fid);
			[prm,prphi,prtheta,pim,piphi,pitheta,pphi,count]=sscanf(text,'%s %s %s %s %s %s %s',"C");
			if (count != 7)
				disp(text);
				error('wrong input');
			endif
			mom.prm=prm;
			mom.prphi=prphi;
			mom.prtheta=prtheta;
			mom.pim=pim;
			mom.piphi=piphi;
			mom.pitheta=pitheta;
			mom.pphase=pphi;
			moment_cell{i+1}=mom;
		endfor
endswitch
moment_cell{i+2}.type=moment_type;

%---- number of symmetry operations
[dummy]=fgetl(fid);
[nr_sym, count]=fscanf(fid,'%f\n',"C");

if (count != 1)
 	cmd = sprintf('zenity --info --text="entry: should be the # of symmetry elements/spin"');
        [status, output] = system(cmd);
        system('vi crys.inp +/space');
        error("\nplease restart!\n");
endif

spgsym_cell{1}=nr_sym;

%----- symmetry operations
for i=1:nr_sym*nr_at
	text=fgetl(fid);
	[x, y, z,rmx,rmy,rmz,imx,imy,imz,symphase,count]=sscanf(text,'%s %s %s %s %s %s %s %s %s %f',"C");
	if (count != 10)
 		cmd = sprintf('zenity --info --text="input error at symmetry element %i"',i);
                [status, output] = system(cmd);
                system('vi crys.inp +/space');
                error("\nplease restart!\n");
	endif
	spg.x=x;spg.y=y;spg.z=z;
	spg.rmx=rmx;spg.rmy=rmy;spg.rmz=rmz;
	spg.imx=imx;spg.imy=imy;spg.imz=imz;
	spg.phase=symphase;
	spgsym_cell{i+1}=spg;	
endfor

%---- number of TRANSLATIONS
[dummy]=fgetl(fid);
[nr_transl, count]=fscanf(fid,'%f\n',"C");

if (count != 1)
	error('wrong input number of translations');
endif

spgtrans_cell{1}=nr_transl;

%----- symmetry operations
for i=1:nr_transl
	[x, y, z,count]=fscanf(fid,'%f %f %f',"C");
	if (count != 3)
		error('wrong input by the translations');
	endif
	spgtrans.x=x*as;spgtrans.y=y*bs;spgtrans.z=z*cs;  %translations in AA
	spgtrans_cell{i+1}=spgtrans;
endfor

%----- read propagation vector
[dumyy]=fgetl(fid);
[nr_propvec, count]=fscanf(fid,'%f\n',"C");
if (count != 1)
	error('something wrong with the propagaton vector (or close to)');
endif

propvec_cell_rlu{1}=nr_propvec;
propvec_cell_AA{1}=nr_propvec;
for i=1:nr_propvec
        text=fgetl(fid);
        [k0,kx,ky,kz,count]=sscanf(text,'%s %f %f %f',"C");
        prop_AA.label=k0;
        prop_rlu.label=k0;
        k0_rec_ang=Mrec2cart*[kx; ky; kz]; %propagation vector in reciprocal AA
        prop_AA.kx=k0_rec_ang(1);prop_AA.ky=k0_rec_ang(2);prop_AA.kz=k0_rec_ang(3);
        prop_rlu.kx=kx;prop_rlu.ky=ky;prop_rlu.kz=kz;
	propvec_cell_rlu{i+1}=prop_rlu;
	propvec_cell_AA{i+1}=prop_AA;
endfor


%---- read domain populations
[dummy]=fgetl(fid);
[domains,count]=fscanf(fid,'%f\n',"C");
nr_domains=domains;

%----- read number of domains and rotation angles/direction
[dummy]=fgetl(fid);
domains_cell{1}=nr_domains;

for i=1:nr_domains
	[domains_pop,raa,rab,rac,rba,rbb,rbc,rca,rcb,rcc,td,count]=fscanf(fid,'%f %f %f %f %f %f %f %f %f %f %s',"C");
	if (count != 11)
		error('wrong input at domain populations/matrices/type');
	endif
	dom.rot=[raa rab rac; rba rbb rbc; rca rcb rcc]';
	dom.pop=domains_pop;
        dom.det=det(dom.rot);
        dom.type=td;
 	n=sprintf(num2str(i));
	dom.par_pop=['D',n];
	domains_cell{i+1}=dom;
endfor

%---- read 'draw structure' options
[dummy]=fgetl(fid);
[dummy]=fgetl(fid);
[dummy]=fgetl(fid);
nr_draw_at=fscanf(fid,"%f\n","C");
i=0;
for i=1:nr_draw_at
        text=fgetl(fid);
        [label1, label2, mindist, maxdist, r, g, b, rad,counts]=...
        sscanf(text,'%s %s %f %f %f %f %f %f \n',"C");
        if (counts != 8)
                error('vrml-bond parameters != 8');
        endif   
        draw_mag_opt{1,i}.label=strcat(label1,'-',label2);
        draw_mag_opt{1,i}.mindist=mindist;
        draw_mag_opt{1,i}.maxdist=maxdist;
        draw_mag_opt{1,i}.color=[r g b];
        draw_mag_opt{1,i}.radius=rad;
endfor

text=fgetl(fid);
[unitcell, counts]=sscanf(text,'%f \n',"C");
if (counts != 1)
        error('vrm-flag unitcell missing')
endif
draw_mag_opt{1,i+1}.unitcell=unitcell;
draw_mag_opt{1,i+1}.style=0;  %--- draw balls only for magnetic structure

%---- read fit parameters
parameters=[];
dparameters=[];
nr_param=0;
for k=1:3
	[dummy]=fgetl(fid);
	[par,count]=fscanf(fid,'%f');
	if (count != 10)
		disp(par);
		disp(count);
		error('number of parameters per line not 10');	
	endif
	parameters=[parameters;par];
	nr_param=nr_param+count;
	%----- read parameters errors
	[dummy,counts]=fgetl(fid);
%	if (count != 10)
%		error('number of dpar per line not 10');	
%	endif
	[dpar,count]=fscanf(fid,'%f');
	dparameters=[dparameters;dpar];
endfor

%-----  read values and steps for domains populations 
[dummy]=fgetl(fid);
[par,count]=fscanf(fid,'%f');
parameters=[parameters;par];
[dummy]=fgetl(fid);
[dpar]=fscanf(fid,'%f');
dparameters=[dparameters;dpar];

%---- switch for fit alogorithm
[dummy]=fgetl(fid);
[fit_type, count]=fscanf(fid,'%f\n',"C");
if (count != 1)
	error('input should be fit algorithm');
endif

%-----  read levenberg-marquard fit options tol, niter, etc...
[dummy]=fgetl(fid);
[stol,niter, verbose, count]=fscanf(fid,'%f %f %f\n',"C");
if (count != 3) 
        error('input should be niter and stol');
endif
lmfit_options.verbose=verbose;
lmfit_options.stol=stol;
lmfit_options.niter=niter;

%-----  read simplex fit options tol, niter, etc...
[dummy]=fgetl(fid);
[stol,niter, fval, simplex_sides, verbose, count]=fscanf(fid,'%f %f %f %f %f\n',"C");
if (count != 5) 
        error('input should be stol, niter, fval, simplex sides, verbose');
endif
simplex_options.stol=stol;
simplex_options.niter=niter;
simplex_options.fval=fval;
simplex_options.sides=simplex_sides;
simplex_options.verbose=verbose;


% read simulated annealing options
#{
        * LB  - vector of lower bounds
        * UB - vector of upper bounds 
        * nt - integer: # of iterations between temperature reductions
        * ns - integer: # of iterations between bounds adjustments
        * rt - (0 < rt <1): temperature reduction factor
        * maxevals - integer: limit on function evaluations
        * neps - integer:  number of values final result is compared to
        * functol -   (> 0): the required tolerance level for function value comparisons
        * paramtol -  (> 0): the required tolerance level for parameters
        * verbosity - scalar: 0, 1, or 2.
                * 0 = no screen output   
                * 1 = only final results to screen
                * 2 = summary every temperature change
#}
for i=1:9
[dummy]=fgetl(fid);
endfor

[nt,ns,rt,maxevals,neps,functol,paramtol,verbosity,counts]=fscanf(fid,'%f %f %f %f %f %f %f %f\n',"C");
if (counts != 8)
	error('number of simulated annealing options wrong')
endif
sim_ann_par.nt=nt;
sim_ann_par.ns=ns;
sim_ann_par.rt=rt;
sim_ann_par.maexevals=maxevals;
sim_ann_par.neps=neps;
sim_ann_par.functol=functol;
sim_ann_par.paramtol=paramtol;
sim_ann_par.verbosity=verbosity;

%---- read switch for calculation of magnetic interaction vector for k0 and -k0 (F=1) and case of pure magnetic reflection (F=0)
[dummy,counts]=fgetl(fid);
[flag_k0,flag_nmi,counts]=fscanf(fid,'%f %f',"C");

%---- loop over HKL until end of file is reached
singlecrys{2}=0;
nr=3;
[dummy]=fgetl(fid);

while (1)
        if ((test=feof(fid))==1) %leaves when end-of-file reached
                fclose(fid);
                break;
        endif

        text=fgetl(fid);
        [ptype,H,K,L,f2obs,f2sd,count]=sscanf(text,'%s %f %f %f %f %f\n',"C");
        %---- Q-vector
        hhkl=(Mrec2cart*[H; K; L])';
	singlecrys{nr}.type=ptype;
	singlecrys{nr}.hhkl_rlu=[H K L];
	singlecrys{nr}.hhkl_AA=hhkl;
	singlecrys{nr}.f2obs=f2obs;
	singlecrys{nr}.f2sd=f2sd;	
        nr++;
endwhile
singlecrys{2}=nr-3;

endfunction
