## Copyright (C) 2007,2008,2009 Bertrand Roessli <bertrand.roessli@psi.ch>
##
##
## MuFit is free software; you can redistribute it and/or
## modify it under the terms of the GNU General Public
## License as published by the Free Software Foundation;
## either version 2, or (at your option) any later version.
##
## Mufit is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied
## warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
## PURPOSE.  See the GNU General Public License for more
## details.
##
## You should have received a copy of the GNU General Public
## License along with Octave; see the file COPYING.  If not,
## write to the Free Software Foundation, Inc., 51 Franklin Street,
## Fifth Floor, Boston, MA 02110-1301, USA.
##

## -*- texinfo -*-
## @deftypefn{Function File}  crys_int_fit2 (@var{p},@var{varargin})
## @deftypefnx{Function File} {[@var{I_crys},@var{param}]}= crys_int_fit2 (@var{p},@var{varargin})
## Calculate single crystal intensitites or structure factors using
## model parameters stored in array p and return intensity list and
## parameters
## @end deftypefn


function [I_crys,param]=crys_int_fit2(p,varargin)
%---- extract parameters
param=p;
hkl=varargin{1};
QQ=varargin{2};
lat_cell=varargin{3};
ion_cell=varargin{4};
spgsym_cell=varargin{5};
spgtrans_cell=varargin{6};
moment_cell=varargin{7};
propvec_cell_AA=varargin{8};
fstar=varargin{9};
flag_k0=varargin{10};
formfactor=varargin{11};
genpar=varargin{12};
intfsqr=varargin{13};
llor=varargin{14};
domains_cell=varargin{15};


lambda=genpar.lambda;
zach=genpar.zach;

%----- domain population
D1=abs(param(31));
D2=abs(param(32));
D3=abs(param(33));
D4=abs(param(34));
D5=abs(param(35));
D6=abs(param(36));
D7=abs(param(37));
D8=abs(param(38));
D9=abs(param(39));
D10=abs(param(40));

sum_dom=D1+D2+D3+D4+D5+D6+D7+D8+D9+D10;
D1=D1/sum_dom; 
D2=D2/sum_dom; 
D3=D3/sum_dom; 
D4=D4/sum_dom; 
D5=D5/sum_dom;
D6=D6/sum_dom;
D7=D7/sum_dom;
D8=D8/sum_dom;
D9=D9/sum_dom;
D10=D10/sum_dom;

param(31)=D1; 
param(32)=D2; 
param(33)=D3; 
param(34)=D4; 
param(35)=D5;
param(36)=D6;
param(37)=D7;
param(38)=D8; 
param(39)=D9;
param(40)=D10;

%----- get the list of magnetic moments in the unit cell in cartesian coordinates
[spins_cart_cell]=moment_fit(lat_cell,ion_cell,spgsym_cell,spgtrans_cell,propvec_cell_AA,moment_cell,param);

%----- get magnetic structure factor for the hkl-list with domains
[mag_inter_vec,fsqr,mag_vec_cart]=magfac2(spins_cart_cell,fstar,QQ,flag_k0,formfactor);

%----- calculate extinction
[extinc]=extinc(fsqr',genpar.lambda,genpar.zach,QQ);

%----- evaluate domain population
for k=1:domains_cell{1}
	popul_inp(k)=eval(domains_cell{k+1}.par_pop);
endfor
popul=repmat(popul_inp,1,rows(QQ)/domains_cell{1});

%----- magnetic intensity for each hkl
if (intfsqr == 0)                                 %---- true if integrated intensities not corrected for Lorentz factor
	I_hkl=popul.*fsqr'.*llor.*extinc;
else   
	I_hkl=popul.*fsqr'.*extinc;
endif

%----- reshape to sum over the magnetic domains
I_hkl_dom=reshape(I_hkl,domains_cell{1},columns(I_hkl)/domains_cell{1})';

%----- calculate magnetic intensities averaged over the domains
I_crys=sum(I_hkl_dom,2);

%----- scale factor
I_crys=eval(genpar.scale).*I_crys;

endfunction
