## Copyright (C) 2007,2008,2009 Bertrand Roessli <bertrand.roessli@psi.ch>
##
##
## MuFit is free software; you can redistribute it and/or
## modify it under the terms of the GNU General Public
## License as published by the Free Software Foundation;
## either version 2, or (at your option) any later version.
##
## Mufit is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied
## warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
## PURPOSE.  See the GNU General Public License for more
## details.
##
## You should have received a copy of the GNU General Public
## License along with Octave; see the file COPYING.  If not,
## write to the Free Software Foundation, Inc., 51 Franklin Street,
## Fifth Floor, Boston, MA 02110-1301, USA.
##

## -*- texinfo -*-
## @deftypefn{Function File}  extinc (@var{fsqr},@var{lambda},@var{c},@var{hkl_AA})
## @deftypefnx{Function File} {[@var{zacor}]= extinc (@var{fsqr},@var{lambda},@var{c},@var{hkl_AA})
## Calculate extinction based on Zachariasen formula like Shelx
## @*input:
## @*@var{fsqr}: structure factor^2
## @*@var{lambda}: wavelength
## @*@var{c}: scale factor
## @*@var{hkl_AA}: HKL in A
## @*output:
## @*@var{zacor}: extinction correction (primary extinction coefficient)
## @end deftypefn


function [zachcor]=extinc(fsqr,lambda,c,hkl_AA)
%---- primary extinction
%---- Acta Crys. 23, 558 (1967)
%---- International Tables for Crystallography Vol. C, 609-616 
%---- (T.Sabine)
%---- valid for small extinction
d_sp=2.*pi./dot(hkl_AA',hkl_AA').^0.5;
ttheta=2.*asin(lambda./2./d_sp);
x=c.*fsqr.*lambda^3./sin(ttheta);
zachcor=(1+0.001*x).^-0.5;   %factor 0.001 mimics shelx
#{
%---- anisotropic form 
%---- y=[1+cF^2lambda^3/(4*dspacing^2(sin(2theta))*(e1*h^2+e2^k^2+e3*l^3+e4*hk+e5*hl+e6*kl)]^-0.5
#}

endfunction
