## Copyright (C) 2007,2008,2009 Bertrand Roessli <bertrand.roessli@psi.ch>
##
##
## MuFit is free software; you can redistribute it and/or
## modify it under the terms of the GNU General Public
## License as published by the Free Software Foundation;
## either version 2, or (at your option) any later version.
##
## Mufit is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied
## warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
## PURPOSE.  See the GNU General Public License for more
## details.
##
## You should have received a copy of the GNU General Public
## License along with Octave; see the file COPYING.  If not,
## write to the Free Software Foundation, Inc., 51 Franklin Street,
## Fifth Floor, Boston, MA 02110-1301, USA.
##
## usage: mufit
##

## -*- texinfo -*-
## @deftypefn{Function File}  formfac (@var{Q},@var{at_n})
## @deftypefnx{Function File} {[@var{j_0}}= formfac (@var{Q},@var{at_n})
## @*Calculate magnetic form factor using the parameters tabulated in the
## International Tables by P.J. Brown
## @*The function does not contain tabulated parameters for only few ions at moment. 
## The list will be will be augmented with time.
##
## @*input:
## @*@var{Q}: scattering vector
## @*@var{at_n}: atom label from file mag.inp 
## @*output:
## @*var{j_0}: magnetic form factor at @var{Q}
## @end deftypefn

function [j_0]=formfac(Q,at_n);

%---- function calculates the neutron form factor
%---- input: Q vector and ion lablel
%---- output: form factor according to Jane Brown in International Tables
%---- broessli@psi.ch, 16.02.2007

fm_at=[];
%---- coefficients of the form factor
%---- other elements should be added to the list when required
MF_V0=[0.4086 28.8109 0.6077 8.5437 -0.0295 0.2768 0.0123];
MF_V1=[0.4444 32.6479 0.5683 9.0971 -0.2285 0.0218 0.2150]; 
MF_V2=[0.4085 23.8526 0.6091 8.2456 -0.1676 0.0415 0.1496]; 
MF_V3=[0.3598 19.3364 0.6632 7.6172 -0.3064 0.0296 0.2835]; 
MF_V4=[0.3106 16.8160 0.7198 7.0487 -0.0521 0.3020 0.0221]; 

MF_Er3= [  0.0389   5.3118   0.2598   8.1732   0.6784   2.0828   0.0222 ];%er3+
MF_Fe3= [  0.3972  13.2442   0.6295   4.9034  -0.0314   0.3496   0.0044 ];%fe3+
MF_Mn2= [  0.4220   17.684   0.5984   6.0050   0.0043  -0.6090  -0.0219 ];%mn2+
MF_Mn4= [  0.3760  12.5661   0.6602   5.1329  -0.0372   0.5630   0.0011 ];%mn3+
MF_Mn3= [  0.4198  14.2829   0.6054   5.4689   0.9241  -0.0088  -0.9498 ];%mn4+
MF_Nd3= [  0.0733   4.4124   0.3715   4.0196   0.5395   1.5580   0.0173 ];%nd3+
MF_Co3= [  0.3902  12.5078   0.6324   4.4574  -0.1500   0.0343   0.1272 ];%Co3+
MF_Cu2= [  0.0232  34.9686   0.4023  11.5640   0.5882   3.8428  -0.0137 ];%Cu2+
MF_Tb3= [  0.0177  25.5095   0.2921  10.5769   0.7133   3.5122  -0.0231 ];%tb3+
MF_Yb3= [  0.0416  16.0949   0.2849   7.8341   0.6961   2.6725  -0.0229 ];%yb3+
fm_at=eval(eval("at_n"));
Q=Q/4/pi;
q2=Q*Q;
j_0=fm_at(1).*exp(-fm_at(2).*q2).+fm_at(3).*exp(-fm_at(4).*q2).+fm_at(5).*exp(-fm_at(6).*q2).+fm_at(7);
endfunction

