## Copyright (C) 2007,2008,2009 Bertrand Roessli <bertrand.roessli@psi.ch>
##
##
## MuFit is free software; you can redistribute it and/or
## modify it under the terms of the GNU General Public
## License as published by the Free Software Foundation;
## either version 2, or (at your option) any later version.
##
## Mufit is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied
## warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
## PURPOSE.  See the GNU General Public License for more
## details.
##
## You should have received a copy of the GNU General Public
## License along with Octave; see the file COPYING.  If not,
## write to the Free Software Foundation, Inc., 51 Franklin Street,
## Fifth Floor, Boston, MA 02110-1301, USA.
##
## usage: mufit

## -*- texinfo -*-
## @deftypefn{Function File} hklprep (@var{as},@var{bs},@var{cs},@var{aa},@var{bb},@var{cc},@var{lambda},@var{singlecrys},@var{domains_cell},@var{moment_cell},@var{propvec_cell_rlu})
## @deftypefnx{Function File}
## {[@var{QQ},@var{fstar},@var{forfmactor},@var{popul},@var{llor} = 
## hklprep (@var{as},@var{bs},@var{cs},@var{aa},@var{bb},@var{cc},@var{lambda},@var{singlecrys},@var{domains_cell},@var{moment_cell},@var{propvec_cell_rlu})
## @*Utility function that creates various lists like HKL in rlu or A to
## take into account the presence of the magnetic domains
## @*Called by mufit.m
## @end deftypefn


function [QQ,fstar,formfactor,popul,llor]=hklprep(as,bs,cs,aa,bb,cc,lambda,singlecrys,domains_cell,moment_cell,propvec_cell_rlu)

%----- initialise variables
I_hkl_av=zeros(1,singlecrys{2});
lor=zeros(1,singlecrys{2});

ll=1;
array_size=singlecrys{2}*domains_cell{1};
[QQ]=zeros(array_size,3);
[llor]=zeros(1,array_size);
[popul]=zeros(1,array_size);
[fstar]=zeros(1,array_size);

%-----
%----- create an array of all hkl's with domains
%----- create arrays like the form factor, the Lorentz factor, the domain population
for kk=3:singlecrys{2}+2
        Q=[[singlecrys{kk}].hhkl_AA];
        d_sp=2*pi/norm(Q);
        th_angle=asin(lambda./2./d_sp);
        %---- calculate form factor at Q
        [ffq]=zeros(1,moment_cell{1});
        for k=1:moment_cell{1} 
                fq=moment_cell{k+1}.fq;
                ffq(k)=formfac(norm(Q),fq); %compute form factor at Q for each spin
        endfor

        I_singcrys_dom=0;
        for ii=1:domains_cell{1}
                %if (domains_cell{ii+1}.pop > 0.01)
                        %-----rotation matrices for domains should be given in a cartesian coordinates in input file
			%---- P^-1 = P' for orthogonal matrices
			%---- not satisfied for some oblique crystal systems
			%---- higher than orthorhombic (International Tables
			%---- for Crystallography)
			transfo=transpose(inv(domains_cell{ii+1}.rot));
			hkl_dom(1:3)=transfo*(singlecrys{kk}.hhkl_AA)';
                        %Q-vector
                        H=hkl_dom(1);
                        K=hkl_dom(2);
                        L=hkl_dom(3);
                        %----- Q vector in cartesian coordinates and 1/A
                        QQ(ll,1:3)=[H K L];
                        %-----Lorentz factor for each hkl
                        llor(ll)=1./sin(2.*th_angle);
                        %-----domain population for each hkl
                        popul(ll)=domains_cell{ii+1}.pop;   
                        %----dtermine if + or - satellite for each hkl
                        tau_0=[propvec_cell_rlu{2}.kx; propvec_cell_rlu{2}.ky; propvec_cell_rlu{2}.kz];
                        P=domains_cell{ii+1}.rot*singlecrys{kk}.hhkl_rlu'.-tau_0;  %phi^*
                        fstar(ll)=1;
                        %-----check if P is an integer or not
                        if (abs(P(1)-round(P(1)))>1e-3) fstar(ll)=-1; endif;%phase
                        if (abs(P(2)-round(P(2)))>1e-3) fstar(ll)=-1; endif;
                        if (abs(P(3)-round(P(3)))>1e-3) fstar(ll)=-1; endif;

                        %----- magnetic form factor
                        formfactor(ll,1:moment_cell{1})=ffq;
                %endif
	                ll=ll+1;
        endfor
endfor

endfunction
