function [Mcryst2cart,Mrec2cart]=lat2cart(as,bs,cs,aa,bb,cc);

%input: lattice parameters (as,bs,cs) and angles (aa,bb,cc)
%output:
%Mcryst2cart: transformation matrix: lattice to cartesian coordinates
%Mrec2cart:   transformation matrix: reciprocal lattice to cartesion coordinates
%taken from MFIT
%last change: 16.06.07
%broessli@psi.ch
if (nargin != 6)
	usage ("[Mcryst2cart,Mrec2cart]=lat2cart(as,bs,cs,aa,bb,cc)");
endif


%transform angles to radians
alpha=pi*aa/180;
bet =pi*bb/180;
gam=pi*cc/180;

%----- Real space lattice vectors in cartesian coordinates
a_vec=as.*[1; 0; 0];
b_vec=bs.*[cos(gam); sin(gam); 0];
c1=cos(bet);
c2=(cos(alpha)-cos(gam)*cos(bet))/sin(gam);
c3=sqrt(1-c1^2-c2^2);
c_vec=cs*[c1; c2; c3;];

%-----normalised basis vectors in cartesian coordinates
a_vec_nor=a_vec/sqrt(dot(a_vec,a_vec));
b_vec_nor=b_vec/sqrt(dot(b_vec,b_vec));
c_vec_nor=c_vec/sqrt(dot(c_vec,c_vec));

%transformation matrix to cartesian coordinates
Mcryst2cart=[a_vec_nor, b_vec_nor, c_vec_nor];

%----- Reciprocal space lattice basis vectors
%volume of cell
V=dot(a_vec,cross(b_vec,c_vec));

a_star_vec=2*pi*cross(b_vec,c_vec)/V;
b_star_vec=2*pi*cross(c_vec,a_vec)/V;
c_star_vec=2*pi*cross(a_vec,b_vec)/V;

a_star=sqrt(sum(a_star_vec.*a_star_vec));
b_star=sqrt(sum(b_star_vec.*b_star_vec));
c_star=sqrt(sum(c_star_vec.*c_star_vec));

Mrec2cart=[a_star_vec b_star_vec c_star_vec];

endfunction
