## Copyright (C) 2007,2008,2009 Bertrand Roessli <bertrand.roessli@psi.ch>
##
##
## Octave is free software; you can redistribute it and/or
## modify it under the terms of the GNU General Public
## License as published by the Free Software Foundation;
## either version 2, or (at your option) any later version.
##
## Octave is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied
## warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
## PURPOSE.  See the GNU General Public License for more
## details.
##
## You should have received a copy of the GNU General Public
## License along with Octave; see the file COPYING.  If not,
## write to the Free Software Foundation, Inc., 51 Franklin Street,
## Fifth Floor, Boston, MA 02110-1301, USA.
##

## -*- texinfo -*-
## @deftypefn{Function File}  magfac
## (@var{spins_cell},@var{Q},@var{hhkl_rlu},@var{propvec_cell},@var{flag_k0})
## @deftypefnx{Function File}
## {[@var{mag_inter_vec},@var{fsq},@var{mag_vec_cart}]} = magfac
## (@var{spins_cell},@var{Q},@var{hhkl_rlu},@var{propvec_cell},@var{flag_k0})
## @*Calculate magnetic interaction vector and structure factor
##
## @*input:
##        @*1.- @var{spins_cell}: magnetic model
##        @*2.- @var{Q}: norm of scattering vector
##        @*3.- @var{hhkl_rlu}: [h,k,l]
##        @*4.- @var{propvec_cell}: propagation vector
##        @*5.- @var{flag_k0}: switch k_0 = -k_0 ?
##
## @*output:
##        @*1.- @var{mag_inter_vec}: magnetic interaction vector
##        @*2.- @var{fsq}: structure factor^2
##        @*3.- @var{mag_vec_cart}]: component of structure factor
## @end deftypefn

%%% OBSOLETE USE magfac2.m %%%
function [mag_inter_vec,fsqr,mag_vec_cart,fstar]=magfac(spins_cell,Q,hhkl_rlu,propvec_cell,flag_k0);

tau_0=[propvec_cell{2}.kx; propvec_cell{2}.ky; propvec_cell{2}.kz];
if (all(tau_0==0))
   flag_k0 = 1;
else
   P=hhkl_rlu'.-tau_0;  %phi^*
   fstar=1;

   %---- check if P is an integer or not
   if (abs(P(1)-round(P(1)))>1e-3) fstar=-1; endif; %phi
   if (abs(P(2)-round(P(2)))>1e-3) fstar=-1; endif;
   if (abs(P(3)-round(P(3)))>1e-3) fstar=-1; endif;
endif
magstrufac_x=zeros(spins_cell{1},1);
magstrufac_y=zeros(spins_cell{1},1);
magstrufac_z=zeros(spins_cell{1},1);

%NOT USED WILL BE REMOVED
chi=1;
%---- take the complex conjugate if Bragg comes from  satellite at g-tau 
%if (fstar == 1)  chi=-1; endif

%---- compute the fourier transform

switch flag_k0
case 0 %---- usual case

	nr_spins=spins_cell{1};
	A=[2:nr_spins+1];
	phase=fstar.*[[spins_cell{1,A}].phase]';
	p=[[spins_cell{1,A}].spins_p];
	spinsp=(reshape(p,3,columns(p)/3))';
	q=[[spins_cell{1,A}].spins_q];
	spinsq=(reshape(q,3,columns(p)/3))';
	atpos=[[spins_cell{1,A}].ionpar];
	ionpar=(reshape(atpos,5,columns(atpos)/5))';
	B=ionpar(:,4);
	occ=ionpar(:,5);
	ffq=zeros(nr_spins,1);
	for k=1:nr_spins
		fq=spins_cell{k+1}.fq;
		ffq(k)=formfac(norm(Q),fq); %compute form factor at Q for each spin
	endfor
	%fourier transform; occupation number; form factor; Debye-Waller factor
	QQ=dot(Q',Q')/(4*pi).^2;
	ftr=[exp(I.*ionpar(:,1:3)*Q').*occ.*ffq.*exp(-QQ.*B)];

	%add real and complex coefficient
	magstrufac_x=[(spinsp(:,1).*exp(-I.*2.*pi.*phase).+chi.*fstar.*I.*spinsq(:,1).*exp(-chi.*I.*2.*pi.*phase)).*ftr];
	magstrufac_y=[(spinsp(:,2).*exp(-I.*2.*pi.*phase).+chi.*fstar.*I.*spinsq(:,2).*exp(-chi.*I.*2.*pi.*phase)).*ftr];
	magstrufac_z=[(spinsp(:,3).*exp(-I.*2.*pi.*phase).+chi.*fstar.*I.*spinsq(:,3).*exp(-chi.*I.*2.*pi.*phase)).*ftr];

case 1 %---- if k0 and -k0 superimpose
	nr_spins=spins_cell{1};
	A=[2:nr_spins+1];
	phase=[[spins_cell{1,A}].phase]';
	p=[[spins_cell{1,A}].spins_p];
	spinsp=(reshape(p,3,columns(p)/3))';
	q=[[spins_cell{1,A}].spins_q];
	spinsq=(reshape(q,3,columns(p)/3))';
	atpos=[[spins_cell{1,A}].ionpar];
	ionpar=(reshape(atpos,5,columns(atpos)/5))';
	B=ionpar(:,4);
	occ=ionpar(:,5);
	ffq=zeros(nr_spins,1);
	for k=1:nr_spins
		fq=spins_cell{k+1}.fq;
		ffq(k)=formfac(norm(Q),fq);
	endfor
	QQ=dot(Q',Q')/(4*pi).^2;	
	ftr=[exp(I.*ionpar(:,1:3)*Q').*occ.*ffq.*exp(-QQ.*B)];
        magstrufac_x=2.*[(spinsp(:,1).*cos(2.*pi.*phase).+spinsq(:,1).*sin(2.*pi.*phase)).*ftr];
        magstrufac_y=2.*[(spinsp(:,2).*cos(2.*pi.*phase).+spinsq(:,2).*sin(2.*pi.*phase)).*ftr];
        magstrufac_z=2.*[(spinsp(:,3).*cos(2.*pi.*phase).+spinsq(:,3).*sin(2.*pi.*phase)).*ftr];	
endswitch
Smagstrufac_x=sum(magstrufac_x);
Smagstrufac_y=sum(magstrufac_y);
Smagstrufac_z=sum(magstrufac_z);

%------------------------------------------------------------------------------------------
%---- calculate magnetic interaction vector k x (M x k)
mag_vec_cart=0.2695.*[Smagstrufac_x; Smagstrufac_y; Smagstrufac_z];
normQ=dot(Q',Q').^0.5;
Qnorm=Q'./normQ;
mag_inter_vec=cross(Qnorm,cross(mag_vec_cart,Qnorm));

%---- calculate the square of the magnetic structure factor
fsqr=mag_inter_vec(1).*conj(mag_inter_vec(1)).+mag_inter_vec(2).*conj(mag_inter_vec(2)).+mag_inter_vec(3).*conj(mag_inter_vec(3));

endfunction

