## Copyright (C) 2007,2008,2009 Bertrand Roessli <bertrand.roessli@psi.ch>
##
##
## MuFit is free software; you can redistribute it and/or
## modify it under the terms of the GNU General Public
## License as published by the Free Software Foundation;
## either version 2, or (at your option) any later version.
##
## Mufit is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied
## warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
## PURPOSE.  See the GNU General Public License for more
## details.
##
## You should have received a copy of the GNU General Public
## License along with Octave; see the file COPYING.  If not,
## write to the Free Software Foundation, Inc., 51 Franklin Street,
## Fifth Floor, Boston, MA 02110-1301, USA.
##
## usage: mufit
##

## -*- texinfo -*-
## @deftypefn{Function File} moment_fit (@var{lat_cell},@var{ion_cell},@var{spgsym_cell},@var{spgtrans_cell},@var{propvec_cell_AA},@var{moment_cell},@var{param})
## @deftypefnx{Function File}
## {[@var{S_cart_sys_cell}]}= moment_fit (@var{lat_cell},@var{ion_cell},@var{spgsym_cell},@var{spgtrans_cell},@var{propvec_cell_AA},@var{moment_cell},@var{param})
## @*Calculate the magnetic moment configuration from the paramaeter of the magnetic model in mag.inp
## @*Used to fit magnetic intensities and polarimetry matrices
##
## @*input:
##      @*1.- the lattice parameters (@var{lat_cell})
##      @*2.- the atomic positions (@var{ion_cell})  
##      @*3.- the magnetic symmetry operations (@var{spgsym_cell})
##      @*4.- the list of translation (@var{spgtrans_cell})
##      @*5.- the propagation vector in rlu and AA^1 (@var{propvec_cell_AA})
##      @*6.- the magnetic model (@var{moment_cell}) 
##      @*7.- the parameter list
##
## @*output:
##      @*the magnetic moment directions/size calculated from the @var{param} list
## @end deftypefn


function [S_cart_sys_cell]=moment_fit(lat_cell,ion_cell,spgsym_cell,spgtrans_cell,propvec_cell_AA,moment_cell,param);

%---- parameters for magnetic moments
C1=param(1);
C2=param(2);
C3=param(3);
C4=param(4);
C5=param(5);
C6=param(6);
C7=param(7);
C8=param(8);
C9=param(9);
C10=param(10);
C11=param(11);
C12=param(12);
C13=param(13);
C14=param(14);
C15=param(15);
C16=param(16);
C17=param(17);
C18=param(18);
C19=param(19);
C20=param(20);
C21=param(21);
C22=param(22);
C23=param(23);
C24=param(24);
C25=param(25);
C26=param(26);
C27=param(27);
C28=param(28);
C29=param(29);
C30=param(30);

as=lat_cell.ll(1);
bs=lat_cell.ll(2);
cs=lat_cell.ll(3);
aa=lat_cell.angl(1);
bb=lat_cell.angl(2);
cc=lat_cell.angl(3);

[Mcryst2cart,Mrec2cart]=lat2cart(as,bs,cs,aa,bb,cc);

%---- 1) transform from polar coordinates to xyz coordinates if necessary
%---- note that polar coordinates should be defined in a cartesian coordinate system in the input file
moment_cart_cell=cell(moment_cell{1},1);
moment_cart_cell{1}=moment_cell{1};
if (moment_cell{columns(moment_cell)}.type  == 1)
        for i=1:moment_cell{1}
                mom.fq=moment_cell{i+1}.fq;
		PRM=eval(moment_cell{i+1}.prm);
		IRM=eval(moment_cell{i+1}.pim);
		prtheta=eval(moment_cell{i+1}.prtheta)/180*pi;
		pitheta=eval(moment_cell{i+1}.pitheta)/180*pi;
		prphi=eval(moment_cell{i+1}.prphi)/180*pi;
		piphi=eval(moment_cell{i+1}.piphi)/180*pi;
                prmz=PRM.*cos(prtheta);
                prmx=PRM.*sin(prtheta)*cos(prphi);
                prmy=PRM.*sin(prtheta)*sin(prphi);
                pimz=IRM.*cos(pitheta);
                pimx=IRM.*sin(pitheta).*cos(piphi);
                pimy=IRM.*sin(pitheta).*sin(piphi);
                %
                %transform to crystal system to apply later symmetry operations
                %
                prm_cart=Mcryst2cart^-1*[prmx; prmy; prmz];
                mom.pmx=prm_cart(1);mom.pmy=prm_cart(2);mom.pmz=prm_cart(3);
                pim_cart=Mcryst2cart^-1*[pimx; pimy; pimz];
                mom.pimx=pim_cart(1);mom.pimy=pim_cart(2);mom.pimz=pim_cart(3);
                mom.phase=eval(moment_cell{i+1}.pphase);
                moment_cart_cell{i+1}=mom;
        endfor
endif

mag_vec_cart=[0;0;0];
k0=[propvec_cell_AA{2}.kx propvec_cell_AA{2}.ky propvec_cell_AA{2}.kz];

%---- create list of ion/magnetic moment according to symmetry elements
l=1;

spins_cell=cell(spgsym_cell{1}*ion_cell{1}+1,1);
spins_cell{1}=ion_cell{1}*spgsym_cell{1};
atoms=zeros(spgsym_cell{1}*ion_cell{1},11);
while (l<=spgsym_cell{1}*ion_cell{1})
        for k=1:ion_cell{1}
                X=ion_cell{k+1}.x;
                Y=ion_cell{k+1}.y;
                Z=ion_cell{k+1}.z;
                for j=1:spgsym_cell{1}
%---- apply symmetry operations to atomic positions and transform to cartesian coordinates
                	atoms_pos(1:3)=Mcryst2cart*([eval(spgsym_cell{l+1}.x).*as ...
                                                     eval(spgsym_cell{l+1}.y).*bs ...
                                                     eval(spgsym_cell{l+1}.z).*cs])';
%---- apply symmetry operations to magnetic moments
			if (moment_cell{1,columns(moment_cell)}.type == 1)
	                        RMx=moment_cart_cell{k+1}.pmx;
	                        RMy=moment_cart_cell{k+1}.pmy;
	                        RMz=moment_cart_cell{k+1}.pmz; 
	                        IMx=moment_cart_cell{k+1}.pimx;
	                        IMy=moment_cart_cell{k+1}.pimy;
	                        IMz=moment_cart_cell{k+1}.pimz;
			else
				RMx=eval(moment_cell{k+1}.pmx);
				RMy=eval(moment_cell{k+1}.pmy);
				RMz=eval(moment_cell{k+1}.pmz);
				IMx=eval(moment_cell{k+1}.pimx);
				IMy=eval(moment_cell{k+1}.pimy);
				IMz=eval(moment_cell{k+1}.pimz);
                        	%RMx=str2num(moment_cell{k+1}.pmx)
                        	%RMy=str2num(moment_cell{k+1}.pmy);
                        	%RMz=str2num(moment_cell{k+1}.pmz); 
                        	%IMx=str2num(moment_cell{k+1}.pimx);
                        	%IMy=str2num(moment_cell{k+1}.pimy);
                        	%IMz=str2num(moment_cell{k+1}.pimz);
			endif
                        %%propagate the magnetic moment with the propagation vector and translation part of symmetry operations
                        a1=spgsym_cell{l+1}.x;a2=spgsym_cell{l+1}.y;a3=spgsym_cell{l+1}.z;
                        b1=a1(max([rindex(a1,"X"),rindex(a1,"Y"),rindex(a1,"Z")])+1:length(a1));
                        b2=a2(max([rindex(a2,"X"),rindex(a2,"Y"),rindex(a2,"Z")])+1:length(a2));
                        b3=a3(max([rindex(a3,"X"),rindex(a3,"Y"),rindex(a3,"Z")])+1:length(a3));
                        if (length(b1)==0)
                                b1='0';
                        endif
                        if (length(b2)==0)
                                b2='0';
                        endif
                        if (length(b3)==0)
                                b3='0';
                        endif
                        trans=exp(I*k0*Mcryst2cart*[eval(b1)*as;eval(b2)*bs;eval(b3)*cs]);
                        atoms(l,1:11)=[atoms_pos ion_cell{k+1}.B ion_cell{k+1}.occ...
                                eval(spgsym_cell{l+1}.rmx).*trans eval(spgsym_cell{l+1}.rmy).*trans eval(spgsym_cell{l+1}.rmz).*trans...
                                eval(spgsym_cell{l+1}.imx).*trans eval(spgsym_cell{l+1}.imy).*trans eval(spgsym_cell{l+1}.imz).*trans];
                        MM.ion=ion_cell{k+1}.ion; 
                        MM.fq=moment_cell{k+1}.fq;
                        MM.ionpar=atoms(l,1:5);
                        MM.spins_p=real(atoms(l,6:8))./2; 
                        MM.spins_q=real(atoms(l,9:11))./2;
                        %the phase is the sum of two components, nth(spgsym,l+1).phase is given by symmetry
			if (moment_cell{1,columns(moment_cell)}.type == 1)
	                        MM.phase=moment_cart_cell{k+1}.phase.+spgsym_cell{l+1}.phase;
			else
	                        MM.phase=eval(moment_cell{k+1}.pphase).+spgsym_cell{l+1}.phase;
			endif
                        spins_cell{l+1}=MM;
                        l++;
                endfor
        endfor
endwhile

%---- 3)transform spins to cartesian coordinates for calculation of structure factor
S_cart_cell=cell(1,spins_cell{1}+1);
S_cart_sys_cell{1}=spins_cell{1};
for k=1:spins_cell{1}
        p.fq=spins_cell{k+1}.fq;
        p.ion=spins_cell{k+1}.ion;
        p.ionpar=spins_cell{k+1}.ionpar;

        p1=spins_cell{k+1}.spins_p(1);p2=spins_cell{k+1}.spins_p(2);p3=spins_cell{k+1}.spins_p(3);
        q1=spins_cell{k+1}.spins_q(1);q2=spins_cell{k+1}.spins_q(2);q3=spins_cell{k+1}.spins_q(3);
        if (moment_cell{columns(moment_cell)}.type  == 1)
                %magnetic moments in polar coordinates are already defined in a cartesian system
                p.spins_p=[p1;p2;p3]';
                p.spins_q=[q1;q2;q3]';
        else
                %otherwise transform to cartesian system
                p.spins_p=(Mcryst2cart*[p1;p2;p3])';
                p.spins_q=(Mcryst2cart*[q1;q2;q3])';
        endif

        p.phase=spins_cell{k+1}.phase;
        S_cart_sys_cell{k+1}=p;
endfor

endfunction
