## Copyright (C) 2009 Bertrand Roessli <bertrand.roessli@psi.ch>
##
##
## MuFit is free software; you can redistribute it and/or
## modify it under the terms of the GNU General Public
## License as published by the Free Software Foundation;
## either version 2, or (at your option) any later version.
##
## Mufit is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied
## warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
## PURPOSE.  See the GNU General Public License for more
## details.
##
## You should have received a copy of the GNU General Public
## License along with Octave; see the file COPYING.  If not,
## write to the Free Software Foundation, Inc., 51 Franklin Street,
## Fifth Floor, Boston, MA 02110-1301, USA.
##
## usage: mufit   
##
## Author: Bertrand Roessli <bertrand.roessli@psi.ch>

## -*- texinfo -*-
## @deftypefn{Function File}  mt_atom_dist (@var{varargin})
## @deftypefnx{Function File} {[@var{atoms_cell},@var{atdists_cell}]}= mt_atom_dist (@var{varargin})
## Calculate atomic distances;
##
## @*input:
##       @*1.- @var{varargin(1)}: atomic parameters
##       @*2.- @var{varargin(2)}: translations
##       @*3.- @var{varargin(3)}: symmetry elements
##       @*4.- @var{varargin(4)}: drawing options
##
## @*output:
##       @*1.- @var{atoms_cell}: cell array with cell/atomic parameters (AA)
##       @*2.- @var{atdists_cell}: cell array with atomic distances (AA)
##
## @end deftypefn


function [atoms_cell,atdists]=mt_atom_dist(varargin)

ion_cell=varargin{1,1};
spgtrans_cell=varargin{1,2};
spgsym_cell=varargin{1,3};
draw_nuc_opt=varargin{1,4};

as=ion_cell{1,2}.as;
bs=ion_cell{1,2}.bs;
cs=ion_cell{1,2}.cs;
aa=ion_cell{1,2}.aa;
bb=ion_cell{1,2}.bb;
cc=ion_cell{1,2}.cc;

lat_cell=[as bs cs];


%---- loop over translations/symmetry elements of the space group to create
%---- ions in the unit cell
l=1;
for k=1:ion_cell{1}
  X=ion_cell{k+2}.x;
  Y=ion_cell{k+2}.y;
  Z=ion_cell{k+2}.z;
  for i=1:spgtrans_cell{1}
    for j=1:spgsym_cell{1}
      %---- create atomic positions in AA
      atoms_pos(1:3)=([eval(spgsym_cell{j+1}.x).*as ...
      eval(spgsym_cell{j+1}.y).*bs eval(spgsym_cell{j+1}.z).*cs].+...
      [spgtrans_cell{i+1}.x.*as spgtrans_cell{i+1}.y.*bs spgtrans_cell{i+1}.z.*cs])';
      %if (abs(atoms_pos(1)) <= as & abs(atoms_pos(2)) <= bs & abs(atoms_pos(3)) <= cs)
        if (draw_nuc_opt{1,end}.unitcell == 1) 
	%----- make coordinates >0 and <1 if .unitcell = 1           
          if (atoms_pos(1) > as) atoms_pos(1)-=as; endif
          if (atoms_pos(2) > bs) atoms_pos(2)-=bs; endif
          if (atoms_pos(3) > cs) atoms_pos(3)-=cs; endif

          if (atoms_pos(1) < 0) atoms_pos(1)+=as; endif
          if (atoms_pos(2) < 0) atoms_pos(2)+=bs; endif
          if (atoms_pos(3) < 0) atoms_pos(3)+=cs; endif

          if (atoms_pos(1) < -as) atoms_pos(1)+=as; endif
          if (atoms_pos(2) < -bs) atoms_pos(2)+=bs; endif
          if (atoms_pos(3) < -cs) atoms_pos(3)+=cs; endif
        endif
        atoms(l,1:10)=[ion_cell{k+2}.bd atoms_pos ion_cell{k+2}.B ion_cell{k+2}.occ ...
        ion_cell{k+2}.radius ion_cell{k+2}.red ion_cell{k+2}.green ion_cell{k+2}.blue];
        atoms_cell{1,l}.ion=ion_cell{k+2}.ion;
        atoms_cell{1,l}.pos_AA=atoms_pos;


	if (draw_nuc_opt{1,end}.style == 0)
	  %--- ball and stick;
          atoms_cell{1,l}.radius=ion_cell{k+2}.radius;
	else
	  %---- anisotropic displacements
	  %---- form matrix from symmetry elements
	  mat=zeros(3,3);
          %---- create rotation matrices from symmetry elements
	  if (~isempty(strfind(spgsym_cell{j+1}.x,"-X")))  
		mat(1,1)=-1;
	  elseif (~isempty(strfind(spgsym_cell{j+1}.x,"X")) | ~isempty(strfind(spgsym_cell{j+1}.x,"+X"))) 
		mat(1,1)=1; 
	  endif
	  if (~isempty(strfind(spgsym_cell{j+1}.x,"-Y")))  
		mat(1,2)=-1; 
	  elseif (~isempty(strfind(spgsym_cell{j+1}.x,"Y")) | ~isempty(strfind(spgsym_cell{j+1}.x,"+Y"))) 
		mat(1,2)=1; 
	  endif
	  if (~isempty(strfind(spgsym_cell{j+1}.x,"-Z")))  
         	mat(1,3)=-1;
	  elseif (~isempty(strfind(spgsym_cell{j+1}.x,"Z")) | ~isempty(strfind(spgsym_cell{j+1}.x,"+Z"))) 
		mat(1,3)=1; 
	  endif


	  if (~isempty(strfind(spgsym_cell{j+1}.y,"-X")))  
		mat(2,1)=-1;
	  elseif (~isempty(strfind(spgsym_cell{j+1}.y,"X")) | ~isempty(strfind(spgsym_cell{j+1}.y,"+X"))) 
		mat(2,1)=1; 
	  endif
	  if (~isempty(strfind(spgsym_cell{j+1}.y,"-Y")))  
		mat(2,2)=-1; 
	  elseif (~isempty(strfind(spgsym_cell{j+1}.y,"Y")) | ~isempty(strfind(spgsym_cell{j+1}.y,"+Y"))) 
		mat(2,2)=1; 
	  endif
	  if (~isempty(strfind(spgsym_cell{j+1}.y,"-Z")))  
		mat(2,3)=-1; 
	  elseif (~isempty(strfind(spgsym_cell{j+1}.y,"Z")) | ~isempty(strfind(spgsym_cell{j+1}.y,"+Z"))) 
		mat(2,3)=1; 
	  endif

	 
	  if (~isempty(strfind(spgsym_cell{j+1}.z,"-X")))  
		mat(3,1)=-1; 
	  elseif (~isempty(strfind(spgsym_cell{j+1}.z,"X")) | ~isempty(strfind(spgsym_cell{j+1}.z,"+X"))) 
		mat(3,1)=1; 
	  endif
	  if (~isempty(strfind(spgsym_cell{j+1}.z,"-Y")))  
		mat(3,2)=-1; 	
  	  elseif (~isempty(strfind(spgsym_cell{j+1}.z,"Y")) | ~isempty(strfind(spgsym_cell{j+1}.z,"+Y"))) 
		mat(3,2)=1; 
	  endif
	  if (~isempty(strfind(spgsym_cell{j+1}.z,"-Z")))  
		mat(3,3)=-1; 	
	  elseif (~isempty(strfind(spgsym_cell{j+1}.z,"Z")) | ~isempty(strfind(spgsym_cell{j+1}.z,"+Z"))) 
		mat(3,3)=1; 
	  endif
	 
          u11=ion_cell{k+2}.u11;
          u22=ion_cell{k+2}.u22;
          u33=ion_cell{k+2}.u33;
          u12=u21=ion_cell{k+2}.u12;
          u13=u31=ion_cell{k+2}.u13;
          u23=u32=ion_cell{k+2}.u23;
	  Ucrys=mat*[u11 u12 u13;...
                     u21 u22 u23;...
                     u31 u32 u33]*mat';
	  atoms_cell{1,l}.radius=Ucrys;
	endif 
        atoms_cell{1,l}.color=[ion_cell{k+2}.red ion_cell{k+2}.blue ion_cell{k+2}.green];
        l=l+1;
      %endif
    endfor
  endfor
endfor

[r,c]=size(atoms_cell);
for k=1:c
   [r,cc]=size(atoms_cell);
   if (k >= cc) break; endif
   str1=num2str(atoms_cell{1,k}.pos_AA)
   for l=k+1:cc
      str2=num2str(atoms_cell{1,l}.pos_AA)
      if (strcmp(str1,str2) == 1) 
          %if positions equal remove atom 
          atoms_cell{1,l}=[];
      endif
   endfor
   %remove empty cells
   atoms_cell=atoms_cell(~cellfun('isempty',atoms_cell));  
endfor
%---- compute distances
[fid]=fopen('atomic_dist.mft','wt');
fprintf(fid,"Distance\n");

l=1;
%---- calculate distances between atoms 
%---- to create bonds later
for i=1:columns(atoms_cell)
  for j=i+1:columns(atoms_cell)
    atdists{1,l}.label=strcat(atoms_cell{1,i}.ion,'-',atoms_cell{1,j}.ion);
    atdists{1,l}.coord_at_1_AA=atoms_cell{1,i}.pos_AA;
    atdists{1,l}.coord_at_2_AA=atoms_cell{1,j}.pos_AA;
    sep_vec=abs(atoms_cell{1,i}.pos_AA.-atoms_cell{1,j}.pos_AA);
    atdists{1,l}.dist=sqrt(sep_vec(1)^2+sep_vec(2)^2+sep_vec(3)^2+...
                          2*cosd(aa)*sep_vec(2)*sep_vec(3)+...
                          2*cosd(bb)*sep_vec(3)*sep_vec(1)+...
                          2*cosd(cc)*sep_vec(1)*sep_vec(2));
    %---- print out distance only if smaller than lattice parameters
    %if (atdists{1,l}.dist < lat_cell)
      fprintf(fid,"%5s at (%+3.2f %+3.2f %+3.2f) and (%+3.2f %+3.2f %+3.2f): %+4.2f AA\n",...
      atdists{1,l}.label,atdists{1,l}.coord_at_1_AA./lat_cell,atdists{1,l}.coord_at_2_AA./lat_cell,atdists{1,l}.dist);
    %endif
    l=l+1;
  endfor
endfor
fclose(fid);
disp('* File ''atomic_dist.mft'' written to disk');

endfunction
