## Copyright (C) 2009 Bertrand Roessli <bertrand.roessli@psi.ch>
##
##
## MuFit is free software; you can redistribute it and/or
## modify it under the terms of the GNU General Public
## License as published by the Free Software Foundation;
## either version 2, or (at your option) any later version.
##
## Mufit is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied
## warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
## PURPOSE.  See the GNU General Public License for more
## details.
##
## You should have received a copy of the GNU General Public
## License along with Octave; see the file COPYING.  If not,
## write to the Free Software Foundation, Inc., 51 Franklin Street,
## Fifth Floor, Boston, MA 02110-1301, USA.
##
## usage: mufit
##
## Author: Bertrand Roessli <bertrand.roessli@psi.ch>

## -*- texinfo -*-
## @deftypefn{Function File}  mt_draw_mag (@var{varargin})
## @deftypefnx{Function File} {[]}= mt_draw_mag (@var{varargin})
## Draw the magnetic structure
## @*input:
##       @*1.- @var{varargin(1)} transformation matrix to cartesian axis
##       @*2.- @var{varargin(2)} lattice constants and angles 
##       @*4.- @var{varargin(3)} atomic positions and magnetic moments
##	 @*5.- @var{varargin(4)} switch polar/cartesian coordinates
##       @*6.- @var{varargin(5)} drawing options
##       cordinates 
## @end deftypefn


function mt_draw_mag(varargin)


navig=sprintf([...
	"NavigationInfo {\n",...
	"   type  [ ""EXAMINE"", ""ANY"" ]\n",...
	"    avatarSize 0\n",...
	"}\n",...
	]);

view=sprintf([...
	"Viewpoint {\n",...
	"    position    0.0 1.0 0.0\n",...
	"    orientation 0.0 0.0 1.0 0.0\n",...
	"}\n",...
	]);

bck_color=sprintf([...
	"Background {\n",...
	"    skyColor    1.0 1.0 1.0\n",...
	"}\n",...
	]);

%---- variables
Mcryst2cart=varargin{1};
lat=varargin{2};
M_cell=varargin{3};
%---- entry type either polar or cartesian coordinates
polar=varargin{4};
%---- drawing options
draw_mag_opt=varargin{5};


%---- draw magnetic cell
a_cart_cell=Mcryst2cart*[lat.as; 0; 0 ];
b_cart_cell=Mcryst2cart*[0; lat.bs; 0];
c_cart_cell=Mcryst2cart*[0; 0; lat.cs];
a=[0.0 0.0 0.0, a_cart_cell', b_cart_cell', c_cart_cell',...
   a_cart_cell'.+b_cart_cell',  a_cart_cell'.+c_cart_cell', ...
   b_cart_cell'.+c_cart_cell', a_cart_cell'.+b_cart_cell'.+c_cart_cell'];
f=[0,1,-1,  0,2,-1,  0,3,-1, 1,4,-1, 1,5,-1, 2,4,-1, 2,6,-1 3,5,-1, 3,6,-1, 4,7,-1, 5,7,-1, 6,7,-1].+1;
col=[0.3 0.4 0.9];
cell=vrml_lines(a,f);

%---- add labels
%---- a-axis
trans=[0.2,0,0]*lat.as; 
a_label_pos=trans.+a_cart_cell';  %---- x
A_axis_labels=vrml_transfo([vrml_text("a","col",[0.3 0.4 0.9],"size",1.5,"family", "SERIF", "style", "PLAIN", "justify", "END")],a_label_pos,[0 0 0], 1);
%---- b-axis
trans=[0,0.2,0]*lat.bs; 
b_label_pos=trans+b_cart_cell'; %---- y
B_axis_labels=vrml_transfo([vrml_text("b","col",[0.3 0.4 0.9],"size",1.5,"family", "SERIF", "style", "PLAIN", "justify", "END")],b_label_pos, [0 0 0], 1);
%----- c-axis
trans=[0,0,0.2]*lat.cs; 
c_label_pos=trans+c_cart_cell';  %---- z
C_axis_labels=vrml_transfo([vrml_text("c","col",[0.3 0.4 0.9],"size",1.5,"family", "SERIF", "style", "PLAIN", "justify", "END")],c_label_pos, [0 0 0], 1);
%---- axis labels
[axis_labels]=vrml_group(A_axis_labels,B_axis_labels,C_axis_labels);
%---- end cell

%---- start balls
nr_ion=cell2mat(varargin{3}(1));
%---- atomic positions in cartesian coordinates
ion=reshape([[M_cell{2:nr_ion+1,1}].mat],3,nr_ion);

%--- atomic positions
at_rec_units.x=ion(1,:)/lat.as;
at_rec_units.y=ion(2,:)./lat.bs;
at_rec_units.z=ion(3,:)./lat.cs;

%--- ion size
radius=[[M_cell{2:nr_ion+1,1}].radius];
%--- colors
color_red=[[M_cell{2:nr_ion+1,1}].red];
color_blue=[[M_cell{2:nr_ion+1,1}].blue];
color_green=[[M_cell{2:nr_ion+1,1}].green];

%----- create balls at atomic positions
for k=1:nr_ion
  radius_ball=radius(k)*eye(3); 	%---- ball
  col_ball=[color_red(k) color_blue(k) color_green(k)];  	   	%---- color
  translation(k,1:3)=[at_rec_units.x(k).*lat.as;...
		at_rec_units.y(k).*lat.bs;...
		at_rec_units.z(k).*lat.cs];
  ball=vrml_transfo([vrml_ellipsoid(radius_ball,col_ball,draw_mag_opt{1,end}.style)],translation(k,1:3),eye(3),1);
%	atoms=vrml_group(atoms,ball);
  atoms{k}=ball;
endfor

for k=2:M_cell{1}+1
	mm(k)=sqrt((M_cell{k,1}.mpx.+M_cell{k,1}.mqx).^2.+...
                   (M_cell{k,1}.mpy.+M_cell{k,1}.mqy).^2.+...
                   (M_cell{k,1}.mpz.+M_cell{k,1}.mqz).^2);
        if (mm(k) == 0)
		mm(k)=0.01;
	endif
endfor
mm=real(mm);

%---- add arrows
for k=2:M_cell{1}+1
  len=mm(k)/max(mm)*2;
  alen=len/4;
  dc=len/16;
  dr=min(dc,len/32);
  col_arrow=[0.4 0.4 0.9];

  arrow=vrml_arrow([len,alen,dc,dr],col_arrow);
  %---- find axis and angle of rotation 
  %---- between two vectors for input in vrml_transfo
  u1=[0 1 0];
  Mx=M_cell{k,1}.mpx.+M_cell{k,1}.mqx;
  My=M_cell{k,1}.mpy.+M_cell{k,1}.mqy;
  Mz=M_cell{k,1}.mpz.+M_cell{k,1}.mqz;
  if (polar == 0)  %---- if spins in polar coordinates, already cartesian
    u2=Mcryst2cart*[Mx My Mz]';
  else
    u2=[Mx My Mz]';
  endif
  % rotation axis
  u1_norm=u1/norm(u1);
  u2_norm=u2/norm(u2);

  rot_axis=cross(u1_norm,u2_norm);
  angle=acos(dot(u1_norm,u2_norm));
  rot_m=[rot_axis' angle];
	
  scale=1;
  spin=vrml_transfo([arrow],translation(k-1,1:3),rot_m,scale);
  mag_ion{k-1}=spin;
endfor

%---- calculate distances between atoms 
%---- to create bonds later
%---- compute distances
[fid]=fopen('atomic_dist.mft','wt');
fprintf(fid,"Distance\n");
l=1;

lat_cell=[lat.as lat.bs lat.cs];
for i=2:rows(M_cell)
  for j=i+1:rows(M_cell)
    atom_dists{1,l}.label=strcat(M_cell{i,1}.label,'-',M_cell{j,1}.label);
    atom_dists{1,l}.coord_at_1_AA=M_cell{i,1}.mat;  % coordinates 1. atom
    atom_dists{1,l}.coord_at_2_AA=M_cell{j,1}.mat;  % coordinates 2. atom
    sep_vec=M_cell{i,1}.mat.-M_cell{j,1}.mat;
    atom_dists{1,l}.dist=norm(sep_vec);
    %---- print out distance only if smaller than lattice parameters
    %if (atom_dists{1,l}.dist < lat_cell)
      fprintf(fid,"%5s at (%+3.2f %+3.2f %+3.2f) and (%+3.2f %+3.2f %+3.2f): %+4.2f AA\n",...
      atom_dists{1,l}.label,atom_dists{1,l}.coord_at_1_AA./lat_cell,atom_dists{1,l}.coord_at_2_AA./lat_cell,atom_dists{1,l}.dist);
   %endif
    l=l+1;
  endfor
endfor
fclose(fid);
disp('* File ''atomic_dist.mft'' written to disk');

%---- create bonds between atoms
l=1;
for i=1:columns(draw_mag_opt)-1
  for k=1:columns(atom_dists)
  %--- if matching labels
    if (strcmp([[atom_dists{1,k}].label],draw_mag_opt{1,i}.label))
      %---- check if within min./max. distances
      if (atom_dists{1,k}.dist > draw_mag_opt{1,i}.mindist & ...
          atom_dists{1,k}.dist < draw_mag_opt{1,i}.maxdist & ...
          atom_dists{1,k}.dist ~= 0)
	bd1=atom_dists{1,k}.coord_at_1_AA';
	bd2=atom_dists{1,k}.coord_at_2_AA';
	x=[bd1'; bd2']';
	bond{1,l}=vrml_cyl(x,"col",draw_mag_opt{1,i}.color,"rad",draw_mag_opt{1,i}.radius,"tran",0.1);
	l++;
      endif
     endif
    endfor 
endfor

if (l~=1)
	all=[view,navig,bck_color,axis_labels,cell,cell2mat(atoms),cell2mat(mag_ion),cell2mat(bond)];
else
	all=[view,navig,bck_color,axis_labels,cell,cell2mat(atoms),cell2mat(mag_ion)];
endif
vrml_browse(all);

%---- save vrml-file
save_vrml('magnetic_structure',all);
disp("* File 'magnetic_structure.wrml' written to disk");

endfunction
