## Copyright (C) 2009 Bertrand Roessli <bertrand.roessli@psi.ch>
##
##
## MuFit is free software; you can redistribute it and/or
## modify it under the terms of the GNU General Public
## License as published by the Free Software Foundation;
## either version 2, or (at your option) any later version.
##
## Mufit is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied
## warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
## PURPOSE.  See the GNU General Public License for more
## details.
##
## You should have received a copy of the GNU General Public
## License along with Octave; see the file COPYING.  If not,
## write to the Free Software Foundation, Inc., 51 Franklin Street,
## Fifth Floor, Boston, MA 02110-1301, USA.
##
## usage: mufit
##
## Author: Bertrand Roessli <bertrand.roessli@psi.ch>

## -*- texinfo -*-
## @deftypefn{Function File}  mt_draw_mag (@var{varargin})
## @deftypefnx{Function File} {[]}= mt_draw_mag (@var{varargin})
## Draw the magnetic structure
## @*input:
##       @*1.- @var{varargin(1)} lattice constants and angles 
##       @*2.- @var{varargin(2)} atomic parameters coordinates 
##       @*3.- @var{varargin(3)} atomic distances 
##       @*4.- @var{varargin(4)} transformation matrix to cartesian coordinates 
##       @*5.- @var{varargin(5)} draw options 
## @end deftypefn


function mt_draw_nuc(varargin)

%---- lattice parameters and angles
lat=varargin{1};
%---- atomic positions in cartesian coordinates
ion=varargin{2};
%---- atomic distances
atom_dists=varargin{3};
%---- transformation matrix to cartesian coordinates
Mcryst2cart=varargin{4};
%---- options 
drawopt=varargin{5};


navig=sprintf([...
	"NavigationInfo {\n",...
	"   type  [ ""EXAMINE"", ""ANY"" ]\n",...
	"    avatarSize 0\n",...
	"}\n",...
	]);

view=sprintf([...
	"Viewpoint {\n",...
	"    position    0.0 1.0 0.0\n",...
	"    orientation 0.0 0.0 1.0 0.0\n",...
	"}\n",...
	]);

bck_color=sprintf([...
	"Background {\n",...
	"    skyColor    1.0 1.0 1.0\n",...
	"}\n",...
	]);



%---- draw a cell
a_cart_cell=Mcryst2cart*[lat.as; 0; 0 ];
b_cart_cell=Mcryst2cart*[0; lat.bs; 0];
c_cart_cell=Mcryst2cart*[0; 0; lat.cs];
a=[0.0 0.0 0.0, a_cart_cell', b_cart_cell', c_cart_cell',...
   a_cart_cell'.+b_cart_cell',  a_cart_cell'.+c_cart_cell', ...
   b_cart_cell'.+c_cart_cell', a_cart_cell'.+b_cart_cell'.+c_cart_cell'];
f=[0,1,-1,  0,2,-1,  0,3,-1, 1,4,-1, 1,5,-1, 2,4,-1, 2,6,-1 3,5,-1, 3,6,-1, 4,7,-1, 5,7,-1, 6,7,-1].+1;
col=[0.3 0.4 0.9];
cell=vrml_lines(a,f);

%---- add labels to axis
translation=[0,0,0]; 
col=eye(3);
scale=2;
%a
a_label_pos=translation.+a_cart_cell';  %---- x
A_axis_labels=vrml_transfo([vrml_text("a","col",[0.3 0.4 0.9],"size",1.5,"family", "SERIF", "style", "PLAIN", "justify", "END")],a_label_pos, [0,0,0], 1);
%b
b_label_pos=translation+b_cart_cell'; %---- y
B_axis_labels=vrml_transfo([vrml_text("b","col",[0.3 0.4 0.9],"size",1.5,"family", "SERIF", "style", "PLAIN", "justify", "END")],b_label_pos, [0,0,0], 1);
%c
c_label_pos=translation+c_cart_cell';  %---- z
C_axis_labels=vrml_transfo([vrml_text("c","col",[0.3 0.4 0.9],"size",1.5,"family", "SERIF", "style", "PLAIN", "justify", "END")],c_label_pos, [0,0,0], 1);
%
%---- group axis, axis labels
[coord_axis]=vrml_group(A_axis_labels,B_axis_labels,C_axis_labels);

atoms=" ";
%--- create balls/ellipsoids at atomic positions
for k=1:columns(varargin{2})
	%--- ball and sticks
	radius_ball=ion{1,k}.radius; 	%---- radius 
	col_ball=ion{1,k}.color;        %---- color
	translation(1,1:3)=Mcryst2cart*ion{1,k}.pos_AA';     %---- translation
	ball=vrml_transfo([vrml_ellipsoid(radius_ball, col_ball, drawopt{1,end}.style)],translation(1,1:3),eye(3),1);
	atoms=vrml_group(atoms,ball);
endfor

%---- create bonds between atoms
l=1;

for i=1:columns(drawopt)-1
  for k=1:columns(atom_dists)
  %--- if matching labels
    if (strcmp([[atom_dists{1,k}].label],drawopt{1,i}.label))
      %---- check if within min./max. distances
      if (atom_dists{1,k}.dist > drawopt{1,i}.mindist & ...
          atom_dists{1,k}.dist < drawopt{1,i}.maxdist & ...
          atom_dists{1,k}.dist ~= 0)
	bd1=Mcryst2cart*atom_dists{1,k}.coord_at_1_AA';
	bd2=Mcryst2cart*atom_dists{1,k}.coord_at_2_AA';
	x=[bd1'; bd2']';
	bond{1,l}=vrml_cyl(x,"col",drawopt{1,i}.color,"rad",drawopt{1,i}.radius,"tran",0.1);
	l++;
      endif
     endif
    endfor 
endfor

if (l~=1) 
	h=vrml_group(view,navig,bck_color,cell2mat(bond),atoms);
else
	h=vrml_group(view,navig,bck_color,coord_axis,cell,atoms);
endif

vrml_browse(h);

save_vrml('nuclear_structure',h);
disp("* File 'nuclear_structure.wrml' written to disk");

endfunction
