## Copyright (C) 2007,2008,2009 Bertrand Roessli <bertrand.roessli@psi.ch>
##
##
## MuFit is free software; you can redistribute it and/or
## modify it under the terms of the GNU General Public
## License as published by the Free Software Foundation;
## either version 2, or (at your option) any later version.
##
## Mufit is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied
## warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
## PURPOSE.  See the GNU General Public License for more
## details.
##
## You should have received a copy of the GNU General Public
## License along with Octave; see the file COPYING.  If not,
## write to the Free Software Foundation, Inc., 51 Franklin Street,
## Fifth Floor, Boston, MA 02110-1301, USA.
##

## -*- texinfo -*-
## @deftypefn{Function File}  nucfac(@var{hkl_crys_rlu},@var{hkl_crys_AA})
## @deftypefnx{Function File} {[@var{strufac2}] =
## nucfac(@var{hkl_crys_rlu},@var{hkl_crys_AA})
## @*Calculate the nuclear structure factors to obtain the nuclear-magnetic 
## interference term in the polarisation matrices;
## The nuclear model is defined in the file 'nuc.inp'
##
## @*input:
##      @*1.- @var{varargin@{1@}}: hkl-list (rlu)
##      @*2.- @var{varargin@{2@}}: hkl-list (AA)
##
## @*output:
##     @* @var{strufac2}: nuclear structure factor^2

function [strufac]=nucfac(as,bs,cs,aa,bb,cc,ion_cell,spgsym_cell,spgtrans_cell,hkl_crys_AA)


%---- transformation matrices to cartesian system
[Mcryst2cart,Mrec2cart]=lat2cart(as,bs,cs,aa,bb,cc);


atoms=zeros(ion_cell{1}*spgsym_cell{1}*spgtrans_cell{1},6);
atoms_pos=zeros(1,3);

l=1;
%---- loop over translations/symmetry elements of the space group to create ions in the unit cell
for k=1:ion_cell{1}
	X=ion_cell{k+2}.x;
	Y=ion_cell{k+2}.y;
	Z=ion_cell{k+2}.z;
	for i=1:spgtrans_cell{1}
		for j=1:spgsym_cell{1}
			%create atomic positions in AA
			atoms_pos(1:3)=Mcryst2cart*([eval(spgsym_cell{j+1}.x).*as eval(spgsym_cell{j+1}.y).*bs eval(spgsym_cell{j+1}.z).*cs].+...
		       	            [spgtrans_cell{i+1}.x.*as spgtrans_cell{i+1}.y.*bs spgtrans_cell{i+1}.z.*cs])';
			%create array with scattering length, x,y,z, B-factor and occupation number
			atoms(l,1:6)=[ion_cell{k+2}.bd atoms_pos ion_cell{k+2}.B ion_cell{k+2}.occ];
			l=l+1;
		endfor
	endfor
endfor

%structure factor
sl=[atoms(:,1)];   %scattering length
ap=[atoms(:,2:4)];  %atomic positions 
B_iso=[atoms(:,5)]; %B-factor
occ=[atoms(:,6)];   %occupation number
Q2=dot(hkl_crys_AA,hkl_crys_AA,2)./(4*pi)^2;
B_iso=[atoms(:,5)];
two_W=Q2*abs(B_iso)';
ft=exp(I.*ap*hkl_crys_AA').*exp(-two_W)';
strufac=(sl.*occ)'*ft;
%the sum over the atoms in the unit cell is done automatically

endfunction

#{
%----remove extinction and lorentz factor corrections for calculation of polarisation matrices
%add extinction correction
if (genpar.zach ~= 0)
        x=real(strufac.^2);
        zachpar=genpar.zach;
        lambda=genpar.lambda;
        fhandle=@(x) extinc(x,lambda,zachpar,hkl_crys_AA);
        [ys]=fhandle(x);
else
        ys=1;
endif

%Lorentz factor
if (singlecrys{1}.intfsqr == 0)
        Q=hkl_crys_AA;
        d_sp=2*pi./dot(Q,Q,2).^0.5;
        th_angle=asin(genpar.lambda./2./d_sp);
        lor=1./sin(2.*th_angle);
        strufac2=strufac.*conj(strufac).*ys.*lor';
else
        strufac2=strufac.*conj(strufac).*ys;
endif
strufac2=genpar.scale.*real(strufac2);
#}