## Copyright (C) 2007,2008,2009 Bertrand Roessli <bertrand.roessli@psi.ch>
##
##
## MuFit is free software; you can redistribute it and/or
## modify it under the terms of the GNU General Public
## License as published by the Free Software Foundation;
## either version 2, or (at your option) any later version.
##
## Mufit is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied
## warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
## PURPOSE.  See the GNU General Public License for more
## details.
##
## You should have received a copy of the GNU General Public
## License along with Octave; see the file COPYING.  If not,
## write to the Free Software Foundation, Inc., 51 Franklin Street,
## Fifth Floor, Boston, MA 02110-1301, USA.
##
## usage: mufit
##

## -*- texinfo -*-
## @deftypefn{Function File}  nucfacfit2(@var{p},@var{varargin})
## @deftypefnx{Function File} {[@var{strufac2},@var{param}]} = nucfacfit2(@var{p},@var{varargin})
## @*Fit the nuclear structure factors or intensities
##
## @*input: 
##      @*1.-  @var{p}: list of parameters @var{p}
##      @*2.-  @var{varargin@{1@}}: hkl-list (rlu)
##      @*3.-  @var{varargin@{2@}}: hkl-list (AA)
##      @*4.-  @var{varargin@{3@}}: wavelength, extinction 
##      @*5.-  @var{varargin@{4@}}: lattice parameters
##      @*6.-  @var{varargin@{5@}}: intensities
##      @*7.-  @var{varargin@{6@}}: atomic parameters
##      @*8.-  @var{varargin@{7@}}: space group symmetry elemetns
##      @*9.-  @var{varargin@{8@}}: translations of the space group (can be put in
##      @*10.- @var{varargin@{9@}}}: nr. of twins
##      @var{varargin@{7@}}})
##
## @*output: 
##     @* @var{strufac2}: nuclear structure factor^2
##     @* @var{parameters}: parameters


function [strufac2,param]=nucfacfit2(p,varargin)
param=p;;
hkl_crys_rlu=varargin{1};
hkl_cart_AA=varargin{2};
genpar=varargin{3};
lat_cell=varargin{4};
intfsqr=varargin{5};
ion_cell=varargin{6};
spgsym_cell=varargin{7};
spgtrans_cell=varargin{8};
nr_twins=varargin{9};
Rmat = varargin{10};
Rmatinv = varargin{11};

N1=param(1);
N2=param(2);
N3=param(3);
N4=param(4);
N5=param(5);
N6=param(6);
N7=param(7);
N8=param(8);
N9=param(9);
N10=param(10);
N11=param(11);
N12=param(12);
N13=param(13);
N14=param(14);
N15=param(15);
N16=param(16);
N17=param(17);
N18=param(18);
N19=param(19);
N20=param(20);
N21=param(21);
N22=param(22);
N23=param(23);
N24=param(24);
N25=param(25);
N26=param(26);
N27=param(27);
N28=param(28);
N29=param(29);
N30=param(30);
N31=param(31);
N32=param(32);
N33=param(33);
N34=param(34);
N35=param(35);
N36=param(36);
N37=param(37);
N38=param(38);
N39=param(39);
N40=param(40);
N41=param(41);
N42=param(42);
N43=param(43);
N44=param(44);
N45=param(45);
N46=param(46);
N47=param(47);
N48=param(48);
N49=param(49);
N50=param(50);

as=lat_cell.as;
bs=lat_cell.bs;
cs=lat_cell.cs;
aa=lat_cell.aa;
bb=lat_cell.bb;
cc=lat_cell.cc;


%---- transformation matrices to cartesian system
[Mcryst2cart,Mrec2cart]=lat2cart(as,bs,cs,aa,bb,cc);

atoms=zeros(ion_cell{1}*spgsym_cell{1}*spgtrans_cell{1},12);
atoms_pos=zeros(1,3);

l=1;
%---- loop over translations/symmetry elements of the space group to create ions in the unit cell
for k=1:ion_cell{1}
        %--- atomic positions
	X=eval(ion_cell{k+2}.x_par);
	Y=eval(ion_cell{k+2}.y_par);
	Z=eval(ion_cell{k+2}.z_par);
        %---- anisotropic displacements
        if (ion_cell{columns(ion_cell)}.Btype == 1)
          U11 = eval(ion_cell{k+2}.u11_par);
          U12 = eval(ion_cell{k+2}.u12_par);
          U13 = eval(ion_cell{k+2}.u13_par);
          U21 = U12;
          U22 = eval(ion_cell{k+2}.u22_par);
          U23 = eval(ion_cell{k+2}.u23_par);
          U31 = U13;
          U32 = U23;
          U33 = eval(ion_cell{k+2}.u33_par);
          U = [U11 U12 U13; U21 U22 U23; U31 U32 U33];
        endif
	for i=1:spgtrans_cell{1}
		for j=1:spgsym_cell{1}
			%---- create array of atomic positions in cartesian coordinates (AA)
			atoms_pos(1:3)=Mcryst2cart*([eval(spgsym_cell{j+1}.x).*as eval(spgsym_cell{j+1}.y).*bs eval(spgsym_cell{j+1}.z).*cs].+...
		       	            [spgtrans_cell{i+1}.x.*as spgtrans_cell{i+1}.y.*bs spgtrans_cell{i+1}.z.*cs])';
			%	
			%---- create array with scattering length, x,y,z, B-factor and occupation number
			atoms(l,1:6)=[ion_cell{k+2}.bd atoms_pos eval(ion_cell{k+2}.B_par) eval(ion_cell{k+2}.occ_par)];
                        switch ion_cell{columns(ion_cell)}.Btype 
                        case 0 %---- isotropic B-factor
                          atoms(l,7:12)=[0 0 0 0 0 0];
                        case 1
			  %---- anisotropic displacements
                          % transform U tensor according to symmetry element
                          T = Rmatinv(:,:,j)*U*Rmat(:,:,j);
			  atoms(l,7:12)=[T(1,1) T(2,2) T(3,3) T(1,2) T(1,3) T(2,3)];
                        endswitch
			l=l+1;
		endfor
	endfor
endfor
%---- structure factor
sl=[atoms(:,1)];    %scattering length
ap=[atoms(:,2:4)];  %atomic positions

%---- thermal factors
switch ion_cell{columns(ion_cell)}.Btype 
  case 0 %---- isotropic B-factor
        Q2=dot(hkl_cart_AA,hkl_cart_AA,2)./(4*pi)^2;
	B_iso=[atoms(:,5)];
	two_W=Q2*abs(B_iso)';
  case 1 %---- anisotropic temperature factors
#{
        two_W=atoms(:,7)*transpose(hkl_crys_AA(:,1).^2).+...    
              atoms(:,8)*transpose(hkl_crys_AA(:,2).^2).+...    
              atoms(:,9)*transpose(hkl_crys_AA(:,3).^2).+...
              2.*atoms(:,10)*transpose(hkl_crys_AA(:,1).*hkl_crys_AA(:,2)).+...
              2.*atoms(:,11)*transpose(hkl_crys_AA(:,1).*hkl_crys_AA(:,3)).+...
              2.*atoms(:,12)*transpose(hkl_crys_AA(:,2).*hkl_crys_AA(:,3));
        two_W /= (2*pi)^2;
#}
        gij=metric(as,bs,cs,aa,bb,cc);
        gsij = inv(gij);
        astar = sqrt(gsij(1,1));
        bstar = sqrt(gsij(2,2));
        cstar = sqrt(gsij(3,3));
        cosgs = gsij(2,1)/astar/bstar;
        cosbs = gsij(3,1)/astar/cstar;
        cosas = gsij(3,2)/bstar/cstar; 
        two_W=atoms(:,7)*transpose(hkl_crys_rlu(:,1).^2)*astar^2.+...    
              atoms(:,8)*transpose(hkl_crys_rlu(:,2).^2)*bstar^2.+...    
              atoms(:,9)*transpose(hkl_crys_rlu(:,3).^2)*cstar^2.+...
              2.*atoms(:,10)*transpose(hkl_crys_rlu(:,1).*hkl_crys_rlu(:,2))*astar*bstar*cosgs.+...
              2.*atoms(:,11)*transpose(hkl_crys_rlu(:,1).*hkl_crys_rlu(:,3))*astar*cstar*cosbs.+...
              2.*atoms(:,12)*transpose(hkl_crys_rlu(:,2).*hkl_crys_rlu(:,3))*bstar*cstar*cosas;
        two_W=(2*pi^2).*transpose(two_W);
endswitch

occ=[atoms(:,6)];   %---- occupation number
%---- vectorised version of the structure factor (27.03.09)
ft=exp(I.*ap*hkl_cart_AA').*exp(-two_W)';
strufac=(sl.*occ)'*ft;
#{
% or:
ft=exp(I.*ap*hkl_cart_AA').*exp(-two_W)';
tmp = sl.*occ;
dim = size(hkl_cart_AA);
sl_occ = repmat(tmp,1,dim(1));
strufac_mat = sl_occ.*ft;
strufac = sum(strufac_mat);
#}

%---- the sum over the atoms in the unit cell is done automatically 

%---- add extinction correction
if (eval(genpar.zach_par) ~= 0)
	x=strufac.*conj(strufac);
	zachpar=eval(genpar.zach_par);
	lambda=genpar.lambda;
	fhandle=@(x) extinc(x,lambda,zachpar,hkl_cart_AA);
	[ys]=fhandle(x);
else
	ys=ones(1,rows(hkl_cart_AA));
endif

%---- Lorentz factor
if (intfsqr == 0)
	Q=hkl_cart_AA;
        d_sp=2*pi./dot(Q,Q,2).^0.5;
        th_angle=asin(genpar.lambda./2./d_sp);
        lor=1./sin(2.*th_angle);
	strufac2=strufac.*conj(strufac).*ys.*lor';
else
	strufac2=strufac.*conj(strufac).*ys;
endif

%----- reshape to sum over the twins
if (nr_twins > 1)
	I_hkl_dom=reshape(strufac2,columns(strufac2)/nr_twins,nr_twins)';
	%----- calculate  intensities averaged over the domains
	I_crys=sum(I_hkl_dom);
	strufac2=eval(genpar.scale_par).*real(I_crys)/nr_twins;
else
	strufac2=eval(genpar.scale_par).*real(strufac2);
endif
endfunction
