## Copyright (C) 2007,2008,2009 Bertrand Roessli <bertrand.roessli@psi.ch>
##
##
## MuFit is free software; you can redistribute it and/or
## modify it under the terms of the GNU General Public
## License as published by the Free Software Foundation;
## either version 2, or (at your option) any later version.
##
## Mufit is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied
## warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
## PURPOSE.  See the GNU General Public License for more
## details.
##
## You should have received a copy of the GNU General Public
## License along with Octave; see the file COPYING.  If not,
## write to the Free Software Foundation, Inc., 51 Franklin Street,
## Fifth Floor, Boston, MA 02110-1301, USA.
##
## usage: mufit
##

## -*- texinfo -*-
## @deftypefn{Function File}  onedomain
## (@var{P_1},@var{P_2},@var{hhkl},@var{Q_rlu},@var{AA1},@var{flag_k0},@var{flag_nmi},@var{spins_cell},@var{dtype},@var{pi_direction},@var{propvec_cell},@var{det_r})
## @deftypefnx{Function File} {[@var{Ip_ij},@var{Im_ij},@var{polar}]} = onedomain
## (@var{P_1},@var{P_2},@var{hhkl},@var{Q_rlu},@var{AA1},@var{flag_k0},@var{flag_nmi},@var{spins_cell},@var{dtype},@var{pi_direction},@var{propvec_cell},@var{det_r})
## @*Calculate scattered polarization of a single magnetic domain
## 
## @*input:
##       @*1.- @var{P_1}, @var{P_2}: polarizer efficiencies
##       @*2.- @var{hhkl}: hkl
##       @*3.- @var{Q_rlu}: scattering vector 
##       @*4.- @var{AA1}: second vector in scattering plane 
##       @*5.- @var{flag_k0}: flag k_0 = -k_0
##       @*6.- @var{flag_nmi}: flag nuclear-magnetic interaction vector
##       @*7.- @var{spins_cell}: magnetic model
##       @*8.- @var{dtype}: not used
##       @*9.- @var{pi_direction}: switch for incident polarization direction
##       (+z/-z)
##       @*10.- @var{propvec_cell}: propagation vector
##       @*11.- @var{det_r}: determinant of domain creation matrix
##       @*12.- @var{nucstrucfac}: nuclear structure factor
## @*output:
##       @*1.- @var{Ip_ij}: polarized neutron intensity (P_i +z)
##       @*2.- @var{Im_ij}: polarized neutron intensity (P_i -z)
##       @*3.- @var{polar}: scattered polarization matrix
## @end deftypefn


function [Ip_ij,Im_ij,polar]=onedomain(P_1,P_2,hhkl,Q_rlu,AA1,flag_k0,flag_nmi,spins_cell,dtype,pi_direction,propvec_cell_rlu,det_r,nucstrucfac)


%---- first calculate neutron cross-sections for incident polarisation Pi 
%---- i.e. get back from pol_sd 2x2 matrix
[poln_cs]=pol_sd(hhkl,AA1,Q_rlu,flag_k0,flag_nmi,spins_cell,propvec_cell_rlu,dtype,det_r,nucstrucfac);


s_px=1/sqrt(2)*[1; 1];s_mx=1/sqrt(2)*[1; -1];
s_py=1/sqrt(2)*[1; I];s_my=1/sqrt(2)*[1; -I];
s_pz=[1; 0];s_mz=[0; 1];
vec_p={s_px,s_py,s_pz};
vec_m={s_mx,s_my,s_mz};


if (length(pi_direction) == 5)
%---- if (pi_direction == 'pi_up')
%---- 1) sigma x -> x, sigma x -> -x and pol_xx, and the same for y,z
	for i=1:3
		for j=1:3
			[I_pp]=sigma_tot(vec_p{i},vec_p{j},vec_m{i},vec_m{j},poln_cs,P_1,P_2);
			[I_pm]=sigma_tot(vec_p{i},vec_m{j},vec_m{i},vec_p{j},poln_cs,P_1,P_2);
			Ip_ij(i,j)=I_pp;
			Im_ij(i,j)=I_pm;
			if ((I_pp + I_pm) < 1.0e-5);
				polar(i,j)=0;
			else
				polar(i,j)=(I_pp-I_pm)/(I_pp+I_pm);
			endif	
			if (Ip_ij(i,j) < 1.0e-5) Ip_ij(i,j)=0; endif;
			if (Im_ij(i,j) < 1.0e-5) Im_ij(i,j)=0; endif;
		endfor
	endfor
else 
%---- if incident polarisation along -x,-y,-z
	for i=1:3
		for j=1:3
			[I_mp]=sigma_tot(vec_m{i},vec_p{j},vec_p{i},vec_m{j},poln_cs,P_1,P_2);
			[I_mm]=sigma_tot(vec_m{i},vec_m{j},vec_p{i},vec_p{j},poln_cs,P_1,P_2);
			Ip_ij(i,j)=I_mp;
			Im_ij(i,j)=I_mm;
			if ((I_mp + I_mm) < 1.0e-5)
				polar(i,j)=0;
			else
				polar(i,j)=(I_mp-I_mm)/(I_mp+I_mm);
			endif	
			if (Ip_ij(i,j) < 1.0e-5) Ip_ij(i,j)=0; endif;
			if (Im_ij(i,j) < 1.0e-5) Im_ij(i,j)=0; endif;
		endfor
	endfor
endif

endfunction
