## Copyright (C) 2007,2008,2009 Bertrand Roessli <bertrand.roessli@psi.ch>
##
##
## MuFit is free software; you can redistribute it and/or
## modify it under the terms of the GNU General Public
## License as published by the Free Software Foundation;
## either version 2, or (at your option) any later version.
##
## Mufit is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied
## warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
## PURPOSE.  See the GNU General Public License for more
## details.
##
## You should have received a copy of the GNU General Public
## License along with Octave; see the file COPYING.  If not,
## write to the Free Software Foundation, Inc., 51 Franklin Street,
## Fifth Floor, Boston, MA 02110-1301, USA.
##

function [pol_final]=pol(Pi,hhkl,hhkl_rlu,AA1,nr,flag_k0,flag_nmi,...
			lat_cell,ion_cell,spgsym_cell,spgtrans_cell,propvec_cell_rlu,propvec_cell_AA,moment_cell)


Pi=(reshape(Pi,3,nr))';

%---- calculate spin directions
[M_list,spins_cell]=moment(lat_cell,ion_cell,moment_cell,propvec_cell_AA,spgsym_cell,spgtrans_cell);

%---- read atomic parameters if flag_nmi ~= 0
if (flag_nmi ~= 0)
  [genpar,as,bs,cs,aa,bb,cc,ion_cell,spgsym_cell,spgtrans_cell,twins,singlecrys,...
   parameters,dparameters,...
   nfit_type,simplexnuc_options,lmfitnuc_options,nsim_ann_par,...
   draw_nuc_opt]=readnuc;
endif

%---- calculate polarisation matrices for each hkl
jj=0;
for k=1:nr
	%----- Q vector in cartesian coordinates (1/A)
	Q=hhkl(k,1:3);

	%---- calculate magnetic interaction vector
	[mag_inter_vec,fsqr,mag_vec_cart]=feval("magfac",spins_cell,Q,hhkl_rlu(k,:),propvec_cell_rlu,flag_k0,'S');

        if (flag_nmi == 0)
           nucstrucfac = 0;
        else
           %---- calculate nuclear structure factor without extinction and Lorenz corrections
           [nucstrucfac]=feval("nucfac",as,bs,cs,aa,bb,cc,ion_cell,spgsym_cell,spgtrans_cell,Q);
        endif

	%---- second vector in scattering plane
	%---- first vector is given by Q
	A1=AA1(k,1:3);

	%---- incident polarisation
	P0_x=Pi(k,1);
	P0_y=Pi(k,2);
	P0_z=Pi(k,3);

	%---- calculate cross-section and final polarisation
	[Pf_1d,cs_1d]=feval("ncs",Q,A1,P0_x,P0_y,P0_z,mag_inter_vec,flag_nmi,1,nucstrucfac);	

	pol_final((3*jj)+1:3*k)=Pf_1d;
	jj++;
endfor

endfunction
