## Copyright (C) 2007,2008,2009 Bertrand Roessli <bertrand.roessli@psi.ch>
##
##
## MuFit is free software; you can redistribute it and/or
## modify it under the terms of the GNU General Public
## License as published by the Free Software Foundation;
## either version 2, or (at your option) any later version.
##
## Mufit is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied
## warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
## PURPOSE.  See the GNU General Public License for more
## details.
##
## You should have received a copy of the GNU General Public
## License along with Octave; see the file COPYING.  If not,
## write to the Free Software Foundation, Inc., 51 Franklin Street,
## Fifth Floor, Boston, MA 02110-1301, USA.
##

## -*- texinfo -*-
## @deftypefn{Function File} pol_sd (@var{hhkl},@var{AA1},@var{Q_rlu},@var{flag_k0},@var{flag_nmi},@var{spins_cell},@var{propvec_cell_rlu},@var{dtype},@var{det_r})
## @deftypefnx{Function File} {[@var{poln_cs}]} =  pol_sd (@var{hhkl},@var{AA1},@var{Q_rlu},@var{flag_k0},@var{flag_nmi},@var{spins_cell},@var{propvec_cell_rlu},@var{dtype},@var{det_r})
## @*Calculate neutron intensities in one magnetic domain as 2x2 matrix
##
## @*input: 
##	 @*1.-: @var{hhkl}: list of hhkl
##	 @*2.-: @var{AA1}: second vector to define scattering plane
##       @*3.-: @var{Q_rlu}: Q vector
##       @*4.-: @var{flag_k0}: k_0=-k_0?
##	 @*5.-: @var{flag_nmi} : flag for pure magnetic/mixed nuclear-magnetic reflections
##	 @*6.-: @var{spins_cell}: spin positions and directions
##       @*7.-: @var{propvec_cell_rlu}: propagation vector
##       @*8.-: @var{dtype}: not used
##       @*9.-: @var{det_r}: determinant of domain operation
##       @*10.-: @var{nucstrucfac}: nuclear structure factor
##
## @*output: 
##       @*1.-: @var{poln_cs}: neutron cross-section

## @end deftypefn


function [poln_cs]=pol_sd(hhkl,AA1,Q_rlu,flag_k0,flag_nmi,spins_cell,propvec_cell_rlu,dtype,det_r,nucstrucfac)



%----- Q vector in cartesian coordinates (1/A)
hkl=hhkl';
H=hkl(1);
K=hkl(2);
L=hkl(3);
Q=[H;K;L]';

%---- calculate magnetic interaction vector
[mag_inter_vec,fsqr,mag_vec_cart]=magfac(spins_cell,Q,Q_rlu,propvec_cell_rlu,flag_k0,dtype,det_r);

%--- 180 deg. domains are lated by time-inversion symmetry
%--- i.e. Mperp --> -Mperp
%--- only distinguishable if propagation vector k=0, i.e. mixed
%--- nuclear-magnetic reflections

if dtype=='T'
    mag_inter_vec=-mag_inter_vec;
endif

P_f=zeros(1,3);
%---- Pauli Matrices
sigx=[0 1; 1 0];
sigy=[0 -I; I 0];
sigz=[1 0; 0 -1];

%---- define coordinate system in polarimetry coordinates
%---- component of (HKL) in cartesian coordinates (1/A)
%---- x along Q
V1=[H;K;L];
%---- second vector in scattering plane
%---- first vector is given by Q
A1=AA1';
%---- component of vector in scattering plane in cartesian coordinates (1/A)
V2=A1;

if (dot(V1/norm(V1),V2/norm(V2)) == 1 )
	error("\nredefine scattering plane\n");
	return;
endif
%----- Form unit vectors V1, V2, V3 in scattering plane
V3=cross(V1,V2);
V2=cross(V3,V1);
V3=V3/sqrt(sum(V3.*V3));
V2=V2/sqrt(sum(V2.*V2));
V1=V1/sqrt(sum(V1.*V1));
V3=V3.*det_r;

%---- coordinates of HKL in V1, V2, V3 frame
%---- transformation matrix
U=[V1';V2';V3'];

%---- rotate magnetic interaction vector into MuPAD system
magstruc=U*mag_inter_vec;

%---- calculate cross-section terms
poln_cs=nucstrucfac*eye(2).+(magstruc(1).*sigx.+magstruc(2).*sigy.+magstruc(3).*sigz);
endfunction
