function polyf()
%
% Create HKL list using extinction rules
% and
% Calculate scattering angles in the normal beam geometry
% (uses UB-matrix)
%

[fid,msg]=fopen('poly.inp','r');
if fid == -1
   msg=[msg, '\n'];
   fprintf(stdout,msg);
   return
endif
while (line=fgetl(fid)) != -1
   if !isempty(strfind(line,"TITLE"))
      [tok, remain]=strtok(line);
      title = remain;
   endif
   if !isempty(strfind(line,"WAVELENGTH"))
      [tok, remain]=strtok(line);
      lambda = str2num(remain);
   endif
   if !isempty(strfind(line,"CELL"))
      [tok, remain]=strtok(line);
      cell = str2num(remain);
   endif
   if !isempty(strfind(line,"HMIN"))
      [tok, remain]=strtok(line);
      hmin = remain;
   endif
   if !isempty(strfind(line,"HMAX"))
      [tok, remain]=strtok(line);
      hmax = remain;
   endif
   if !isempty(strfind(line,"KMIN"))
      [tok, remain]=strtok(line);
      kmin = remain;
   endif
   if !isempty(strfind(line,"KMAX"))
      [tok, remain]=strtok(line);
      kmax = remain;
   endif
   if !isempty(strfind(line,"LMIN"))
      [tok, remain]=strtok(line);
      lmin = remain;
   endif
   if !isempty(strfind(line,"LMAX"))
      [tok, remain]=strtok(line);
      lmax = remain;
   endif
   if !isempty(strfind(line,"NU_MIN"))
      [tok, remain]=strtok(line);
      nu_min = remain
   endif
   if !isempty(strfind(line,"NU_MAX"))
      [tok, remain]=strtok(line);
      nu_max = remain
   endif
   if !isempty(strfind(line,"EXTINCT"))
      hklcond="";
      while (line=fgetl(fid)) != -1
         if isempty(strfind(line,"END"))
            hklcond=[hklcond;line];
            break;
         endif
      endwhile
   endif
   if !isempty(strfind(line,"UBMAT"))
      UB=zeros(3,3);
      for i=1:3
          line=fgetl(fid);
          UB(i,1:3)=str2num(line);
      endfor
   endif
endwhile
fclose(fid);

%create hkl list
h=str2num(hmin):str2num(hmax);
k=str2num(kmin):str2num(kmax);
l=str2num(lmin):str2num(lmax);
i=1;
hkl=zeros(columns(h)*columns(k)*columns(l),3);
for hh = h
   for kk = k
      for ll = l
         hkl(i,1:3)=[hh kk ll];
         ++i;
      endfor
   endfor
endfor
for i=1:rows(hklcond)
   c=strsplit(hklcond(i,:),' ');
   c=c(~cellfun('isempty', c));
   l=1;
   for k=1:rows(hkl)
      %extinction condition contains a 0?
      if isempty(strfind(c{1,1},'O'))
         %condition on HKL
         [tok,remain]=strtok(c{1,2},'=');
         H=hkl(k,1);
         K=hkl(k,2);
         L=hkl(k,3);
         res=eval(tok);
         [tok,remain]=strtok(remain,'=');
         if (!isempty(str2num(remain))) 
            res=res-str2num(remain); 
         endif
         if (rem(res,str2num(tok(1))) != 0)
            indx(l)=k;
            l=l+1;
         endif
      else
         % which of h,k or l is 0?
         O=strfind(c{1,1},'O');
         if (isequal(hkl(k,O),0))
            [tok,remain]=strtok(c{1,2},"=");
            H=hkl(k,1);
            K=hkl(k,2);
            L=hkl(k,3);
            res=eval(tok);
            [tok,remain]=strtok(remain,'=');
            if (!isempty(str2num(remain))) 
                res=res-str2num(remain); 
            endif
            if (rem(res,str2num(tok(1))) != 0)
                indx(l)=k;
                l=l+1;
            endif
         endif
      endif
   endfor
   hkl(indx,:)=[];
   indx=[];
endfor

%---calculate scattering angles for hkl list
%---normale beam geometry
[fid]=fopen('flipf.out','w');
datahkl=zeros(rows(hkl),7);
for k=1:rows(hkl)
   hphi = UB * hkl(k,1:3)';
   [gamma,om,nu,status] = z1mToNormalBeam(lambda,hphi);
   datahkl(k,1:7)=[hkl(k,1), hkl(k,2), hkl(k,3),...
                   gamma, om, nu, status];
   if status != 0
      if nu < str2num(nu_max) && nu > str2num(nu_min)
         fprintf(fid,"%i %i %i  %f\n", hkl(k,1), hkl(k,2), hkl(k,3), nu);
      endif
   endif
endfor
fclose(fid);
endfunction


#{
TITLE_:     BaMnF4
WAVELENGTH: 0.7748  
CELL_:      5.980  15.098 4.2200 90.0 90.0 90.0
HMIN_:      0
HMAX_:      5
KMIN_:      0
KMAX_:      5
LMIN_:      0
LMAX_:      5
EXTINCT_:
            HKL K+L=2N 
            HOL H=2N   
            HOO H=2N  
END_
UBMAT:
-0.017950  0.000610  0.188620
-0.042370 -0.173200 -0.003470
 0.127620 -0.031460  0.012250
#}

