## Copyright (C) 2007,2008,2009 Bertrand Roessli <bertrand.roessli@psi.ch>
##
##
## Octave is free software; you can redistribute it and/or
## modify it under the terms of the GNU General Public
## License as published by the Free Software Foundation;
## either version 2, or (at your option) any later version.
##
## Octave is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied
## warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
## PURPOSE.  See the GNU General Public License for more
## details.
##
## You should have received a copy of the GNU General Public
## License along with Octave; see the file COPYING.  If not,
## write to the Free Software Foundation, Inc., 51 Franklin Street,
## Fifth Floor, Boston, MA 02110-1301, USA.
##


function [genpar,as,bs,cs,aa,bb,cc,ion_cell,spgsym_cell,spgtrans_cell,p,singlecrys]=read_flipping_ratio_input;


%---- import from file 'fliprat.inp' information
[fid, msg]=fopen('flipratio.inp','rt');
if fid == -1
   msg=[msg, '\n'];
   fprintf(stdout,msg);
   return
endif

%---- wavelength
[dummy]=fgetl(fid);
[lambda]=fscanf(fid,'%f\n',"C");

%---- extinction
[dummy]=fgetl(fid);
[zach]=fscanf(fid,'%f \n',"C");
%[zach_par,zach_dpar]=fscanf(fid,'%s  %s\n',"C");
[zach_par]=fscanf(fid,'%s  %s\n',"C");

%---- scale factor
[dummy]=fgetl(fid);
[scale]=fscanf(fid,'%f\n',"C");
%[scale_par,scale_dpar]=fscanf(fid,'%s  %s\n',"C");
[scale_par]=fscanf(fid,'%s  %s\n',"C");

genpar.lambda=lambda;
genpar.zach=zach;
genpar.zach_par=zach_par;
%genpar.zach_dpar=zach_dpar;
genpar.scale=scale;
genpar.scale_par=scale_par;
%genpar.dscale=scale_dpar;

%---- lattice parameters and angles
[dummy]=fgetl(fid);
[as,bs,cs,aa,bb,cc,count]=fscanf(fid,'%f %f %f %f %f %f\n',"C");
if (count ~= 6)
	error('wrong input for lattice');
endif

[Mcryst2cart,Mrec2cart]=lat2cart(as,bs,cs,aa,bb,cc);

%---- number atoms
[dummy]=fgetl(fid);
[nr_at, Btype, count]=fscanf(fid,'%f %f\n',"C");

if (count ~= 2)
	error('wrong input number of atoms of B-factor type');
endif
ion_cell{1}=nr_at;
lat.as=as;
lat.bs=bs;
lat.cs=cs;
lat.aa=aa;
lat.bb=bb;
lat.cc=cc;
ion_cell{2}=lat;

%---- ion type, scattering length, magnetic moment, position, occupation and B-factor
%---- (iso/anisotropic)
for i=1:nr_at
	text=fgetl(fid);
	[ion, scat_length, mx, my, mz, x, y, z, occ, B,u11, u22, u33, u12, u13, u23,count]=...
         sscanf(text,'%s %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f\n',"C");
	if (count ~= 16)
		disp(text);
		error('wrong input close to:');
	endif
	at.ion=ion;at.bd=scat_length;
        at.sx=mx;
        at.sy=my;
        at.sz=mz;
        at.x=x;at.y=y;at.z=z;
	at.B=B;at.u11=u11;at.u22=u22;at.u33=u33;at.u12=u12;at.u13=u13;at.u23=u23;
	at.occ=occ;
	text=fgetl(fid);
        [sx_par, sy_par, sz_par, x_par, y_par, z_par, B_par, occ_par,...
         u11_par, u22_par, u33_par, u12_par, u13_par,u23_par, count]=...
         sscanf(text,'%s %s %s %s %s %s %s %s %s %s %s %s %s %s\n',"C");
	if (count ~= 14)
		disp(text);
		error('wrong input close to atomic parameters');
	endif
        at.sx_par=sx_par;
        at.sy_par=sy_par;
        at.sz_par=sz_par;
	at.x_par=x_par;at.y_par=y_par;at.z_par=z_par;
	at.B_par=B_par;at.u11_par=u11_par;at.u22_par=u22_par;at.u33_par=u33_par;at.u12_par=u12_par;at.u13_par=u13_par;at.u23_par=u23_par;
	at.occ_par=occ_par;
        %---- add dummy values for unused parameters
        at.radius=0;
        at.red=0;
        at.blue=0;
        at.green=0;
	ion_cell{i+2}=at;
endfor
ion_cell{columns(ion_cell)+1}.Btype=Btype;

%---- number of symmetry operations
[dummy]=fgetl(fid);
[nr_sym, count]=fscanf(fid,'%f\n',"C");

if (count ~= 1)
	error('wrong input number of symmetry operations');
endif
spgsym_cell{1}=nr_sym;

%---- symmetry operations
for i=1:nr_sym
	text=fgetl(fid);
	[x, y, z,count]=sscanf(text,'%s %s %s',"C");
	if (count ~= 3)
		disp(text);
		error('wrong input\n');
	endif
	spg.x=x;spg.y=y;spg.z=z;
        %---- add symmetries for a ferromagnet
        spg.rmx='RMx';
        spg.rmy='RMy';
        spg.rmz='RMz';
        spg.imx='IMx';
        spg.imy='IMy';
        spg.imz='IMz';
        spg.phase=0;
	spgsym_cell{i+1}=spg;
endfor

#do not use translations here
spgtrans_cell{1}=1;
spgtrans.x=0;spgtrans.y=0;spgtrans.z=0;
spgtrans_cell{2}=spgtrans;

%---- Int (0)/ fsqr (1) 
%---- not needed
singlecrys{1}.intfsqr=0;

%---- UB matrix
[dummy]=fgetl(fid);
UB=zeros(3,3);
for i=1:3
   line=fgetl(fid);
   UB(i,1:3)=str2num(line);
endfor

%---- Polarisation degree
[dummy]=fgetl(fid);
text=fgetl(fid);
[p]=sscanf(text,'%f\n',"C");

%---- loop over HKL until end of file is reached
singlecrys{2}=0;
nr=3;
[dummy]=fgetl(fid);
while (1)
        if ((test=feof(fid))==1) %leaves when end-of-file reached
                fclose(fid);
                break;
        endif

        text=fgetl(fid);
        [H,K,L,nu,count]=sscanf(text,'%i %i %i %f \n',"C");
        if count != 4
            error('input should be: H K L nu (normal beam geometry)')
        endif
        %---- Q-vector
        hhkl=(Mrec2cart*[H; K; L])';
	singlecrys{nr}.hhkl_rlu=[H K L];
	singlecrys{nr}.hhkl_AA=hhkl;
	singlecrys{nr}.nu=nu;
        nr++;
endwhile
singlecrys{2}=nr-3;
endfunction
