## Copyright (C) 2007,2008,2009 Bertrand Roessli <bertrand.roessli@psi.ch>
##
##
## Octave is free software; you can redistribute it and/or
## modify it under the terms of the GNU General Public
## License as published by the Free Software Foundation;
## either version 2, or (at your option) any later version.
##
## Octave is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied
## warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
## PURPOSE.  See the GNU General Public License for more
## details.
##
## You should have received a copy of the GNU General Public
## License along with Octave; see the file COPYING.  If not,
## write to the Free Software Foundation, Inc., 51 Franklin Street,
## Fifth Floor, Boston, MA 02110-1301, USA.
##

## -*- texinfo -*-
## @deftypefn{Function File}  readnuc
## @deftypefnx{Function File} {[@var{genpar},@var{as},@var{bs},@var{cs},@var{aa},@var{bb},@var{cc},@var{ion_cell},@var{spgsym_cell},@var{spgtrans_cell},@var{singlecrys},    
## @var{parameters},@var{dparameters},@var{nfit_type},@var{simplex_nuc_options},@var{lmfitnuc_options},@var{nsim_ann_par}]} = readnuc
## @* Read parameters from file 'nuc.inp' fir nuclear fit (see ./examples/nuc.inp)
##
## @*output:
##        @*1.-: @var{genpar} :wavelength, extinction, scale factor
##        @*2.-: @var{as},@var{bs},{...}, @var{cc}: cell parameters
##        @*3.-: @var{ion_cell}: atomic paramters
##        @*4.-: @var{spgsgsym_cell}: space group symmetry elements
##        @*5.-: @var{spgtrans_cell}: space group translations
##        @*6.-: @var{singlecrys}: switch structure factor/intensities and list of hkl with intensities and standard deviations
##        @*7.-: @var{nfit_type}: switch to select fit algorithm (0/1/2)
##        @*8.-: @var{simplexnuc_options}: simplex options
##        @*9.-: @var{lmfitnuc_options}: levenberg-marquard options
##        @*10.-: @var{nsim_ann_par}: simulated annealing options
##        @*11.-: @var{draw_nuc_opt}: vrml crystal structure options 

function [genpar,as,bs,cs,aa,bb,cc,ion_cell,spgsym_cell,spgtrans_cell,twins,singlecrys,...
	 parameters,dparameters,...
	 nfit_type,simplexnuc_options,lmfitnuc_options,nsim_ann_par,...
	 draw_nuc_opt]=readnuc;


%---- import from file 'nuc.inp' information
[fid]=fopen('nuc.inp','rt');

%---- wavelength
[dummy]=fgetl(fid);
[lambda]=fscanf(fid,'%f\n',"C");

%---- extinction
[dummy]=fgetl(fid);
[zach]=fscanf(fid,'%f \n',"C");
%[zach_par,zach_dpar]=fscanf(fid,'%s  %s\n',"C");
[zach_par]=fscanf(fid,'%s  %s\n',"C");

%---- scale factor
[dummy]=fgetl(fid);
[scale]=fscanf(fid,'%f\n',"C");
%[scale_par,scale_dpar]=fscanf(fid,'%s  %s\n',"C");
[scale_par]=fscanf(fid,'%s  %s\n',"C");

genpar.lambda=lambda;
genpar.zach=zach;
genpar.zach_par=zach_par;
%genpar.zach_dpar=zach_dpar;
genpar.scale=scale;
genpar.scale_par=scale_par;
%genpar.dscale=scale_dpar;

%---- lattice parameters and angles
[dummy]=fgetl(fid);
[as,bs,cs,aa,bb,cc,count]=fscanf(fid,'%f %f %f %f %f %f\n',"C");
if (count ~= 6)
	error('wrong input for lattice');
endif

[Mcryst2cart,Mrec2cart]=lat2cart(as,bs,cs,aa,bb,cc);

%---- number atoms
[dummy]=fgetl(fid);
[nr_at, Btype, count]=fscanf(fid,'%f %f\n',"C");

if (count ~= 2)
	error('wrong input number of atoms of B-factor type');
endif
ion_cell{1}=nr_at;
lat.as=as;
lat.bs=bs;
lat.cs=cs;
lat.aa=aa;
lat.bb=bb;
lat.cc=cc;
ion_cell{2}=lat;

%---- ion type, scattering length position, B-factor and occupation
for i=1:nr_at
	text=fgetl(fid);
	[ion, scat_length, x, y, z, B, occ, u11, u22, u33, u12, u13, u23, radius, red, green, blue, count]=...
	sscanf(text,'%s %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f\n',"C");
	if (count ~= 17)
		disp(text);
		error('wrong input close to:');
	endif
	at.ion=ion;at.bd=scat_length;at.x=x;at.y=y;at.z=z;
	at.B=B;at.u11=u11;at.u22=u22;at.u33=u33;at.u12=u12;at.u13=u13;at.u23=u23;
	at.occ=occ;
	at.radius=radius; at.red=red; at.green=green; at.blue=blue;
	text=fgetl(fid);
        [x_par, y_par, z_par, B_par, occ_par,...
	 u11_par, u22_par, u33_par, u12_par, u13_par,u23_par, count]=sscanf(text,'%s %s %s %s %s %s %s %s %s %s %s\n',"C");
	if (count ~= 11)
		disp(text);
		error('wrong input close to atomic parameters');
	endif
	at.x_par=x_par;at.y_par=y_par;at.z_par=z_par;
	at.B_par=B_par;at.u11_par=u11_par;at.u22_par=u22_par;at.u33_par=u33_par;at.u12_par=u12_par;at.u13_par=u13_par;at.u23_par=u23_par;
	at.occ_par=occ_par;
	ion_cell{i+2}=at;
endfor
ion_cell{columns(ion_cell)+1}.Btype=Btype;

%---- number of symmetry operations
[dummy]=fgetl(fid);
[nr_sym, count]=fscanf(fid,'%f\n',"C");

if (count ~= 1)
	error('wrong input number of symmetry operations');
endif
spgsym_cell{1}=nr_sym;

%---- symmetry operations
for i=1:nr_sym
	text=fgetl(fid);
	[x, y, z,count]=sscanf(text,'%s %s %s',"C");
	if (count ~= 3)
		disp(text);
		error('symmetry operator; wrong input, requires 3 operations\n');
	endif
	spg.x=x;spg.y=y;spg.z=z;
	spgsym_cell{i+1}=spg;
endfor

%---- number of TRANSLATIONS
[dummy]=fgetl(fid);
[nr_transl, count]=fscanf(fid,'%f\n',"C");

if (count ~= 1)
	error('wrong input number of translations');
endif
spgtrans_cell{1}=nr_transl;

%---- symmetry operations
for i=1:nr_transl
	[x, y, z,count]=fscanf(fid,'%s %s %s\n',"C");
	if (count ~= 3)
		error('wrong input with translation operations');
	endif
	spgtrans.x=str2num(x);spgtrans.y=str2num(y);spgtrans.z=str2num(z);
	spgtrans_cell{i+1}=spgtrans;	
endfor

%---- twins
[dummy]=fgetl(fid);
nr_twins=fscanf(fid,"%f\n","C");
for i=1:nr_twins
 	text=fgetl(fid);
	[raa,rab,rac,rba,rbb,rbc,rca,rcb,rcc,count]=sscanf(text,'%f %f %f %f %f %f %f %f %f\n',"C");
	if (count ~= 9) 
   		error('wrong input with twin operation');
	endif
	twins{1,i}.rot=[raa rab rac; rba rbb rbc; rca rcb rcc]';
endfor

%---- read draw structure options
[dummy]=fgetl(fid);
[dummy]=fgetl(fid);
[dummy]=fgetl(fid);
nr_draw_at=fscanf(fid,"%f\n","C");
i=0;
for i=1:nr_draw_at
	text=fgetl(fid);
	[label1, label2, mindist, maxdist, r, g, b, rad,counts]=...
	sscanf(text,'%s %s %f %f %f %f %f %f \n',"C");
	if (counts != 8)
		error('vrml-bond parameters != 8');
	endif	
	draw_nuc_opt{1,i}.label=strcat(label1,'-',label2);
	draw_nuc_opt{1,i}.mindist=mindist;
	draw_nuc_opt{1,i}.maxdist=maxdist;
	draw_nuc_opt{1,i}.color=[r g b];
	draw_nuc_opt{1,i}.radius=rad;
endfor
text=fgetl(fid);
[unitcell, style, counts]=sscanf(text,'%f %f \n',"C");
if (counts != 2)
	error('vrm-flag unitcell or style  missing');
endif
draw_nuc_opt{1,i+1}.unitcell=unitcell;
draw_nuc_opt{1,i+1}.style=style;

%---- read parameters to fit atomic parameters
parameters=[];
dparameters=[];
nr_param=0;
for k=1:5
	%---- read parameters
	[dummy]=fgetl(fid);
	[par,count]=fscanf(fid,'%f');
	if (count != 10)
		disp(par);
		disp(count);
		error('number of parameters per line not 10');	
	endif
	parameters=[parameters;par];
	nr_param=nr_param.+count;
	%read parameters errors
	[dummy,counts]=fgetl(fid);
	if (count != 10)
		error('number of dpar per line not 10');	
	endif
	[dpar]=fscanf(fid,'%f');
	dparameters=[dparameters;dpar];
endfor


%---- switch for fit alogorithm
[dummy]=fgetl(fid);
[nfit_type, count]=fscanf(fid,'%f\n',"C");
if (count != 1)
        error('input should be fit algorithm');
endif

%-----  read levenberg-marquard fit options tol, niter, etc...
[dummy]=fgetl(fid);
[stol,niter, verbose, count]=fscanf(fid,'%f %f %f\n',"C");
if (count != 3) 
        error('input should be niter and stol');
endif
lmfitnuc_options.verbose=verbose;
lmfitnuc_options.stol=stol;
lmfitnuc_options.niter=niter;

%-----  read leasqr fit options tol, niter, etc...
[dummy]=fgetl(fid);
[stol,niter, fval, simplex_sides, verbose, count]=fscanf(fid,'%f %f %f %f %f',"C");
if (count != 5)
        error('input should be simplex fit options');
endif
simplexnuc_options.stol=stol;
simplexnuc_options.niter=niter;
simplexnuc_options.fval=fval;
simplexnuc_options.sides=simplex_sides;
simplexnuc_options.verbose=verbose;

% read simulated annealing options
#{
        * LB  - vector of lower bounds
        * UB - vector of upper bounds 
        * nt - integer: # of iterations between temperature reductions
        * ns - integer: # of iterations between bounds adjustments
        * rt - (0 < rt <1): temperature reduction factor
        * maxevals - integer: limit on function evaluations
        * neps - integer:  number of values final result is compared to
        * functol -   (> 0): the required tolerance level for function value
        * comparisons
        * paramtol -  (> 0): the required tolerance level for parameters
        * verbosity - scalar: 0, 1, or 2.
                * 0 = no screen output   
                * 1 = only final results to screen
                * 2 = summary every temperature change
#}
for i=1:9
[dummy]=fgetl(fid);
endfor

[nt,ns,rt,maxevals,neps,functol,paramtol,verbosity,counts]=fscanf(fid,'%f %f %f %f %f %f %f %f\n',"C");
if (counts != 8)
	error('number of simulated annealing options wrong')
endif
nsim_ann_par.nt=nt;
nsim_ann_par.ns=ns;
nsim_ann_par.rt=rt;
nsim_ann_par.maexevals=maxevals;
nsim_ann_par.neps=neps;
nsim_ann_par.functol=functol;
nsim_ann_par.paramtol=paramtol;
nsim_ann_par.verbosity=verbosity;


%---- Int (0)/ fsqr (1) 
[dummy]=fgetl(fid);
[intfsqr,count]=fscanf(fid,'%f\n',"C");
singlecrys{1}.intfsqr=intfsqr;
%title line
dummy=fgetl(fid);


%---- loop over HKL until end of file is reached
singlecrys{2}=0;
nr=3;
while (1)
        if ((test=feof(fid))==1) %leaves when end-of-file reached
                fclose(fid);
                break;
        endif

        text=fgetl(fid);
        [ptype,H,K,L,f2obs,f2sd,count]=sscanf(text,'%s %f %f %f %f %f\n',"C");
        %---- Q-vector
        hhkl=(Mrec2cart*[H; K; L])';
	singlecrys{nr}.type=ptype;
	singlecrys{nr}.hhkl_rlu=[H K L];
	singlecrys{nr}.hhkl_AA=hhkl;
	singlecrys{nr}.f2obs=f2obs;
	singlecrys{nr}.f2sd=f2sd;	
        nr++;
endwhile
singlecrys{2}=nr-3;

endfunction
