## Copyright (C) 2007,2008,2009 Bertrand Roessli <bertrand.roessli@psi.ch>
##
##
## MuFit is free software; you can redistribute it and/or
## modify it under the terms of the GNU General Public
## License as published by the Free Software Foundation;
## either version 2, or (at your option) any later version.
##
## Mufit is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied
## warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
## PURPOSE.  See the GNU General Public License for more
## details.
##
## You should have received a copy of the GNU General Public
## License along with Octave; see the file COPYING.  If not,
## write to the Free Software Foundation, Inc., 51 Franklin Street,
## Fifth Floor, Boston, MA 02110-1301, USA.
##

## -*- texinfo -*-
## @deftypefn{Function File}  sigma_tot
## (@var{f1},@var{f2},@var{f3},@var{f4},@var{poln_cs},@var{P_1},@var{P_2})
## @deftypefnx{Function File} {[@var{Intensity}]}=sigma_tot
## (@var{f1},@var{f2},@var{f3},@var{f4},@var{poln_cs},@var{P_1},@var{P_2})
## Calculate polarised neutron intensity taking into account the efficiency
## of the polarizers
##
## @*input:
##        @var{f1-->f4}: incident and scattered neutron spin state
##        @var{poln_cs}: neutron interaction potential
##        @var{P_1} and @var{P_1}: polariser efficience 0<=P<=1
##
## @*output:
##        @var{Intensity}: polarised neutron intensity
## @end deftypefn


function [Intensity]=sigma_tot(f1,f2,f3,f4,poln_cs,P_1,P_2)


%---- last change 30/06/09 following a comment of Amy Poole

Intensity=(f1'*poln_cs'*f2)*(f2'*poln_cs*f1).*P_1.*P_2+...
          (f1'*poln_cs'*f4)*(f4'*poln_cs*f1).*P_1.*(1.-P_2).+...
          (f3'*poln_cs'*f2)*(f2'*poln_cs*f3).*(1-P_1).*P_2.+...
          (f3'*poln_cs'*f4)*(f4'*poln_cs*f3).*(1-P_1).*(1.-P_2);

Intensity=real(Intensity);


endfunction

% Intensity can also be calculated as 

%Intensity=(f2'*poln_cs*f1)'*(f2'*poln_cs*f1).*P_1.*P_2+...
%          (f4'*poln_cs*f1)'*(f4'*poln_cs*f1).*P_1.*(1.-P_2).+...
%          (f2'*poln_cs*f3)'*(f2'*poln_cs*f3).*(1-P_1).*P_2.+...
%          (f4'*poln_cs*f3)'*(f4'*poln_cs*f3)*(1-P_1).*(1.-P_2)

%or
% a=(transpose(conj(f2))*poln_cs*f1)
% b=conj(a)*a
% Intensity=b*P_1*P_2+...


