## Copyright (C) 2007,2008,2009 Bertrand Roessli <bertrand.roessli@psi.ch>
##
##
## MuFit is free software; you can redistribute it and/or
## modify it under the terms of the GNU General Public
## License as published by the Free Software Foundation;
## either version 2, or (at your option) any later version.
##
## Mufit is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied
## warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
## PURPOSE.  See the GNU General Public License for more
## details.
##
## You should have received a copy of the GNU General Public
## License along with Octave; see the file COPYING.  If not,
## write to the Free Software Foundation, Inc., 51 Franklin Street,
## Fifth Floor, Boston, MA 02110-1301, USA.
##

function [U,trans]=sym2mat(trs)
%decode symmetry operation and transform to rotation and translation parts
%input: symmetry operation as read from file symm.op
%output: rotational (vec_1,vec_2,vec_3) and translational (trans) part
%broessli@psi.ch: 13.11.2008

%translation part	
a1=trs.X;a2=trs.Y;a3=trs.Z;
b1=a1(max([rindex(a1,"X"),rindex(a1,"Y"),rindex(a1,"Z")])+1:length(a1));
b2=a2(max([rindex(a2,"X"),rindex(a2,"Y"),rindex(a2,"Z")])+1:length(a2));
b3=a3(max([rindex(a3,"X"),rindex(a3,"Y"),rindex(a3,"Z")])+1:length(a3));
if (length(b1)==0)
  b1='0';
endif
if (length(b2)==0)
  b2='0';
endif
if (length(b3)==0)
  b3='0';
endif
trans=[eval(b1);eval(b2);eval(b3)];
%rotation part
rtrs.X=[trs.X];
pos1 = index(a1,b1);
if (pos1>1)
  rtrs.X=rtrs.X(1:pos1-1);
endif
rtrs.Y=[trs.Y];
pos2 = index(a2,b2);
if (pos2>1)
  rtrs.Y=rtrs.Y(1:pos2-1);
endif
rtrs.Z=[trs.Z];
pos3 = index(a3,b3);
if (pos3>1)
  rtrs.Z=rtrs.Z(1:pos3-1);
endif
vec_1=vec_2=vec_3=zeros(1,3);
if strcmp(rtrs.X, '+X') vec_1=[1 0 0]; endif;
if strcmp(rtrs.X, '-X') vec_1=[-1 0 0]; endif;
if strcmp(rtrs.X, '+Y') vec_1=[0 1 0]; endif;
if strcmp(rtrs.X, '-Y') vec_1=[0 -1 0]; endif;
if strcmp(rtrs.X, '+Z') vec_1=[0 0 1]; endif;
if strcmp(rtrs.X, '-Z') vec_1=[0 0 -1]; endif;
if strcmp(rtrs.Y, '+X') vec_2=[1 0 0]; endif;
if strcmp(rtrs.Y, '-X') vec_2=[-1 0 0]; endif;
if strcmp(rtrs.Y, '+Y') vec_2=[0 1 0]; endif;
if strcmp(rtrs.Y, '-Y') vec_2=[0 -1 0]; endif;
if strcmp(rtrs.Y, '+Z') vec_2=[0 0 1]; endif;
if strcmp(rtrs.Y, '-Z') vec_2=[0 0 -1]; endif;
if strcmp(rtrs.Z, '+X') vec_3=[1 0 0]; endif;
if strcmp(rtrs.Z, '-X') vec_3=[-1 0 0]; endif;
if strcmp(rtrs.Z, '+Y') vec_3=[0 1 0]; endif;
if strcmp(rtrs.Z, '-Y') vec_3=[0 -1 0]; endif;
if strcmp(rtrs.Z, '+Z') vec_3=[0 0 1]; endif;
if strcmp(rtrs.Z, '-Z') vec_3=[0 0 -1]; endif;
%
if strcmp(rtrs.X, '-X+Y') vec_1=[-1 1 0]; endif;
if strcmp(rtrs.X, '+X-Y') vec_1=[ 1 -1 0]; endif;
if strcmp(rtrs.Y, '-X+Y') vec_2=[-1 1 0]; endif;
if strcmp(rtrs.Y, '+X-Y') vec_2=[ 1 -1 0]; endif;
%
if (any(vec_1)==0 | any(vec_2)==0 | any(vec_3)==0) 
  disp('***format should be +X+1/2 or -X-1/2, etc***');
  disp('***or***');
  disp('***add symmetry element in symmop.m***');
  error("symmetry operation not yet defined\n"); 
endif;
U = [vec_1' vec_2' vec_3'];
endfunction
