## Copyright (C) 2009 Bertrand Roessli <bertrand.roessli@psi.ch>
##
##
## MuFit is free software; you can redistribute it and/or
## modify it under the terms of the GNU General Public
## License as published by the Free Software Foundation;
## either version 2, or (at your option) any later version.
##
## Mufit is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied
## warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
## PURPOSE.  See the GNU General Public License for more
## details.
##
## You should have received a copy of the GNU General Public
## License along with Octave; see the file COPYING.  If not,
## write to the Free Software Foundation, Inc., 51 Franklin Street,
## Fifth Floor, Boston, MA 02110-1301, USA.
##
## usage: mufit
##
## Author: Bertrand Roessli <bertrand.roessli@psi.ch>

## Inspired from vrml_ellipsoid.m (www.octavforge.net)
## Author:        Etienne Grossmann <etienne@isr.ist.utl.pt>


function v = vrml_ellipsoid(moment, col, style)

if (isempty (col) ~= 0) 
  col = [0.3 0.4 0.9];
endif

col = col';


switch style
  case 0  %---- balls
	d = moment*ones(3,1);
	r = [0 0 0];
	a = 0;
  case 1  %---- thermal ellipsoids
	[u,l,A] = svd(moment);
        %--- check that A is a rotation matrix
	if (diag(A*transpose(A)-ones(3,3)) < 0.01*[1;1;1] & det(A)>0)
		d = sqrt(diag(l));
		fprintf(stdout,"RMS: %f, %f, %f;  U_eq = %f \n",sqrt(l(1,1)),sqrt(l(2,2)),sqrt(l(3,3)),1/3*sum(d));
	        %---- find rotation axis and rotation angle (in radians)
		[V,D] = eig(A);
	        [ignore,ix] = min(abs(diag(D)-1));
		w = V(:,ix);
		t = [A(3,2)-A(2,3),A(1,3)-A(3,1),A(2,1)-A(1,2)];
		theta = atan2(t*w,trace(A)-1);
		if theta<0, theta = -theta; w = -w; end
		r = w;
		a = theta;
		d = 1.5382*d;  %---- 50 % probability
	else
		disp("*ellispoid not positiv definite");
	        d = sqrt(abs(diag(l)));
		r = [0 0 0];
		a = 0;
		d = 1.5382*d  
	endif
  otherwise
        error("Allowed values for atom style 0 (Balls) / 1 (Ellipsoids)");
endswitch

v = sprintf (["Transform {\n",\
              "  translation  0 0 0\n",\
	      "  rotation     %8.3g %8.3g %8.3g %8.3g\n",\
	      "  scale        %8.3g %8.3g %8.3g\n",\
              "  children [\n",\
              "    Shape {\n",\
              "      appearance  Appearance {\n",\
              "        material Material {\n",\
	      "          diffuseColor  %8.3g %8.3g %8.3g\n",\
	      "          emissiveColor %8.3g %8.3g %8.3g\n",\
	      "        }\n",\
              "      }\n",\
              "      geometry Sphere {}\n",\
              "    }\n",\
              "  ]\n",\
              "}\n",\
              ],\
	     r,a,\
	     d,\
	     col,\
	     col/20);

endfunction

