function syntaxcheck(cmnd)

if (nargin ~= 1) error("expected command at input line\n");endif;
SAM = [7.4 7.4 20.6 90 90 120 -1 0 0 0 -1 0];
Kf=1.4
fprintf(stdout, "lattice parameters\n");
fprintf(stdout, "------------------\n");
fprintf(stdout, "as = %f, bs = %f,cs = %f, aa = %f, bb = %f, cc = %f\n\n",
                 SAM(1:6));
fprintf(stdout, "scattering plane\n");
fprintf(stdout, "----------------\n");
fprintf(stdout, "AX = [%f, %f, %f]\n", SAM(7:9));
fprintf(stdout, "BX = [%f, %f, %f]\n\n", SAM(10:12));

% Ki, Kf and scattering sense
DM = 3.354; DA = 3.354;
fprintf(stdout, "dm = %f, da = %f\n", DM, DA); 
ISX = -1;
EF = 2.072*Kf*Kf;
fprintf(stdout, "EF = %f\n", EF);
SS =  1;      

%cmnd = 'sc qh 2 0 0 0 dqh 0 0 0 1 np 3 mn 2000'
%cmnd = 'sc a1 0 a2 40 a3 -14.2 da1 0.1 da2 0.2 da3 0.0 np 3 mn 5000'
%cmnd = 'sc a1 20 da1 0.1 np 21 mn5000'
%cmnd = 'muco x -x'

[tok,remain] = strtok(cmnd);

switch tok
  case "se"
    [tok, remain] = strtok(remain);
    switch tok
       case {"dm", "da", "as", "bs", "cs", "aa", "bb", "cc"}
          fprintf("OK\n");
          if (isempty(remain) && remain == " ") error("nothing has bee set\n");endif;
          [tok, remain] = strtok(remain);
          remain = deblank(remain);
          if (!isempty(remain)) 
            error("trailing mistake in 'se' command\n"); 
          endif;
          if (isempty(str2num(tok)))
            error("value is not a number\n");
          endif;
       case {"title", "user", "local"}
       case {"fx"}
          [tok,remain] =strtok(remain);
          if ((str2num(tok) != 0) && (str2num(tok) != 1) && (str2num(tok) != 2))
             error("fx can be only 0, 1 or 2\n");
          endif
       otherwise
          errmsg = [tok, "cannot be set\n"];
          error("errmsg");
      endswitch
  case "dr"
    [tok, remain] = strtok(remain);
    switch tok
      case {"a1" "a2" "a3"  "a4" "a5" "a6" "sgl" "sgu"}
          fprintf("OK\n");
          if (isempty(remain) && remain == " ") error("nothing has been set\n");endif;
          [tok, remain] = strtok(remain);
          remain = deblank(remain);
          if (!isempty(remain)) 
            error("trailing mistake in 'dr' command\n"); 
          endif;
          if (isempty(str2num(tok)))
            error("value is not a number\n");
          endif;
      case "qh"
          if (isempty(remain) && remain == " ") error("nothing has been set\n");endif;
          QE = str2num(remain);      
          if (columns(QE) != 4) error("'dr qh' need 4 parameters: H K L E\n"); endif;
          %dr qh == sc qh np 1
          newcmnd = ['qh', remain, ' dqh 0 0 0 0 ' , ' np 1', ' mn 10000'];
          [name, motorpos, steps, np, mn] = sccmnd(newcmnd);
          calc_hklen_angles(SAM, EF, SS, DM, ISX, name,motorpos,steps,np,mn);                   
      case "en"
          if (isempty(remain) && remain == " ") error("nothing has been set\n");endif;
          EN = str2num(remain);      
          if (columns(EN) != 1) error("'dr en' needs energy transfer as  parameter\n"); endif;
      otherwise
        errmsg = ["'", tok, "'", " cannot be driven\n"];
        error(errmsg);
    endswitch  
  case "sc"
    [name, motorpos, steps, np, mn] = sccmnd(remain);
    switch name
       case "qh"
         calc_hklen_angles(SAM, EF, SS, DM, ISX, name,motorpos,steps,np,mn);         
       case {"a1" "a2" "a3"  "a4" "a5" "a6" "sgl" "sgu"}
       otherwise
         disp("not implemeted yet");
    endswitch
  otherwise
    errmsg = [tok, ": not recognised yet or not allowed\n"]; 
    error(errmsg);
endswitch
endfunction

function [name, motorpos, steps, np, mn] = sccmnd(cmnd)

  [tok,remain] = strtok(cmnd);
  name = tok;
  switch tok
    case "qh"
      [tok,remain] = strtok(remain,'dqh');
      motorpos = str2num(tok);
      if (tok(end) ~= ' ') error("space missing"); endif;
      l = size(str2num(tok));
      if (l(2) != 4) errmsg = ["syntax error before: '", remain, "'\n"]; error(errmsg); endif;
      [steps, np, mn] = scsteps(remain,name);
    case {"a1" "a2" "a3"  "a4" "a5" "a6", "sgl", "sgu"}
      % have to consider the possiblity that many angles are scanned
      % simultaneously
      [tok, remain] = strtok(remain,'d');
      motorpart = [name tok];
      [tok, remain] = strtok(remain, 'np');
      steppart = tok;
      k = 1;
      while(!isempty(motorpart))
         [tok,motorpart] = strtok(motorpart);
         if (tok != " ") motors{k,1} = tok; ++k; endif;
      endwhile

      k = 1;
      while(!isempty(steppart))
         [tok,steppart] = strtok(steppart);
         if (tok != " ") stepsmotors{k,1} = tok; ++k; endif;
      endwhile
      % possible errors
      if (rem(rows(motors),2) != 0 || rem(rows(stepsmotors),2) != 0) 
        error("syntax error\n");
      endif;
      if (rows(motors) ~= rows(stepsmotors)) error("number of motors different from steps"); endif;
      for i=1:2:rows(motors)
         if (isempty(strfind(stepsmotors(i,:),motors(i,:)))) error("motors and steps do not corespond"); endif;
      endfor
      k = 1;
      for i = 1:2:rows(stepsmotors)
        output{k} = motors(i,:);
        ++k;
      endfor

      output{1} = "start:  ";
      output{2} = "middle: ";
      output{3} = "end:    ";
      k = 1;
      for i = 1:2:rows(stepsmotors)
         sccmnd = [stepsmotors{i,:} " " stepsmotors{i+1,:} " " remain];
         motorname{k,1} = motors{i,:};
         motorpos = str2num(motors{i+1,:});
         [steps, np, mn] = scsteps(sccmnd,motorname{k,1})
         label{k,1} = toupper(motorname{k,1});
         [output] = calcangles(output, motorname{k}, motorpos, steps, np, mn);
         ++k;
      endfor
      title = "           ";
      for i= 1:rows(label)
        title = cstrcat(title, label{i,1}, "         ");
      endfor
      fprintf("%s\n", title);
      if (np == 1) %% drive case
         fprintf("%s\n", output{2,:});
      else
         for i = 1:rows(output)
            fprintf("%s\n", output{i,:});
         endfor
      endif
    otherwise
      error("syntax error\n");
  endswitch
endfunction

function [steps, np, mn] = scsteps(cmnd,name)
  [tok,remain] = strtok(cmnd);
  if (isempty(strfind(tok,name))) 
     error("variable name and step do not match\n");  
  endif
  switch tok
    case "dqh"
      [tok, remain] = strtok(remain,'np');
      if (isempty(remain)) error("number of points is required\n"); endif;
      if (tok(end) ~= " ") error("space missing before 'np'\n"); endif;
      steps = str2num(tok);
      l = size(str2num(tok));
      if (l(2) != 4) errmsg = ["syntax error before '", remain, "'\n"]; error(errmsg); endif;
      if (~isempty(remain)) [np,mn] = npoints(remain); endif;
    case {"da1" "da2" "da3"  "da4" "da5" "da6", "dsgl" "dsgu"}
      [tok, remain] = strtok(remain,'np');
      if (isempty(remain)) error("number of points is required\n"); endif;
      if (tok(end) ~= " ") error("space missing before 'np'\n"); endif;
      steps = str2num(tok);
      l = size(str2num(tok));
      if (l(2) != 1) errmsg = ["syntax error before '", remain, "'\n"]; error(errmsg); endif;
      if (~isempty(remain)) [np,mn] = npoints(remain); endif;
    otherwise
      error("unknoown step variable\n");
  endswitch
endfunction

function [np,mn] = npoints(cmnd)
  [tok,remain] = strtok(cmnd);
  switch tok
    case "np"
      [tok, remain] = strtok(remain,'mn');
      if (isempty(remain)) error("monitor is required\n"); endif;     
      if (tok(end) ~= " ") error("space missing before 'mn'\n"); endif;
      np = str2num(tok);
      if (np < 0) error("np cannot be negative\n"); endif
      l = size(str2num(tok));
      if (l(2) != 1) errmsg = ["expecting number of points\n"]; error(errmsg); endif
      if (~isempty(remain)) [mn] = monitor(remain); endif;
    otherwise
      error("np is missing or syntax error\n");
  endswitch
endfunction

function [mn] = monitor(cmnd)
  [tok,remain] = strtok(cmnd);
  if (isempty(deblank(remain))) error("monitor value expected\n"); endif;
  
  switch tok
    case "mn"
      [tok,remain] = strtok(remain,"\n");
      if (~isempty(remain)) error("unexpected caracter after monitor\n"); endif;
      mn = str2num(tok);
      if (isempty(mn)) error("monitor value is not a number\n"); endif;
    otherwise
      error("syntax error, expected 'mn value'\n");
  endswitch
endfunction

function calc_hklen_angles(sam, ef, ss, dm, isx, name, motorpos, steps, np, mn)
  k =1;
  if (np == 1)
     scanpos(1,:) = motorpos;
  else
     for i = -1:1
        scanpos(k,:) = motorpos .+ i.*steps.*(np-1)/2;
        ++k;
     endfor
  endif
  switch name
    case "qh"
      npoints = size(scanpos);
      fprintf("          H       K       L       EN       A1       A2       A3       A4      KI      KF\n")
      if (np == 1)
         label = "target";
      else
         label = ["start"; "middle"; "end"];
      endif
      for i = 1:npoints(1)
        [A3, A4, KI, KF] = t_rlp(sam, scanpos(i,:), ef, ss);
        [A1, A2] = EX_CASE(dm,isx,KI);  
        fprintf("%s:  %+2.3f  %+2.3f  %+2.3f  %+2.3f  %+2.3f  %+2.3f  %2.3f  %2.3f  %2.3f  %2.3f\n",\
                 label(i,1:end), scanpos(i,1:4), A1, A2, A3, A4, KI, KF);
      endfor
    otherwise
      disp("some unknown error occured");
  endswitch
endfunction


function [output] = calcangles(output, name, motorpos, steps, np, mn)
  k =1;
  for i = -1:1
    scanpos(k,:) = motorpos .+ i.*steps.*(np-1)/2;
    ++k;
  endfor

  switch name
    case {"a1" "a2" "a3"  "a4" "a5" "a6" "sgl" "sgu"}
      npoints = size(scanpos);
        for i = 1:npoints(1)
          text = sprintf( " %+2.3f   ",scanpos(i,1));
          output{i} = cstrcat(output{i}, text);
        endfor
    otherwise
      disp("some unknown error occured");
  endswitch

endfunction
