function [AX3, AX4, AKI, AKF ] = t_rlp(SAM, HKLEN, EF, SS)
%
% calculates scattering angles for TAS using ILL
% convention for the definition of the scattering plane
% taken from TasMad routines
% lattice constants, angles, AX, BX
%

if (nargin != 4)
 SAM = [4.0 4.0 4.0 90 90 90 1 0 0 1 1 0];
% Ki, Kf and scattering sense
 AKI = 2.662;
 AKF = 2.662;
 EI = AKI^2*2.072;
 EF = AKF^2*2.072;
 SS =  1;
% HKL in reciprocal units
 QHKL = [1 0 0];
 EN  = 0;
 HKLEN = [QHKL EN];
endif

AKF = sqrt(EF/2.072);
EI = HKLEN(4) + EF;
if (EI < 0) error("KI too small\n"); endif;
AKI = sqrt(EI/2.072);
QHKL = HKLEN(1:3);
% calculate transformation matrix S from HKL to cartesian coordinate system
[S] = tas_rlp(SAM);
% calculate components of HKL in cartesian basis
%output components in cartesian basis, Modulus and Square of modulus
[QT, QS, QM] = RL2SPV (QHKL,S);
%calculate scattering angles A3 and A4
[AX3, AX4] = SAM_CASE(QT,QM,QS,AKI,AKF,SS);
%inverse calculation, HKL from A3 and A4
%output HKL, Modulus, Square of Modulus and angle between Ki and Q
%[QHKL, QM, QS, PHI] = SAM_UP (AKI, AKF, AX3, AX4, S);
endfunction

function [EX, AKX, IER] = EX_UP(DX,AX2)
%SUBROUTINE EX_UP(DX,EX,AKX,AX2,F,IER)	! Part of
%	================
%
%-----------------------------------------------------------------------
%	 CALCULATE EX AND AKX FORM AX2 VALUES
%-----------------------------------------------------------------------

RD = 180/acos(-1);
ARG=abs(DX*sin(AX2/(2.0*RD)));
if (ARG < 1e-4) 
  IER=1;
else
  IER=0;
  AKX=pi/ARG;
  EX=2.072*AKX^2;
endif
endfunction


function [QHKL, QM, QS, PHI] = SAM_UP (AKI, AKF, A3, A4, S)
%	SUBROUTINE SAM_UP (QHKL, QM, QS, PHI,AKI, AKF, A3, A4, IER)
%
%-----------------------------------------------------------------------
%	QPAR AND QPERP ARE COMPONENTS OF Q PARALLEL AND PERP TO KI
%	TRANSFORM FROM SCATTERING PLANE INTO RECIPROCAL LATTICE
%
%       PHI: ANGLE BETWEEN KI AND Q
%       QM: modulus of Q
%       QS: QM^2

RD = 180/pi;
%decompose scattering vector in components parallel and perpendicular to Ki
QPAR = AKI - AKF * cos(A4/RD);
QPERP = -AKF * sin(A4/RD);
%A3 is angle between Ki and AX = QT(1)
%rotation around z by +-A3 brings (QT(1),QT(2)) <-> (Q//,Qperp)
QT(1) = QPAR * cos(A3/RD) + QPERP * sin(A3/RD);
QT(2) = -QPAR * sin(A3/RD) + QPERP * cos(A3/RD);
QT(3) = 0.0;

%calculate HKL from QT components
[QHKL, QS, QM] = SP2RLV (QT,S^-1);
PHI = 0.0;
if (abs(QPERP) > 1e-3 && abs(QPAR) > 1e-3) 
  PHI = atan2 (QPERP, QPAR);
elseif (abs(QpaR) < 1e-3) 
  if (a4 > 0.0) 
    PHI = -0.5 * pi;
  else
    PHI = 0.5 * pi;
  endif
endif
endfunction


function [AX3, AX4] = SAM_CASE(QT,QM, QS,AKI,AKF,ISS)
%	SUBROUTINE SAM_CASE (				   
%	===================
%     +			      QT,QM,QS,AKI,AKF,AX3,AX4,ISS,IER)
%
%               DEAL WITH SAMPLE ANGLES CALCULATION FROM Q VECTOR IN C-N PLANE
%     		CALCULATE A3 AND A4
%  INPUTS
%       QT	  Q-VECTOR IN SCATTERING PLANE
%       QM,QS	  Q MODULUS AND QMODULUS SQUARED
%       AKI,AKF	  MOMEMTA ON MONO AND ANYLSER
%       ISS       SENS OF SCATTERING ON SAMPLE
%
%  OUTPUTS
%       AX3 AX4   ANGLES ON SAMPLES
%       IER	  SAME ERROR AS RL2SPV
%       IER
%        1       'MATRIX S NOT OK',
%        2       'Q NOT IN SCATTERING PLANE',
%        3       'Q MODULUS TOO SMALL',
%        4       'Q MODULUS TOO BIG',
%-----------------------------------------------------------------------


%       CALCULATE A3 AND MOVE IT INTHE -180 ,+180 INTERVAL
RD = 180/pi;
% calculate A4 angle
ARG = (AKI^2 + AKF^2 - QS)/(2.0*AKI*AKF);
if (abs(ARG) > 1.0) error("Triangle does not close\n"); endif;
AX4 = acos(ARG)*ISS*RD;
%A3 =  angle(QT(1),Ki) = angle(QT(1),Q) - angle(Ki,Q) 
AX3 = (-atan2(QT(2),QT(1))-acos((AKF**2-QS-AKI**2)/(-2.0*QM*AKI))*sign(AX4))*RD;
%not implemented on tasp???
%sax3 = sign(AX3);
%AX3 = mod(AX3+sax3*180.0,360)-sax3*180.0;
endfunction


function [S] = tas_rlp(SAM)
%	SUBROUTINE T_RLP	
%       INPUT SAM  SAMPLE CHARACTERISTICS
%       SAM(1)=AS       LATTICE PARAMETERS
%       SAM(2)=BS       ------------------
%       SAM(3)=CS       ------------------
%       SAM(4)=AA       LATTICE ANGLES
%       SAM(5)=BB       --------------
%       SAM(6)=CC       --------------
%       SAM(7)=AX       VECTOR A IN SCATTERING PLANE
%       SAM(8)=AY       ----------------------------
%       SAM(9)=AZ       ----------------------------
%       SAM(10)=BX      VECTOR B IN SCATTERING PLANE
%       SAM(11)=BY      ----------------------------
%       SAM(12)=BZ      ----------------------------
%       OUTPUT  Transformation matrix from HKL to scattering plane components
%               the scattering plane is defined with two vectors A, B
for ID = 1:3
  A(ID) = SAM(ID);        %lattice parameters
  ALFA(ID) = SAM(ID+3);   %lattice angles
  ASPV(ID,1) = SAM(6+ID); %first vector in scattering plane AX
  ASPV(ID,2) = SAM(9+ID); %second vector in scattering plane BX
endfor

ZP = 2.*pi;
RD = 180/pi;

for ID = 1:3
  A(ID) = A(ID)/ZP;   % lattice parameters/2/pi    
  ALFA(ID) = ALFA(ID)/RD; %angles in radians
  COSA(ID) = cos(ALFA(ID));
  SINA(ID) = sin(ALFA(ID));
endfor

CC = COSA(1)*COSA(1)+COSA(2)*COSA(2)+COSA(3)*COSA(3);
CC = (1.0+2.0*COSA(1)*COSA(2)*COSA(3)-CC);
CC = sqrt(CC);  % Volume_cell = abc*CC

JE = 2;
KG = 3;
for ID = 1:3
  B(ID) = SINA(ID)/(A(ID)*CC);  %reciprocal lattice vector lengthes
  COSB(ID) = (COSA(JE)*COSA(KG)-COSA(ID))/(SINA(JE)*SINA(KG)); %cos reciprocal angles
  SINB(ID) = sqrt(1.0-COSB(ID)*COSB(ID));                      %sin reciprocal angles
  RLB(ID,2) = abs(atan(SINB(ID)/COSB(ID)))*RD;                 %reciprocal angles 
  JE = KG;
  KG = ID;
endfor

%   Transformation reciprocal lattice to cartesian axis 
%   see Busing-Levy
BB(1,1) = B(1);
BB(2,1) = 0.0;
BB(3,1) = 0.0;
BB(1,2) = B(2)*COSB(3);
BB(2,2) = B(2)*SINB(3);
BB(3,2) = 0.0;
BB(1,3) = B(3)*COSB(2);
BB(2,3) = -B(3)*SINB(2)*COSA(1);
BB(3,3) = 1.0/A(3);

% calculate transformation matrix S 
% from HKL to scattering plane as defined by AX and BX
% ie S*[H K L]' gives component of HKL in AX,BX basis
% in A^-1

%not used
%for ID = 1:3
%  RLB(ID,1) = 0.0;
%  for JE = 1:3
%    RLB(ID,1) = RLB(ID,1)+BB(JE,ID)^2;
%  endfor
%  RLB(ID,1) = sqrt(RLB(ID,1));
%endfor

#{
% transform AX BX to cartesian coordinates
for KG = 1:2
  for IE = 1:3
    VV(KG,IE) = 0.0;
    for JF = 1:3
      VV(KG,IE) = VV(KG,IE)+BB(IE,JF)*ASPV(JF,KG);
    endfor
  endfor
endfor

% vector product 
for MD = 3:-1:2
  for NE = 1:3
    ID = mod(MD,3)+1;
    JE = mod(MD+1,3)+1;
    KG = mod(NE,3)+1;
    LH = mod(NE+1,3)+1;
    VV(MD,NE) = VV(ID,KG)*VV(JE,LH)-VV(ID,LH)*VV(JE,KG);
  endfor
endfor

%normalisation 
for ID = 1:3
  C(ID) = 0.0;
  for JE = 1:3
    C(ID) = C(ID)+VV(ID,JE)^2;
  endfor
  C(ID) = sqrt(C(ID));
endfor

for ID = 1:3
  for JE = 1:3
    VV(JE,ID) = VV(JE,ID)/C(JE);
  endfor
endfor

% transformation VV*BB
for KG = 1:3
  for ME = 1:3
    S(KG,ME) = 0.0;
    for LF = 1:3
      S(KG,ME) = S(KG,ME)+VV(KG,LF)*BB(LF,ME);
    endfor
  endfor
endfor
#}

% Do the same as above
A(1,:) = BB*ASPV(:,1);
A(2,:) = BB*ASPV(:,2);
A(3,:) = cross(A(1,:),A(2,:));
A(2,:) = cross(A(3,:),A(1,:));
A(1,:) = A(1,:)/norm(A(1,:));
A(2,:) = A(2,:)/norm(A(2,:));
A(3,:) = A(3,:)/norm(A(3,:));
B = [A(1,:); A(2,:); A(3,:)];
S = B*BB;

endfunction

function [QT, QS, QM] = RL2SPV (QHKL,S)
%function RL2SPV (QHKL, QT, QM, QS)
%       INPUT QHKL		      QHKL -> QT
%	A Q-VECTOR TO BE TRANSFORM FROM RECIP LATTICE TO SCATTERING PLANE
%       CHECK THAT Q-VECTOR IS IN THE PLANE
%
%       INPUT Q-VECTOR  QHKL(3) Q-VECTOR IN RECIPROCICAL LATTICVE
%
%       OUTPUT Q-VECTOR QT(3) Q-VECTOR IN SCATTERING PLANE
%       OUTPUT QM AND QS  QMODULUS AND ITS SQUARE ( TO BE  VERIFIED )
%       OUTPUT ERROR IER
%       IER=1   MATRIX S NOT OK
%       IER=2   Q NOT IN SCATTERING PLANE
%       IER=3   Q MODULUS TOO SMALL

#{
for ID = 1:3
  QT(ID) = 0.0;
  for JE = 1:3
    QT(ID) = QT(ID) + QHKL(JE)*S(ID,JE);
  endfor
endfor
#}
QT = S*QHKL';

if (abs(QT(3)) >1e-4) error("vector not in scattering plane\n"); return; endif;
QS = 0.0;
for ID = 1:3
  QS = QS+QT(ID)^2;
endfor
QM = sqrt(QS);
if (QM < 1.e-4) error("Q-Modulus too small"); endif;
endfunction


function [QHKL, QS, QM] = SP2RLV (QT,SINV) 
%	SUBROUTINE SP2RLV (QHKL, QT, QM, QS, IER)   ! Part of [MAD.SRC]T_RLP.FOR
%	=========================================
%
%------------------------------------------------------------------
%       INPUT QT	   QHKL <- QT
% A Q-VECTOR TO BE TRANSFORM FROM SCATTERING PLANE TO RECIP LATTICE
%       CHECK THAT Q, D & G VECTORS ARE IN THE SCATTERING PLANE
%
%       INPUT Q-VECTOR  QT(3) Q-VECTOR IN SCATTERING PLANE
%
%       OUTPUT Q-VECTOR QHKL(3) Q-VECTOR IN RECIPROCICAL LATTICVE
%       OUTPUT QM AND QS  QMODULUS AND ITS SQUARE ( TO BE  VERIFIED )
%       OUTPUT ERROR IER
%       IER=1   MATRIX S NOT OK
%       IER=2   Q NOT IN SCATTERING PLANE
%       IER=3   Q MODULUS TOO SMALL

QS = 0.0;
for ID = 1:3
  QS = QS+QT(ID)^2;
endfor
QM = sqrt(QS);
if (QM < 1.e-4) error("Q-Modulus too small"); endif;
#{
for ID = 1:3
  QHKL(ID) = 0.0;
  for JE = 1:3
    QHKL(ID) = QHKL(ID)+SINV(ID,JE)*QT(JE);
  endfor
endfor
#}
QHKL = SINV*QT';
endfunction


