## Copyright (C) 2012 Bertrand Roessli <bertrand.roessli@psi.ch>          
##
##
## MuFit is free software; you can redistribute it and/or
## modify it under the terms of the GNU General Public
## License as published by the Free Software Foundation;
## either version 2, or (at your option) any later version.
##
## Mufit is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied
## warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
## PURPOSE.  See the GNU General Public License for more
## details.
##
## You should have received a copy of the GNU General Public
## License along with Octave; see the file COPYING.  If not,
## write to the Free Software Foundation, Inc., 51 Franklin Street,
## Fifth Floor, Boston, MA 02110-1301, USA.
##

function [B] = BMat(a,b,c,aa,bb,cc)
%
% Transformation matrix to 
% cartesian axis (reciprocal space)
% using Bsuing-levy choice of axis
% (Busing-Levy Eq. 2)
% Corresponds to matrix R_21 and R_12 in the 
% crystallographic course

[G,Gstar] = metric(a,b,c,aa,bb,cc);

b1 = 1/sqrt(G(1,1));
b2 = 1/sqrt(G(2,2));
b3 = 1/sqrt(G(3,3));
a3 = a;
alpha1 = aa;

beta1 = acosd(Gstar(2,3)/b2/b3);
beta2 = acosd(Gstar(1,3)/b1/b3);
beta3 = acosd(Gstar(2,1)/b1/b2);

B = [b1 b2*cosd(beta3) b3*cosd(beta2); 
     0  b2*sind(beta3) -b3*sind(beta2)*cosd(alpha1);
     0  0              1/a3];

endfunction
