## Copyright (C) 2012 Bertrand Roessli <bertrand.roessli@psi.ch>          
##
##
## MuFit is free software; you can redistribute it and/or
## modify it under the terms of the GNU General Public
## License as published by the Free Software Foundation;
## either version 2, or (at your option) any later version.
##
## Mufit is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied
## warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
## PURPOSE.  See the GNU General Public License for more
## details.
##
## You should have received a copy of the GNU General Public
## License along with Octave; see the file COPYING.  If not,
## write to the Free Software Foundation, Inc., 51 Franklin Street,
## Fifth Floor, Boston, MA 02110-1301, USA.
##


function [UBMat] = UBcalc(a,b,c,aa,bb,cc,h1,h2,u1,u2,lambda)
%
% Calculate UB matrix according to Busing Levi
% Acta Cryst. 22, 457 (1967)
% Input
%     lattice parameters a,b,c
%     lattice angles     aa,bb,cc
%     first vector       h1
%     second vector      h2
%     observation angles for h1 and h2 [omega;chi;phi]
%
%  Output
%     UB matrix
%


B = BMat(a,b,c,aa,bb,cc);
uphi1 = Uphi(u1);
uphi2 = Uphi(u2);
U = UMat(B,h1,h2,uphi1,uphi2);

UBMat = U * B;
endfunction
