## Copyright (C) 2012 Bertrand Roessli <bertrand.roessli@psi.ch>          
##
##
## MuFit is free software; you can redistribute it and/or
## modify it under the terms of the GNU General Public
## License as published by the Free Software Foundation;
## either version 2, or (at your option) any later version.
##
## Mufit is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied
## warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
## PURPOSE.  See the GNU General Public License for more
## details.
##
## You should have received a copy of the GNU General Public
## License along with Octave; see the file COPYING.  If not,
## write to the Free Software Foundation, Inc., 51 Franklin Street,
## Fifth Floor, Boston, MA 02110-1301, USA.
##

function [U] = UMat(B,H1,H2,U1,U2)
%
% Construct U Matrix 
% Input 
%     B-matrix
%     H1 and H2 two reflections (Miller indices)
%     U1 and U2 corresponding coordinates in spectrometer system 
%     (from Uphi.m) 
%
% Busing-Levy Eq. 27
%

H1C = B * H1;
H2C = B * H2;
H3C = cross(H1C,H2C);
H3C = H3C/norm(H3C);
H1C = H1C/norm(H1C);
H2C = cross(H3C,H1C);

Tc = [H1C H2C H3C];
U1C = B * U1;
U2C = B * U2;
U3C = cross(U1C,U2C);
U3C = U3C/norm(U3C); 
U1C = U1C/norm(U1C);
U2C = cross(U3C,U1C);

Tphi = [U1C U2C U3C];

U = Tphi * inv(Tc);
endfunction
