/*
Copyright (C) 2009  Bertrand Roessli

     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

// Author: Bertrand Roessli <bertrand.roessli@psi.ch>

#include <octave/oct.h>

float sigma(octave_value_list  args_sigma);



DEFUN_DLD(__polmat2, args, , "c++-version of polmat.m")
{
   octave_value_list f_return;

   octave_value_list args_fcn(6, octave_value());

   ComplexMatrix poln_cs = ComplexMatrix(2, 2);
   poln_cs = args(0).complex_matrix_value();

   Matrix Pi = args(3).matrix_value();

   ComplexColumnVector s_px(2), s_py(2), s_pz(2);
   s_px(0) = Complex(1 / sqrt(2), 0);
   s_px(1) = Complex(1 / sqrt(2), 0);
   s_py(0) = Complex(1 / sqrt(2), 0);
   s_py(1) = Complex(0, 1 / sqrt(2));
   s_pz(0) = Complex(1, 0);
   s_pz(1) = Complex(0, 0);

   ComplexColumnVector s_mx(2), s_my(2), s_mz(2);
   s_mx(0) = Complex(1 / sqrt(2), 0);
   s_mx(1) = Complex(-1 / sqrt(2), 0);
   s_my(0) = Complex(1 / sqrt(2), 0);
   s_my(1) = Complex(0, -1 / sqrt(2));
   s_mz(0) = Complex(0, 0);
   s_mz(1) = Complex(-1, 0);

   octave_value_list vec_p, vec_m;
   vec_p(0) = s_px;
   vec_p(1) = s_py;
   vec_p(2) = s_pz;
   vec_m(0) = s_mx;
   vec_m(1) = s_my;
   vec_m(2) = s_mz;

   ComplexMatrix cs_matrix = ComplexMatrix(2, 2, 0);

   Matrix Ipp_ij = Matrix(3, 3, 0);
   Matrix Ipm_ij = Matrix(3, 3, 0);
   Matrix polar = Matrix(3, 3, 0);

   octave_value_list Ipp_list, Ipm_list, polar_list;

   int Pinc;
   cs_matrix(0, 0) = Complex(real(poln_cs(0, 0)), imag(poln_cs(0, 0)));
   cs_matrix(0, 1) = Complex(real(poln_cs(0, 1)), imag(poln_cs(0, 1)));
   cs_matrix(1, 0) = Complex(real(poln_cs(1, 0)), imag(poln_cs(1, 0)));
   cs_matrix(1, 1) = Complex(real(poln_cs(1, 1)), imag(poln_cs(1, 1)));

// loop over incident polarisation directions
   for (int mm = 0; mm <= 2; mm++)
   {
      for (int nn = 0; nn <= 2; nn++)
      {
         Pinc = Pi(0, nn);
// Pinc up
         if (Pinc == 1)
         {
            //calculate polarisation matrices and intensities
            for (int i = 0; i <= 2; i++)
            {
               for (int j = 0; j <= 2; j++)
               {
                  args_fcn(0) = vec_p(i);
                  args_fcn(1) = vec_p(j);
                  args_fcn(2) = vec_m(i);
                  args_fcn(3) = vec_m(j);
                  args_fcn(4) = cs_matrix;
                  args_fcn(5) = args(1).float_value();
                  args_fcn(6) = args(2).float_value();
                  Ipp_ij(i, j) = sigma(args_fcn);
                  args_fcn(0) = vec_p(i);
                  args_fcn(1) = vec_m(j);
                  args_fcn(2) = vec_m(i);
                  args_fcn(3) = vec_p(j);
                  args_fcn(4) = cs_matrix;
                  args_fcn(5) = args(1).float_value();
                  args_fcn(6) = args(2).float_value();
                  Ipm_ij(i, j) = sigma(args_fcn);
                  if ((Ipp_ij(i, j) + Ipm_ij(i, j)) == 0)
                  {
                     polar(i, j) = 0;
                  }
                  else
                  {
                     polar(i, j) = (Ipp_ij(i, j) - Ipm_ij(i, j)) / (Ipp_ij(i, j) + Ipm_ij(i, j));
                  }
/*
                  if (Ipp_ij(i, j) < 1.e-5)
                  {
                     Ipp_ij(i, j) = 0;
                  }
                  if (Ipm_ij(i, j) < 1.e-5)
                  {
                     Ipm_ij(i, j) = 0;
                  }
*/
               }
            }
//Pinc down
         }
         else if (Pinc == -1)
         {
            //calculate polarisation matrices and intensities
            for (int i = 0; i <= 2; i++)
            {
               for (int j = 0; j <= 2; j++)
               {
                  args_fcn(0) = vec_m(i);
                  args_fcn(1) = vec_p(j);
                  args_fcn(2) = vec_p(i);
                  args_fcn(3) = vec_m(j);
                  args_fcn(4) = cs_matrix;
                  args_fcn(5) = args(1).float_value();
                  args_fcn(6) = args(2).float_value();
                  Ipp_ij(i, j) = sigma(args_fcn);
                  args_fcn(0) = vec_m(i);
                  args_fcn(1) = vec_m(j);
                  args_fcn(2) = vec_p(i);
                  args_fcn(3) = vec_p(j);
                  args_fcn(4) = cs_matrix;
                  args_fcn(5) = args(1).float_value();
                  args_fcn(6) = args(2).float_value();
                  Ipm_ij(i, j) = sigma(args_fcn);
                  if ((Ipp_ij(i, j) + Ipm_ij(i, j)) == 0)
                  {
                     polar(i, j) = 0;
                  }
                  else
                  {
                     polar(i, j) = (Ipp_ij(i, j) - Ipm_ij(i, j)) / (Ipp_ij(i, j) + Ipm_ij(i, j));
                  }
/*
                  if (Ipp_ij(i, j) < 1.e-5)
                  {
                     Ipp_ij(i, j) = 0;
                  }
                  if (Ipm_ij(i, j) < 1.e-5)
                  {
                     Ipm_ij(i, j) = 0;
                  }
*/
               }
            }
         }
      }
   }
   f_return(0) = Ipp_ij;
   f_return(1) = Ipm_ij;
   f_return(2) = polar;
   return f_return;
}


float sigma(octave_value_list  args_sigma)
{
   /* c++ version of sigma_tot.m */

   ComplexColumnVector f1, f2, f3, f4;
   f1 = args_sigma(0).complex_column_vector_value();
   f2 = args_sigma(1).complex_column_vector_value();
   f3 = args_sigma(2).complex_column_vector_value();
   f4 = args_sigma(3).complex_column_vector_value();

   ComplexRowVector f1H, f2H, f3H, f4H;
   f1H = f1.hermitian(); // f1H=conj(f1).transpose();
   f2H = f2.hermitian();
   f3H = f3.hermitian();
   f4H = f4.hermitian();

   ComplexMatrix V = args_sigma(4).complex_matrix_value();
   ComplexMatrix VH = V.hermitian(); // VH=conj(V).transpose();

   float P1 = args_sigma(5).float_value();
   float P2 = args_sigma(6).float_value();

   Complex cs;
   float Intensity = 0.0;

   cs = (f1H * VH * f2) * (f2H * V * f1);
   Intensity = cs.real() * P1 * P2;


   cs = (f1H * VH * f4) * (f4H * V * f1);
   Intensity += cs.real() * P1 * (1.0 - P2);


   cs = (f3H * VH * f2) * (f2H * V * f3);
   Intensity += cs.real() * (1.0 - P1) * P2;


   cs = (f3H * VH * f4) * (f4H * V * f3);
   Intensity += cs.real() * (1.0 - P1) * (1.0 - P2);

   return Intensity;
}
