## Copyright (C) 2009 Bertrand Roessli <bertrand.roessli@psi.ch>
##
##
## MuFit is free software; you can redistribute it and/or
## modify it under the terms of the GNU General Public
## License as published by the Free Software Foundation;
## either version 2, or (at your option) any later version.
##
## Mufit is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied 
## warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
## PURPOSE.  See the GNU General Public License for more  
## details.
##
## You should have received a copy of the GNU General Public
## License along with Octave; see the file COPYING.  If not,
## write to the Free Software Foundation, Inc., 51 Franklin Street,
## Fifth Floor, Boston, MA 02110-1301, USA.
##

## -*- texinfo -*-
## @deftypefn{Function File}  mt_space_group () 
## @deftypefnx{Function File} mt_space_group ()
## Gives information about selected Space Group 
## and prepares input for nuclear input file
## @end deftypefn

function mt_space_group


[fid]=fopen('MADRGR.DAT','rt');
i=1;
while(1)
	try
                text = fscanf(fid,"%s\n ","C");
                if (feof(fid)) break; endif;
		spg(i).label= text;
		spg(i).nr_sym=fscanf(fid,"%i\n ","C");
		spg(i).type=fscanf(fid,"%i\n",[3,1]);
		[b]=fscanf(fid,"%f",[1,spg(i).nr_sym*12]);
		nrsym=spg(i).nr_sym;
		spg(i).sym=reshape(b,columns(b)/nrsym,nrsym);
		spg(i).nr=i;
		i++;
	catch
		break;
	end_try_catch
endwhile
fclose(fid);
sp=input("enter space group label (standard setting)? ","s");
if (isempty(sp))
	break;
endif

%---eliminate blanks in input
s=isspace(sp);
c=find(s==0);
sp=sp(c);
%--- is it a number?
if (!isempty(str2num(sp)) && !iscomplex(str2num(sp)))
	nbr = str2num(sp);
else
%---upper case
	sp=toupper(sp);
%---find space group index
	[m]=arrayfun(@(spg)strmatch(sp,spg.label,"exact"), spg, 'UniformOutput',false);
	nbr=find(cellfun(@isempty,m)==0);
endif
if (isempty(nbr))
	error("space group not found\n");
endif
%----print information about spacegroup
%fprintf(stdout,"*--> Space Group Nr.: %i\n",spg(nbr).nr);
fprintf(stdout,"*--> Space Group label: %s\n",spg(nbr).label);
switch spg(nbr).type(3)
	case 1
		fprintf(stdout,"    * Crystal system: Triclinic -> (alpha, beta, gamma != 90)\n");
	case 2
		fprintf(stdout,"    * Crystal system: Monoclinic -> (alpha != 90, beta, gamma = 90)\n");
	case 3
		fprintf(stdout,"    * Crystal system: Othorhombic -> (a != b != c)\n");
	case 4
		fprintf(stdout,"    * Crystal system: Tetragonal -> (a != c)\n");
	case 5
		fprintf(stdout,"    * Crystal system: Trigonal -> (a != c, gamma = 120) \n");
	case 6
		fprintf(stdout,"    * Crystal system: Hexagonal -> (a != c, gamma = 120)\n");
	case 7
		fprintf(stdout,"    * Crystal system: Cubic\n");
endswitch

switch spg(nbr).type(1)
	case 0
		fprintf(stdout,"    * Acentric Space Group \n");
	case 1 
		fprintf(stdout,"    * Centric Space Group: \n");
		fprintf(stdout,"        Inversion must be added \n");
endswitch

switch spg(nbr).type(2)
	case 1
		fprintf(stdout,"    * P-Lattice\n");
	case 2
		fprintf(stdout,"    * I-Lattice: \n");
		fprintf(stdout,"        Additional translation +(1/2,1/2,1/2)\n");
	case 3
		fprintf(stdout,"    * R-Lattice: \n");
		fprintf(stdout,"        Additional translations +(2/3,1/3,1/3), +(1/3,2/3,2/3)\n");
	case 4
		fprintf(stdout,"    * F-Lattice: \n");
		fprintf(stdout,"        Additional translation +(0,1/2,1/2)\n");
		fprintf(stdout,"        Additional translation +(1/2,0,1/2)\n");
		fprintf(stdout,"        Additional translation +(1/2,1/2,0)\n");
	case 5
		fprintf(stdout,"    * A-Lattice: \n");
		fprintf(stdout,"        Additional translation +(0,1/2,1/2)\n");
	case 6
		fprintf(stdout,"    * B-Lattice; \n");
		fprintf(stdout,"        Additional translation +(1/2,0,1/2)\n");
	case 7
		fprintf(stdout,"    * C-Lattice: \n");
		fprintf(stdout,"        Additional translation +(1/2,1/2,0)\n");
endswitch

fprintf(stdout,"    * Symmetry elements:\n\n");
for i=1:spg(nbr).nr_sym
        fprintf(stdout,"%i): \n",i);
	fprintf(stdout,"         |%+i %+i %+i| \n",reshape([spg(nbr).sym(1:9,i)],3,3));
	if (any(spg(nbr).sym(10:12,i)))
        	fprintf(stdout,"         +(%+1.3f, %+1.3f, %+1.3f)\n",spg(nbr).sym(10:12,i)); 
	endif
	fprintf(stdout,"\n");
endfor

%---- print out symmetry elements and translations that can be copy/pasted
%---- into nuc.inp
fprintf(stdout,"\n*-->these lines can be used as input in nuc.inp:\n");
fprintf(stdout,"----\n");

%---- rotation part
fprintf(stdout,"!# symmetry operations in space group and symmetry elements\n");

switch spg(nbr).type(1)
	case 0
		%---- acentric space group
		fprintf(stdout,"%i\n",spg(nbr).nr_sym);
	case 1 
		%---- centric space group
		fprintf(stdout,"%i\n",2*spg(nbr).nr_sym);
		%---- add inversion because space group is centrosymmetric
		k=1;
		for i=spg(nbr).nr_sym+1:2*spg(nbr).nr_sym;
			spg(nbr).sym(1:9,i)=-1*spg(nbr).sym(1:9,k);
			spg(nbr).sym(10:12,i)=spg(nbr).sym(10:12,k);
			k++;
		endfor
		spg(nbr).nr_sym=2*spg(nbr).nr_sym;		
endswitch

%---- put all together
for i=1:spg(nbr).nr_sym
	sym(1).a='';sym(1).b='';sym(1).c='';
	sym(2).a='';sym(2).b='';sym(2).c='';
	sym(3).a='';sym(3).b='';sym(3).c='';

	rot=reshape([spg(nbr).sym(1:9,i)],3,3);
	if (rot(1,1) ~=0)
		if (rot(1,1)==1)
			sym(1).a='+X';
		else
			sym(1).a='-X';
		endif
	endif
	if (rot(2,1) ~=0)
		if (rot(2,1)==1) 
			sym(2).a='+Y';
		else
			sym(2).a='-Y';
		endif
	endif
	if (rot(3,1) ~=0)
		if (rot(3,1)==1) 
			sym(3).a='+Z';
		else
			sym(3).a='-Z';
		endif
	endif
	if (rot(1,2) ~=0)
		if (rot(1,2)==1) 
			sym(1).b='+X';
		else
			sym(1).b='-X';
		endif
	endif
	if (rot(2,2) ~=0)
		if (rot(2,2)==1) 
			sym(2).b='+Y';
		else
			sym(2).b='-Y';
		endif
	endif
	if (rot(3,2) ~=0)
		if (rot(3,2)==1) 
			sym(3).b='+Z';
		else
			sym(3).b='-Z';
		endif
	endif
	if (rot(1,3) ~=0)
		if (rot(1,3)==1) 
			sym(1).c='+X';
		else
			sym(1).c='-X';
		endif
	endif
	if (rot(2,3) ~=0)
		if (rot(2,3)==1) 
			sym(2).c='+Y';
		else
			sym(2).c='-Y';
		endif
	endif
	if (rot(3,3) ~=0)
		if (rot(3,3)==1) 
			sym(3).c='+Z';
		else
			sym(3).c='-Z';
		endif
	endif
	r.ta=blanks(0);
	if (spg(nbr).sym(10,i) > 0) 
		r.ta =['+',num2str(spg(nbr).sym(10,i))];
	endif
	if (spg(nbr).sym(10,i) < 0) 
                r.ta =[num2str(spg(nbr).sym(10,i))];
        endif

	r.tb=blanks(0);
	if (spg(nbr).sym(11,i) > 0) 
		r.tb =['+',num2str(spg(nbr).sym(11,i))];
	endif
	if (spg(nbr).sym(11,i) < 0) 
                r.tb =[num2str(spg(nbr).sym(11,i))];
        endif

	r.tc=blanks(0);
	if (spg(nbr).sym(12,i) > 0) 
		r.tc =['+',num2str(spg(nbr).sym(12,i))];
	endif
	if (spg(nbr).sym(12,i) < 0) 
                r.tc =[num2str(spg(nbr).sym(12,i))];
        endif
	a(1,1)=cellstr([sym(1).a,sym(2).a,sym(3).a,r.ta]);
	a(2,1)=cellstr([sym(1).b,sym(2).b,sym(3).b,r.tb]);
	a(3,1)=cellstr([sym(1).c,sym(2).c,sym(3).c,r.tc]);

	fprintf(stdout,"%s\t\t%s\t\t%s\n",cell2mat(a(1,1)),cell2mat(a(2,1)),cell2mat(a(3,1)));
endfor

%---- additional translations due to lattice centering
fprintf(stdout,"!# number of translations and translations in space group\n");
switch spg(nbr).type(2)
	case 1
		fprintf(stdout,"0\n");
	case 2
		fprintf(stdout,"2\n");
		fprintf(stdout,"0     0     0  \n");
		fprintf(stdout,"1/2   1/2   1/2\n");
	case 3
		fprintf(stdout,"3\n");
		fprintf(stdout,"0     0     0  \n");
		fprintf(stdout,"2/3   1/3   1/3\n");
		fprintf(stdout,"1/3   2/3   2/3\n");
	case 4
		fprintf(stdout,"4\n");
		fprintf(stdout,"0     0     0  \n");
		fprintf(stdout,"0     1/2    1/2\n");
		fprintf(stdout,"1/2     0    1/2 \n");
		fprintf(stdout,"1/2   1/2    0\n");
	case 5
		fprintf(stdout,"2\n");
		fprintf(stdout,"0     0     0  \n");
		fprintf(stdout,"0    1/2    1/2\n");
	case 6
		fprintf(stdout,"2\n");
		fprintf(stdout,"0     0     0  \n");
		fprintf(stdout,"1/2    0   1/2\n");
	case 7
		fprintf(stdout,"2\n");
		fprintf(stdout,"0     0     0  \n");
		fprintf(stdout," 1/2    1/2    0\n");
endswitch
fprintf(stdout,"----\n");

endfunction

#{
P1                                                                              
          1                                                                     
          0 1 1                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
F-1                                                                             
          1                                                                     
          1 4 1                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
P-1                                                                             
          1                                                                     
          1 1 1                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
P2                                                                              
          2                                                                     
          0 1 2                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
P21                                                                             
          2                                                                     
          0 1 2                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .5000000000000 .0000000000000       
C2                                                                              
          2                                                                     
          0 7 2                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
I2                                                                              
          2                                                                     
          0 2 2                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
PM                                                                              
          2                                                                     
          0 1 2                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
PC                                                                              
          2                                                                     
          0 1 2                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
PN                                                                              
          2                                                                     
          0 1 2                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0 1  .5000000000000 .0000000000000 .5000000000000       
CM                                                                              
          2                                                                     
          0 7 2                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
IM                                                                              
          2                                                                     
          0 2 2                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
CC                                                                              
          2                                                                     
          0 7 2                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
P2/M                                                                            
          2                                                                     
          1 1 2                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
P21/M                                                                           
          2                                                                     
          1 1 2                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .5000000000000 .0000000000000       
C2/M                                                                            
          2                                                                     
          1 7 2                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
I2/M                                                                            
          2                                                                     
          1 2 2                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
P2/C                                                                            
          2                                                                     
          1 1 2                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .5000000000000       
P2/N                                                                            
          2                                                                     
          1 1 2                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .5000000000000 .0000000000000 .5000000000000       
P2/A                                                                            
          2                                                                     
          1 1 2                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .5000000000000 .0000000000000 .0000000000000       
P21/C                                                                           
          2                                                                     
          1 1 2                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .5000000000000 .5000000000000       
P21/A                                                                           
          2                                                                     
          1 1 2                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .5000000000000 .5000000000000 .0000000000000       
P21/N                                                                           
          2                                                                     
          1 1 2                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .5000000000000 .5000000000000 .5000000000000       
C2/C                                                                            
          2                                                                     
          1 7 2                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .5000000000000       
P222                                                                            
          4                                                                     
          0 1 3                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
P2221                                                                           
          4                                                                     
          0 1 3                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .5000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
P21212                                                                          
          4                                                                     
          0 1 3                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .5000000000000 .5000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .5000000000000 .5000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
P212121                                                                         
          4                                                                     
          0 1 3                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .5000000000000 .5000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .5000000000000 .5000000000000       
         -1 0 0 0-1 0 0 0 1  .5000000000000 .0000000000000 .5000000000000       
C2221                                                                           
          4                                                                     
          0 7 3                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .5000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
C222                                                                            
          4                                                                     
          0 7 3                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
F222                                                                            
          4                                                                     
          0 4 3                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
I222                                                                            
          4                                                                     
          0 2 3                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
I212121                                                                         
          4                                                                     
          0 2 3                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .5000000000000 .5000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .5000000000000 .5000000000000       
         -1 0 0 0-1 0 0 0 1  .5000000000000 .0000000000000 .5000000000000       
PMM2                                                                            
          4                                                                     
          0 1 3                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
PMC21                                                                           
          4                                                                     
          0 1 3                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
PCC2                                                                            
          4                                                                     
          0 1 3                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
PMA2                                                                            
          4                                                                     
          0 1 3                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0 1  .5000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0 1  .5000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
PCA21                                                                           
          4                                                                     
          0 1 3                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0 1  .5000000000000 .0000000000000 .5000000000000       
          1 0 0 0-1 0 0 0 1  .5000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
PNC2                                                                            
          4                                                                     
          0 1 3                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0 1  .0000000000000 .5000000000000 .5000000000000       
          1 0 0 0-1 0 0 0 1  .0000000000000 .5000000000000 .5000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
PMN21                                                                           
          4                                                                     
          0 1 3                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0 1  .5000000000000 .0000000000000 .5000000000000       
         -1 0 0 0-1 0 0 0 1  .5000000000000 .0000000000000 .5000000000000       
PBA2                                                                            
          4                                                                     
          0 1 3                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0 1  .5000000000000 .5000000000000 .0000000000000       
          1 0 0 0-1 0 0 0 1  .5000000000000 .5000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
PNA21                                                                           
          4                                                                     
          0 1 3                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0 1  .5000000000000 .5000000000000 .5000000000000       
          1 0 0 0-1 0 0 0 1  .5000000000000 .5000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
PNN2                                                                            
          4                                                                     
          0 1 3                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0 1  .5000000000000 .5000000000000 .5000000000000       
          1 0 0 0-1 0 0 0 1  .5000000000000 .5000000000000 .5000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
CMM2                                                                            
          4                                                                     
          0 7 3                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
CMC21                                                                           
          4                                                                     
          0 7 3                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
A21AM                                                                           
          4                                                                     
          0 5 3                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0 1  .5000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .5000000000000 .0000000000000 .0000000000000       
CCC2                                                                            
          4                                                                     
          0 7 3                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
AMM2                                                                            
          4                                                                     
          0 5 3                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
ABM2                                                                            
          4                                                                     
          0 5 3                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0 1  .0000000000000 .5000000000000 .0000000000000       
          1 0 0 0-1 0 0 0 1  .0000000000000 .5000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
AMA2                                                                            
          4                                                                     
          0 5 3                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0 1  .5000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0 1  .5000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
C2CM                                                                            
          4                                                                     
          0 7 3                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          1 0 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .5000000000000       
ABA2                                                                            
          4                                                                     
          0 5 3                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0 1  .5000000000000 .5000000000000 .0000000000000       
          1 0 0 0-1 0 0 0 1  .5000000000000 .5000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
FMM2                                                                            
          4                                                                     
          0 4 3                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
FDD2                                                                            
          4                                                                     
          0 4 3                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0 1  .2500000000000 .2500000000000 .2500000000000       
          1 0 0 0-1 0 0 0 1  .2500000000000 .2500000000000 .2500000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
IMM2                                                                            
          4                                                                     
          0 2 3                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
IBA2                                                                            
          4                                                                     
          0 2 3                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0 1  .5000000000000 .5000000000000 .0000000000000       
          1 0 0 0-1 0 0 0 1  .5000000000000 .5000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
IMA2                                                                            
          4                                                                     
          0 2 3                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0 1  .5000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0 1  .5000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
PMMM                                                                            
          4                                                                     
          1 1 3                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
PNNN                                                                            
          4                                                                     
          1 1 3                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .0000000000000 .5000000000000 .5000000000000       
         -1 0 0 0 1 0 0 0-1  .5000000000000 .0000000000000 .5000000000000       
         -1 0 0 0-1 0 0 0 1  .5000000000000 .5000000000000 .0000000000000       
PCCM                                                                            
          4                                                                     
          1 1 3                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .0000000000000 .0000000000000 .5000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .5000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
PBAN                                                                            
          4                                                                     
          1 1 3                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .0000000000000 .5000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .5000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .5000000000000 .5000000000000 .0000000000000       
PMMA                                                                            
          4                                                                     
          1 1 3                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .5000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .5000000000000 .0000000000000 .0000000000000       
PNNA                                                                            
          4                                                                     
          1 1 3                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .0000000000000 .5000000000000 .5000000000000       
         -1 0 0 0 1 0 0 0-1  .5000000000000 .5000000000000 .5000000000000       
         -1 0 0 0-1 0 0 0 1  .5000000000000 .0000000000000 .0000000000000       
PMNA                                                                            
          4                                                                     
          1 1 3                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .5000000000000 .0000000000000 .5000000000000       
         -1 0 0 0-1 0 0 0 1  .5000000000000 .0000000000000 .5000000000000       
PCCA                                                                            
          4                                                                     
          1 1 3                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .5000000000000 .0000000000000 .5000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .5000000000000       
         -1 0 0 0-1 0 0 0 1  .5000000000000 .0000000000000 .0000000000000       
PBAM                                                                            
          4                                                                     
          1 1 3                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .5000000000000 .5000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .5000000000000 .5000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
PCCN                                                                            
          4                                                                     
          1 1 3                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .5000000000000 .0000000000000 .5000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .5000000000000 .5000000000000       
         -1 0 0 0-1 0 0 0 1  .5000000000000 .5000000000000 .0000000000000       
PBCM                                                                            
          4                                                                     
          1 1 3                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .0000000000000 .5000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .5000000000000 .5000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
PNNM                                                                            
          4                                                                     
          1 1 3                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .5000000000000 .5000000000000 .5000000000000       
         -1 0 0 0 1 0 0 0-1  .5000000000000 .5000000000000 .5000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
PMMN                                                                            
          4                                                                     
          1 1 3                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .5000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .5000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .5000000000000 .5000000000000 .0000000000000       
PBCN                                                                            
          4                                                                     
          1 1 3                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .5000000000000 .5000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .5000000000000       
         -1 0 0 0-1 0 0 0 1  .5000000000000 .5000000000000 .5000000000000       
PBCA                                                                            
          4                                                                     
          1 1 3                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .5000000000000 .5000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .5000000000000 .5000000000000       
         -1 0 0 0-1 0 0 0 1  .5000000000000 .0000000000000 .5000000000000       
PNMA                                                                            
          4                                                                     
          1 1 3                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .5000000000000 .5000000000000 .5000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .5000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .5000000000000 .0000000000000 .5000000000000       
PNAM                                                                            
          4                                                                     
          1 1 3                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .5000000000000 .5000000000000 .5000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
         -1 0 0 0 1 0 0 0-1  .5000000000000 .5000000000000 .0000000000000       
PBNM                                                                            
          4                                                                     
          1 1 3                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .5000000000000 .5000000000000 .5000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          1 0 0 0-1 0 0 0-1  .5000000000000 .5000000000000 .0000000000000       
CMCM                                                                            
          4                                                                     
          1 7 3                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .5000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
CMCA                                                                            
          4                                                                     
          1 7 3                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0 1  .0000000000000 .5000000000000 .5000000000000       
          1 0 0 0 1 0 0 0-1  .0000000000000 .5000000000000 .5000000000000       
CMMM                                                                            
          4                                                                     
          1 7 3                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
CCCM                                                                            
          4                                                                     
          1 7 3                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .0000000000000 .0000000000000 .5000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .5000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
CMMA                                                                            
          4                                                                     
          1 7 3                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0 1  .5000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .5000000000000 .0000000000000 .0000000000000       
CCCA                                                                            
          4                                                                     
          1 7 3                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .5000000000000 .0000000000000 .5000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .5000000000000       
         -1 0 0 0-1 0 0 0 1  .5000000000000 .0000000000000 .0000000000000       
FMMM                                                                            
          4                                                                     
          1 4 3                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
FDDD                                                                            
          4                                                                     
          1 4 3                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .0000000000000 .2500000000000 .2500000000000       
         -1 0 0 0 1 0 0 0-1  .2500000000000 .0000000000000 .2500000000000       
         -1 0 0 0-1 0 0 0 1  .2500000000000 .2500000000000 .0000000000000       
FDAZ                                                                            
          8                                                                     
          0 4 3                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0-1  .2500000000000 .2500000000000 .2500000000000       
          1 0 0 0-1 0 0 0 1  .2500000000000 .2500000000000 .2500000000000       
          1 0 0 0-1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0 1  .2500000000000 .2500000000000 .2500000000000       
          1 0 0 0 1 0 0 0-1  .2500000000000 .2500000000000 .2500000000000       
IMMM                                                                            
          4                                                                     
          1 2 3                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
IBAM                                                                            
          4                                                                     
          1 2 3                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .5000000000000 .5000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .5000000000000 .5000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
IBCA                                                                            
          4                                                                     
          1 2 3                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .5000000000000 .5000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .5000000000000 .5000000000000       
         -1 0 0 0-1 0 0 0 1  .5000000000000 .0000000000000 .5000000000000       
IMMA                                                                            
          4                                                                     
          1 2 3                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0 1  .0000000000000 .5000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0 1 0 0 0-1  .0000000000000 .5000000000000 .0000000000000       
P4                                                                              
          4                                                                     
          0 1 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
P41                                                                             
          4                                                                     
          0 1 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .2500000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          0 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .7500000000000       
P42                                                                             
          4                                                                     
          0 1 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
P43                                                                             
          4                                                                     
          0 1 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .7500000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          0 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .2500000000000       
I4                                                                              
          4                                                                     
          0 2 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
I41                                                                             
          4                                                                     
          0 2 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0 1  .0000000000000 .5000000000000 .2500000000000       
         -1 0 0 0-1 0 0 0 1  .5000000000000 .5000000000000 .5000000000000       
          0 1 0-1 0 0 0 0 1  .5000000000000 .0000000000000 .7500000000000       
P-4                                                                             
          4                                                                     
          0 1 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
I-4                                                                             
          4                                                                     
          0 2 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
P4/M                                                                            
          4                                                                     
          1 1 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
P42/M                                                                           
          4                                                                     
          1 1 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
P4/N                                                                            
          4                                                                     
          1 1 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0 1  .5000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .5000000000000 .5000000000000 .0000000000000       
          0 1 0-1 0 0 0 0 1  .0000000000000 .5000000000000 .0000000000000       
P42/N                                                                           
          4                                                                     
          1 1 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0 1  .0000000000000 .5000000000000 .5000000000000       
         -1 0 0 0-1 0 0 0 1  .5000000000000 .5000000000000 .0000000000000       
          0 1 0-1 0 0 0 0 1  .5000000000000 .0000000000000 .5000000000000       
I4/M                                                                            
          4                                                                     
          1 2 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
I41/A                                                                           
          4                                                                     
          1 2 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0 1  .7500000000000 .2500000000000 .2500000000000       
         -1 0 0 0-1 0 0 0 1  .5000000000000 .0000000000000 .5000000000000       
          0 1 0-1 0 0 0 0 1  .7500000000000 .7500000000000 .7500000000000       
I41/N                                                                           
          8                                                                     
          0 2 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0 1 0 0 0-1  .0000000000000 .5000000000000 .2500000000000       
         -1 0 0 0-1 0 0 0-1  .0000000000000 .5000000000000 .2500000000000       
          0-1 0 1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0 1  .0000000000000 .5000000000000 .2500000000000       
          0 1 0-1 0 0 0 0 1  .0000000000000 .5000000000000 .2500000000000       
P422                                                                            
          8                                                                     
          0 1 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0-1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
P4212                                                                           
          8                                                                     
          0 1 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0 1  .5000000000000 .5000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 0 0 0 0 1  .5000000000000 .5000000000000 .0000000000000       
          0 1 0 1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .5000000000000 .5000000000000 .0000000000000       
          0-1 0-1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .5000000000000 .5000000000000 .0000000000000       
P4122                                                                           
          8                                                                     
          0 1 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .2500000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          0 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .7500000000000       
          0 1 0 1 0 0 0 0-1  .0000000000000 .0000000000000 .7500000000000       
          1 0 0 0-1 0 0 0-1  .0000000000000 .0000000000000 .5000000000000       
          0-1 0-1 0 0 0 0-1  .0000000000000 .0000000000000 .2500000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
P41212                                                                          
          8                                                                     
          0 1 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0 1  .5000000000000 .5000000000000 .2500000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          0 1 0-1 0 0 0 0 1  .5000000000000 .5000000000000 .7500000000000       
          0 1 0 1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .5000000000000 .5000000000000 .7500000000000       
          0-1 0-1 0 0 0 0-1  .0000000000000 .0000000000000 .5000000000000       
         -1 0 0 0 1 0 0 0-1  .5000000000000 .5000000000000 .2500000000000       
P4222                                                                           
          8                                                                     
          0 1 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          0 1 0 1 0 0 0 0-1  .0000000000000 .0000000000000 .5000000000000       
          1 0 0 0-1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0-1 0 0 0 0-1  .0000000000000 .0000000000000 .5000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
P42212                                                                          
          8                                                                     
          0 1 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0 1  .5000000000000 .5000000000000 .5000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 0 0 0 0 1  .5000000000000 .5000000000000 .5000000000000       
          0 1 0 1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .5000000000000 .5000000000000 .5000000000000       
          0-1 0-1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .5000000000000 .5000000000000 .5000000000000       
P4322                                                                           
          8                                                                     
          0 1 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .7500000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          0 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .2500000000000       
          0 1 0 1 0 0 0 0-1  .0000000000000 .0000000000000 .2500000000000       
          1 0 0 0-1 0 0 0-1  .0000000000000 .0000000000000 .5000000000000       
          0-1 0-1 0 0 0 0-1  .0000000000000 .0000000000000 .7500000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
P43212                                                                          
          8                                                                     
          0 1 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0 1  .5000000000000 .5000000000000 .7500000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          0 1 0-1 0 0 0 0 1  .5000000000000 .5000000000000 .2500000000000       
          0 1 0 1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .5000000000000 .5000000000000 .2500000000000       
          0-1 0-1 0 0 0 0-1  .0000000000000 .0000000000000 .5000000000000       
         -1 0 0 0 1 0 0 0-1  .5000000000000 .5000000000000 .7500000000000       
I422                                                                            
          8                                                                     
          0 2 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0-1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
I4122                                                                           
          8                                                                     
          0 2 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0 1  .0000000000000 .5000000000000 .2500000000000       
         -1 0 0 0-1 0 0 0 1  .5000000000000 .5000000000000 .5000000000000       
          0 1 0-1 0 0 0 0 1  .5000000000000 .0000000000000 .7500000000000       
          0 1 0 1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .5000000000000 .0000000000000 .7500000000000       
          0-1 0-1 0 0 0 0-1  .5000000000000 .5000000000000 .5000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .5000000000000 .2500000000000       
P4MM                                                                            
          8                                                                     
          0 1 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
P4BM                                                                            
          8                                                                     
          0 1 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0 1  .5000000000000 .5000000000000 .0000000000000       
          0 1 0 1 0 0 0 0 1  .5000000000000 .5000000000000 .0000000000000       
          1 0 0 0-1 0 0 0 1  .5000000000000 .5000000000000 .0000000000000       
          0-1 0-1 0 0 0 0 1  .5000000000000 .5000000000000 .0000000000000       
P42CM                                                                           
          8                                                                     
          0 1 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
         -1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          0 1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          0-1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
P42NM                                                                           
          8                                                                     
          0 1 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0 1  .5000000000000 .5000000000000 .5000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 0 0 0 0 1  .5000000000000 .5000000000000 .5000000000000       
         -1 0 0 0 1 0 0 0 1  .5000000000000 .5000000000000 .5000000000000       
          0 1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0 1  .5000000000000 .5000000000000 .5000000000000       
          0-1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
P4CC                                                                            
          8                                                                     
          0 1 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          0 1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          0-1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
P4NC                                                                            
          8                                                                     
          0 1 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0 1  .5000000000000 .5000000000000 .5000000000000       
          0 1 0 1 0 0 0 0 1  .5000000000000 .5000000000000 .5000000000000       
          1 0 0 0-1 0 0 0 1  .5000000000000 .5000000000000 .5000000000000       
          0-1 0-1 0 0 0 0 1  .5000000000000 .5000000000000 .5000000000000       
P42MC                                                                           
          8                                                                     
          0 1 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
         -1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
P42BC                                                                           
          8                                                                     
          0 1 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
         -1 0 0 0 1 0 0 0 1  .5000000000000 .5000000000000 .0000000000000       
          0 1 0 1 0 0 0 0 1  .5000000000000 .5000000000000 .5000000000000       
          1 0 0 0-1 0 0 0 1  .5000000000000 .5000000000000 .0000000000000       
          0-1 0-1 0 0 0 0 1  .5000000000000 .5000000000000 .5000000000000       
I4MM                                                                            
          8                                                                     
          0 2 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
I4CM                                                                            
          8                                                                     
          0 2 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          0 1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          0-1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
I41MD                                                                           
          8                                                                     
          0 2 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0 1  .0000000000000 .5000000000000 .2500000000000       
         -1 0 0 0-1 0 0 0 1  .5000000000000 .5000000000000 .5000000000000       
          0 1 0-1 0 0 0 0 1  .5000000000000 .0000000000000 .7500000000000       
         -1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 1 0 0 0 0 1  .0000000000000 .5000000000000 .2500000000000       
          1 0 0 0-1 0 0 0 1  .5000000000000 .5000000000000 .5000000000000       
          0-1 0-1 0 0 0 0 1  .5000000000000 .0000000000000 .7500000000000       
I41CD                                                                           
          8                                                                     
          0 2 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0 1  .0000000000000 .5000000000000 .2500000000000       
         -1 0 0 0-1 0 0 0 1  .5000000000000 .5000000000000 .5000000000000       
          0 1 0-1 0 0 0 0 1  .5000000000000 .0000000000000 .7500000000000       
         -1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          0 1 0 1 0 0 0 0 1  .0000000000000 .5000000000000 .7500000000000       
          1 0 0 0-1 0 0 0 1  .5000000000000 .5000000000000 .0000000000000       
          0-1 0-1 0 0 0 0 1  .5000000000000 .0000000000000 .2500000000000       
P-42M                                                                           
          8                                                                     
          0 1 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
P-42C                                                                           
          8                                                                     
          0 1 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .0000000000000 .0000000000000 .5000000000000       
          0 1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .5000000000000       
          0-1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
P-421M                                                                          
          8                                                                     
          0 1 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .5000000000000 .5000000000000 .0000000000000       
          0 1 0 1 0 0 0 0 1  .5000000000000 .5000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .5000000000000 .5000000000000 .0000000000000       
          0-1 0-1 0 0 0 0 1  .5000000000000 .5000000000000 .0000000000000       
P-421C                                                                          
          8                                                                     
          0 1 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .5000000000000 .5000000000000 .5000000000000       
          0 1 0 1 0 0 0 0 1  .5000000000000 .5000000000000 .5000000000000       
         -1 0 0 0 1 0 0 0-1  .5000000000000 .5000000000000 .5000000000000       
          0-1 0-1 0 0 0 0 1  .5000000000000 .5000000000000 .5000000000000       
I-4M2                                                                           
          8                                                                     
          0 2 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0-1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
P-4C2                                                                           
          8                                                                     
          0 1 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          0-1 0-1 0 0 0 0-1  .0000000000000 .0000000000000 .5000000000000       
          1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          0 1 0 1 0 0 0 0-1  .0000000000000 .0000000000000 .5000000000000       
P-4B2                                                                           
          8                                                                     
          0 1 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0 1  .5000000000000 .5000000000000 .0000000000000       
          0-1 0-1 0 0 0 0-1  .5000000000000 .5000000000000 .0000000000000       
          1 0 0 0-1 0 0 0 1  .5000000000000 .5000000000000 .0000000000000       
          0 1 0 1 0 0 0 0-1  .5000000000000 .5000000000000 .0000000000000       
P-4N2                                                                           
          8                                                                     
          0 1 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0 1  .5000000000000 .5000000000000 .5000000000000       
          0-1 0-1 0 0 0 0-1  .5000000000000 .5000000000000 .5000000000000       
          1 0 0 0-1 0 0 0 1  .5000000000000 .5000000000000 .5000000000000       
          0 1 0 1 0 0 0 0-1  .5000000000000 .5000000000000 .5000000000000       
P-4M2                                                                           
          8                                                                     
          0 1 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0-1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
I-4C2                                                                           
          8                                                                     
          0 2 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          0-1 0-1 0 0 0 0-1  .0000000000000 .0000000000000 .5000000000000       
          1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          0 1 0 1 0 0 0 0-1  .0000000000000 .0000000000000 .5000000000000       
I-42M                                                                           
          8                                                                     
          0 2 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
I-42D                                                                           
          8                                                                     
          0 2 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .0000000000000 .5000000000000 .2500000000000       
          0 1 0 1 0 0 0 0 1  .0000000000000 .5000000000000 .2500000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .5000000000000 .2500000000000       
          0-1 0-1 0 0 0 0 1  .0000000000000 .5000000000000 .2500000000000       
P4/MMM                                                                          
          8                                                                     
          1 1 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
P4/MCC                                                                          
          8                                                                     
          1 1 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          0 1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          0-1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
P4/NBM                                                                          
          8                                                                     
          1 1 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0 1  .5000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .5000000000000 .5000000000000 .0000000000000       
          0 1 0-1 0 0 0 0 1  .0000000000000 .5000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0 1  .0000000000000 .5000000000000 .0000000000000       
          0 1 0 1 0 0 0 0 1  .5000000000000 .5000000000000 .0000000000000       
          1 0 0 0-1 0 0 0 1  .5000000000000 .0000000000000 .0000000000000       
          0-1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
P4/NNC                                                                          
          8                                                                     
          1 1 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0 1  .5000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .5000000000000 .5000000000000 .0000000000000       
          0 1 0-1 0 0 0 0 1  .0000000000000 .5000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0 1  .0000000000000 .5000000000000 .5000000000000       
          0 1 0 1 0 0 0 0 1  .5000000000000 .5000000000000 .5000000000000       
          1 0 0 0-1 0 0 0 1  .5000000000000 .0000000000000 .5000000000000       
          0-1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
P4/MBM                                                                          
          8                                                                     
          1 1 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0 1  .5000000000000 .5000000000000 .0000000000000       
          0 1 0 1 0 0 0 0 1  .5000000000000 .5000000000000 .0000000000000       
          1 0 0 0-1 0 0 0 1  .5000000000000 .5000000000000 .0000000000000       
          0-1 0-1 0 0 0 0 1  .5000000000000 .5000000000000 .0000000000000       
P4/MNC                                                                          
          8                                                                     
          1 1 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0 1  .5000000000000 .5000000000000 .5000000000000       
          0 1 0 1 0 0 0 0 1  .5000000000000 .5000000000000 .5000000000000       
          1 0 0 0-1 0 0 0 1  .5000000000000 .5000000000000 .5000000000000       
          0-1 0-1 0 0 0 0 1  .5000000000000 .5000000000000 .5000000000000       
P4/NMM                                                                          
          8                                                                     
          1 1 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0 1  .5000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .5000000000000 .5000000000000 .0000000000000       
          0 1 0-1 0 0 0 0 1  .0000000000000 .5000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0 1  .5000000000000 .0000000000000 .0000000000000       
          0 1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0 1  .0000000000000 .5000000000000 .0000000000000       
          0-1 0-1 0 0 0 0 1  .5000000000000 .5000000000000 .0000000000000       
P4/NCC                                                                          
          8                                                                     
          1 1 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0 1  .5000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .5000000000000 .5000000000000 .0000000000000       
          0 1 0-1 0 0 0 0 1  .0000000000000 .5000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0 1  .5000000000000 .0000000000000 .5000000000000       
          0 1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          1 0 0 0-1 0 0 0 1  .0000000000000 .5000000000000 .5000000000000       
          0-1 0-1 0 0 0 0 1  .5000000000000 .5000000000000 .5000000000000       
P42/MMC                                                                         
          8                                                                     
          1 1 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
         -1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
P42/MCM                                                                         
          8                                                                     
          1 1 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
         -1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          0 1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          0-1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
P42/NBC                                                                         
          8                                                                     
          1 1 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0 1  .5000000000000 .0000000000000 .5000000000000       
         -1 0 0 0-1 0 0 0 1  .5000000000000 .5000000000000 .0000000000000       
          0 1 0-1 0 0 0 0 1  .0000000000000 .5000000000000 .5000000000000       
         -1 0 0 0 1 0 0 0 1  .0000000000000 .5000000000000 .0000000000000       
          0 1 0 1 0 0 0 0 1  .5000000000000 .5000000000000 .5000000000000       
          1 0 0 0-1 0 0 0 1  .5000000000000 .0000000000000 .0000000000000       
          0-1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
P42/NNM                                                                         
          8                                                                     
          1 1 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0 1  .5000000000000 .0000000000000 .5000000000000       
         -1 0 0 0-1 0 0 0 1  .5000000000000 .5000000000000 .0000000000000       
          0 1 0-1 0 0 0 0 1  .0000000000000 .5000000000000 .5000000000000       
         -1 0 0 0 1 0 0 0 1  .0000000000000 .5000000000000 .5000000000000       
          0 1 0 1 0 0 0 0 1  .5000000000000 .5000000000000 .0000000000000       
          1 0 0 0-1 0 0 0 1  .5000000000000 .0000000000000 .5000000000000       
          0-1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
P42/MBC                                                                         
          8                                                                     
          1 1 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
         -1 0 0 0 1 0 0 0 1  .5000000000000 .5000000000000 .0000000000000       
          0 1 0 1 0 0 0 0 1  .5000000000000 .5000000000000 .5000000000000       
          1 0 0 0-1 0 0 0 1  .5000000000000 .5000000000000 .0000000000000       
          0-1 0-1 0 0 0 0 1  .5000000000000 .5000000000000 .5000000000000       
P42/MNM                                                                         
          8                                                                     
          1 1 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0 1  .5000000000000 .5000000000000 .5000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 0 0 0 0 1  .5000000000000 .5000000000000 .5000000000000       
         -1 0 0 0 1 0 0 0 1  .5000000000000 .5000000000000 .5000000000000       
          0 1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0 1  .5000000000000 .5000000000000 .5000000000000       
          0-1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
P42/NMC                                                                         
          8                                                                     
          1 1 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0 1  .5000000000000 .0000000000000 .5000000000000       
         -1 0 0 0-1 0 0 0 1  .5000000000000 .5000000000000 .0000000000000       
          0 1 0-1 0 0 0 0 1  .0000000000000 .5000000000000 .5000000000000       
         -1 0 0 0 1 0 0 0 1  .5000000000000 .0000000000000 .0000000000000       
          0 1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          1 0 0 0-1 0 0 0 1  .0000000000000 .5000000000000 .0000000000000       
          0-1 0-1 0 0 0 0 1  .5000000000000 .5000000000000 .5000000000000       
P42/NCM                                                                         
          8                                                                     
          1 1 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0 1  .5000000000000 .0000000000000 .5000000000000       
         -1 0 0 0-1 0 0 0 1  .5000000000000 .5000000000000 .0000000000000       
          0 1 0-1 0 0 0 0 1  .0000000000000 .5000000000000 .5000000000000       
         -1 0 0 0 1 0 0 0 1  .5000000000000 .0000000000000 .5000000000000       
          0 1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0 1  .0000000000000 .5000000000000 .5000000000000       
          0-1 0-1 0 0 0 0 1  .5000000000000 .5000000000000 .0000000000000       
I4/MMM                                                                          
          8                                                                     
          1 2 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
I4/MCM                                                                          
          8                                                                     
          1 2 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          0 1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          0-1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
I41/AMD                                                                         
          8                                                                     
          1 2 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0 1  .2500000000000 .7500000000000 .2500000000000       
         -1 0 0 0-1 0 0 0 1  .5000000000000 .0000000000000 .5000000000000       
          0 1 0-1 0 0 0 0 1  .2500000000000 .2500000000000 .7500000000000       
         -1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 1 0 0 0 0 1  .7500000000000 .7500000000000 .2500000000000       
          1 0 0 0-1 0 0 0 1  .5000000000000 .0000000000000 .5000000000000       
          0-1 0-1 0 0 0 0 1  .7500000000000 .2500000000000 .7500000000000       
I41/AMDA                                                                        
         16                                                                     
          0 2 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0 1 0 0 0-1  .0000000000000 .5000000000000 .2500000000000       
         -1 0 0 0-1 0 0 0-1  .0000000000000 .5000000000000 .2500000000000       
         -1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .5000000000000 .2500000000000       
          1 0 0 0-1 0 0 0-1  .0000000000000 .5000000000000 .2500000000000       
          0 1 0 1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0-1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 1 0 0 0 0 1  .0000000000000 .5000000000000 .2500000000000       
          0-1 0-1 0 0 0 0 1  .0000000000000 .5000000000000 .2500000000000       
          0-1 0 1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0 1  .0000000000000 .5000000000000 .2500000000000       
          0 1 0-1 0 0 0 0 1  .0000000000000 .5000000000000 .2500000000000       
I41/ACD                                                                         
          8                                                                     
          1 2 4                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0 1  .2500000000000 .7500000000000 .2500000000000       
         -1 0 0 0-1 0 0 0 1  .5000000000000 .0000000000000 .5000000000000       
          0 1 0-1 0 0 0 0 1  .2500000000000 .2500000000000 .7500000000000       
         -1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          0 1 0 1 0 0 0 0 1  .7500000000000 .7500000000000 .7500000000000       
          1 0 0 0-1 0 0 0 1  .5000000000000 .0000000000000 .0000000000000       
          0-1 0-1 0 0 0 0 1  .7500000000000 .2500000000000 .2500000000000       
P3                                                                              
          3                                                                     
          0 1 5                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
P31                                                                             
          3                                                                     
          0 1 5                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .3333333333333       
         -1 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .6666666666667       
P32                                                                             
          3                                                                     
          0 1 5                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .6666666666667       
         -1 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .3333333333333       
R3                                                                              
          3                                                                     
          0 3 5                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
P-3                                                                             
          3                                                                     
          1 1 5                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
R-3                                                                             
          3                                                                     
          1 3 5                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
P312                                                                            
          6                                                                     
          0 1 5                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0-1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 1 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 1-1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
P321                                                                            
          6                                                                     
          0 1 5                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          1-1 0 0-1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0-1 1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
P3112                                                                           
          6                                                                     
          0 1 5                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .3333333333333       
         -1 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .6666666666667       
          0-1 0-1 0 0 0 0-1  .0000000000000 .0000000000000 .6666666666667       
         -1 1 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .3333333333333       
          1 0 0 1-1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
P3121                                                                           
          6                                                                     
          0 1 5                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .3333333333333       
         -1 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .6666666666667       
          0 1 0 1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          1-1 0 0-1 0 0 0-1  .0000000000000 .0000000000000 .6666666666667       
         -1 0 0-1 1 0 0 0-1  .0000000000000 .0000000000000 .3333333333333       
P3212                                                                           
          6                                                                     
          0 1 5                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .6666666666667       
         -1 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .3333333333333       
          0-1 0-1 0 0 0 0-1  .0000000000000 .0000000000000 .3333333333333       
         -1 1 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .6666666666667       
          1 0 0 1-1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
P3221                                                                           
          6                                                                     
          0 1 5                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .6666666666667       
         -1 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .3333333333333       
          0 1 0 1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          1-1 0 0-1 0 0 0-1  .0000000000000 .0000000000000 .3333333333333       
         -1 0 0-1 1 0 0 0-1  .0000000000000 .0000000000000 .6666666666667       
R32                                                                             
          6                                                                     
          0 3 5                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          1-1 0 0-1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0-1 1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
P3M1                                                                            
          6                                                                     
          0 1 5                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 1 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
P31M                                                                            
          6                                                                     
          0 1 5                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1-1 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0-1 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
P3C1                                                                            
          6                                                                     
          0 1 5                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
         -1 1 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          1 0 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
P31C                                                                            
          6                                                                     
          0 1 5                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          1-1 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
         -1 0 0-1 1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
R3M                                                                             
          6                                                                     
          0 3 5                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 1 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
R3C                                                                             
          6                                                                     
          0 3 5                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
         -1 1 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          1 0 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
P-31M                                                                           
          6                                                                     
          1 1 5                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1-1 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0-1 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
P-31C                                                                           
          6                                                                     
          1 1 5                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          1-1 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
         -1 0 0-1 1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
P-3M1                                                                           
          6                                                                     
          1 1 5                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 1 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
P-3C1                                                                           
          6                                                                     
          1 1 5                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
         -1 1 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          1 0 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
R-3M                                                                            
          6                                                                     
          1 3 5                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 1 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
R-3C                                                                            
          6                                                                     
          1 3 5                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
         -1 1 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          1 0 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
P6                                                                              
          6                                                                     
          0 1 6                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1-1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
P61                                                                             
          6                                                                     
          0 1 6                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1-1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .1666666666667       
          0-1 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .3333333333333       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
         -1 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .6666666666667       
          0 1 0-1 1 0 0 0 1  .0000000000000 .0000000000000 .8333333333333       
P65                                                                             
          6                                                                     
          0 1 6                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1-1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .8333333333333       
          0-1 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .6666666666667       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
         -1 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .3333333333333       
          0 1 0-1 1 0 0 0 1  .0000000000000 .0000000000000 .1666666666667       
P62                                                                             
          6                                                                     
          0 1 6                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1-1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .3333333333333       
          0-1 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .6666666666667       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .3333333333333       
          0 1 0-1 1 0 0 0 1  .0000000000000 .0000000000000 .6666666666667       
P64                                                                             
          6                                                                     
          0 1 6                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1-1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .6666666666667       
          0-1 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .3333333333333       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .6666666666667       
          0 1 0-1 1 0 0 0 1  .0000000000000 .0000000000000 .3333333333333       
P63                                                                             
          6                                                                     
          0 1 6                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1-1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          0-1 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
         -1 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
P-6                                                                             
          6                                                                     
          0 1 6                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1-1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 1 0-1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
P6/M                                                                            
          6                                                                     
          1 1 6                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1-1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
P63/M                                                                           
          6                                                                     
          1 1 6                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1-1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          0-1 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
         -1 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
P622                                                                            
         12                                                                     
          0 1 6                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1-1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 1-1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          1-1 0 0-1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0-1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0-1 1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 1 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
P6122                                                                           
         12                                                                     
          0 1 6                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1-1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .1666666666667       
          0-1 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .3333333333333       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
         -1 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .6666666666667       
          0 1 0-1 1 0 0 0 1  .0000000000000 .0000000000000 .8333333333333       
          0 1 0 1 0 0 0 0-1  .0000000000000 .0000000000000 .3333333333333       
          1 0 0 1-1 0 0 0-1  .0000000000000 .0000000000000 .1666666666667       
          1-1 0 0-1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0-1 0 0 0 0-1  .0000000000000 .0000000000000 .8333333333333       
         -1 0 0-1 1 0 0 0-1  .0000000000000 .0000000000000 .6666666666667       
         -1 1 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .5000000000000       
P6522                                                                           
         12                                                                     
          0 1 6                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1-1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .8333333333333       
          0-1 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .6666666666667       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
         -1 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .3333333333333       
          0 1 0-1 1 0 0 0 1  .0000000000000 .0000000000000 .1666666666667       
          0 1 0 1 0 0 0 0-1  .0000000000000 .0000000000000 .6666666666667       
          1 0 0 1-1 0 0 0-1  .0000000000000 .0000000000000 .8333333333333       
          1-1 0 0-1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0-1 0 0 0 0-1  .0000000000000 .0000000000000 .1666666666667       
         -1 0 0-1 1 0 0 0-1  .0000000000000 .0000000000000 .3333333333333       
         -1 1 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .5000000000000       
P6222                                                                           
         12                                                                     
          0 1 6                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1-1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .3333333333333       
          0-1 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .6666666666667       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .3333333333333       
          0 1 0-1 1 0 0 0 1  .0000000000000 .0000000000000 .6666666666667       
          0 1 0 1 0 0 0 0-1  .0000000000000 .0000000000000 .6666666666667       
          1 0 0 1-1 0 0 0-1  .0000000000000 .0000000000000 .3333333333333       
          1-1 0 0-1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0-1 0 0 0 0-1  .0000000000000 .0000000000000 .6666666666667       
         -1 0 0-1 1 0 0 0-1  .0000000000000 .0000000000000 .3333333333333       
         -1 1 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
P6422                                                                           
         12                                                                     
          0 1 6                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1-1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .6666666666667       
          0-1 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .3333333333333       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .6666666666667       
          0 1 0-1 1 0 0 0 1  .0000000000000 .0000000000000 .3333333333333       
          0 1 0 1 0 0 0 0-1  .0000000000000 .0000000000000 .3333333333333       
          1 0 0 1-1 0 0 0-1  .0000000000000 .0000000000000 .6666666666667       
          1-1 0 0-1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0-1 0 0 0 0-1  .0000000000000 .0000000000000 .3333333333333       
         -1 0 0-1 1 0 0 0-1  .0000000000000 .0000000000000 .6666666666667       
         -1 1 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
P6322                                                                           
         12                                                                     
          0 1 6                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1-1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          0-1 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
         -1 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          0 1 0 1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 1-1 0 0 0-1  .0000000000000 .0000000000000 .5000000000000       
          1-1 0 0-1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0-1 0 0 0 0-1  .0000000000000 .0000000000000 .5000000000000       
         -1 0 0-1 1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 1 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .5000000000000       
P6MM                                                                            
         12                                                                     
          0 1 6                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1-1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0-1 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 1 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1-1 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
P6CC                                                                            
         12                                                                     
          0 1 6                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1-1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
         -1 0 0-1 1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
         -1 1 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          0 1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          1 0 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          1-1 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
P63CM                                                                           
         12                                                                     
          0 1 6                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1-1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          0-1 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
         -1 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          0-1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
         -1 0 0-1 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 1 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          0 1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          1-1 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
P63MC                                                                           
         12                                                                     
          0 1 6                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1-1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          0-1 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
         -1 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          0-1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0-1 1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
         -1 1 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          1 0 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1-1 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
P-6M2                                                                           
         12                                                                     
          0 1 6                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1-1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 1 0-1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 1 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0-1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 1 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 1-1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
P-6C2                                                                           
         12                                                                     
          0 1 6                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .5000000000000       
          0-1 0 1-1 0 0 0-1  .0000000000000 .0000000000000 .5000000000000       
         -1 1 0-1 0 0 0 0-1  .0000000000000 .0000000000000 .5000000000000       
          0-1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
         -1 1 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          1 0 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          0-1 0-1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 1 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 1-1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
P-62M                                                                           
         12                                                                     
          0 1 6                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1-1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 1 0-1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          1-1 0 0-1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0-1 1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1-1 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0-1 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
P-62C                                                                           
         12                                                                     
          0 1 6                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .5000000000000       
          0-1 0 1-1 0 0 0-1  .0000000000000 .0000000000000 .5000000000000       
         -1 1 0-1 0 0 0 0-1  .0000000000000 .0000000000000 .5000000000000       
          0 1 0 1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          1-1 0 0-1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0-1 1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          1-1 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
         -1 0 0-1 1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
P6/MMM                                                                          
         12                                                                     
          1 1 6                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1-1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0-1 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 1 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1-1 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
P6/MCC                                                                          
         12                                                                     
          1 1 6                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1-1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
         -1 0 0-1 1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
         -1 1 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          0 1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          1 0 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          1-1 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
P63/MCM                                                                         
         12                                                                     
          1 1 6                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1-1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          0-1 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
         -1 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          0-1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
         -1 0 0-1 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 1 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          0 1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          1-1 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
P63/MMC                                                                         
         12                                                                     
          1 1 6                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1-1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          0-1 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
         -1 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          0-1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0-1 1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
         -1 1 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          1 0 0 1-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1-1 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
P23                                                                             
         12                                                                     
          0 1 7                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 0 0 1 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 0 0-1 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 0 0-1-1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 0 0 1-1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 0 1 1 0 0 0 1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1 1 0 0 0-1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1-1 0 0 0 1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0 1-1 0 0 0-1 0  .0000000000000 .0000000000000 .0000000000000       
F23                                                                             
         12                                                                     
          0 4 7                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 0 0 1 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 0 0-1 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 0 0-1-1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 0 0 1-1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 0 1 1 0 0 0 1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1 1 0 0 0-1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1-1 0 0 0 1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0 1-1 0 0 0-1 0  .0000000000000 .0000000000000 .0000000000000       
I23                                                                             
         12                                                                     
          0 2 7                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 0 0 1 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 0 0-1 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 0 0-1-1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 0 0 1-1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 0 1 1 0 0 0 1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1 1 0 0 0-1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1-1 0 0 0 1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0 1-1 0 0 0-1 0  .0000000000000 .0000000000000 .0000000000000       
P213                                                                            
         12                                                                     
          0 1 7                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .5000000000000 .5000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .5000000000000 .5000000000000       
         -1 0 0 0-1 0 0 0 1  .5000000000000 .0000000000000 .5000000000000       
          0 1 0 0 0 1 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 0 0-1 1 0 0  .5000000000000 .0000000000000 .5000000000000       
          0 1 0 0 0-1-1 0 0  .5000000000000 .5000000000000 .0000000000000       
          0-1 0 0 0 1-1 0 0  .0000000000000 .5000000000000 .5000000000000       
          0 0 1 1 0 0 0 1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1 1 0 0 0-1 0  .0000000000000 .5000000000000 .5000000000000       
          0 0-1-1 0 0 0 1 0  .5000000000000 .0000000000000 .5000000000000       
          0 0 1-1 0 0 0-1 0  .5000000000000 .5000000000000 .0000000000000       
I213                                                                            
         12                                                                     
          0 2 7                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .5000000000000 .5000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .5000000000000 .5000000000000       
         -1 0 0 0-1 0 0 0 1  .5000000000000 .0000000000000 .5000000000000       
          0 1 0 0 0 1 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 0 0-1 1 0 0  .5000000000000 .0000000000000 .5000000000000       
          0 1 0 0 0-1-1 0 0  .5000000000000 .5000000000000 .0000000000000       
          0-1 0 0 0 1-1 0 0  .0000000000000 .5000000000000 .5000000000000       
          0 0 1 1 0 0 0 1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1 1 0 0 0-1 0  .0000000000000 .5000000000000 .5000000000000       
          0 0-1-1 0 0 0 1 0  .5000000000000 .0000000000000 .5000000000000       
          0 0 1-1 0 0 0-1 0  .5000000000000 .5000000000000 .0000000000000       
PM3                                                                             
         12                                                                     
          1 1 7                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 0 0 1 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 0 0-1 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 0 0-1-1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 0 0 1-1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 0 1 1 0 0 0 1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1 1 0 0 0-1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1-1 0 0 0 1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0 1-1 0 0 0-1 0  .0000000000000 .0000000000000 .0000000000000       
PN3                                                                             
         12                                                                     
          1 1 7                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .0000000000000 .5000000000000 .5000000000000       
         -1 0 0 0 1 0 0 0-1  .5000000000000 .0000000000000 .5000000000000       
         -1 0 0 0-1 0 0 0 1  .5000000000000 .5000000000000 .0000000000000       
          0 1 0 0 0 1 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 0 0-1 1 0 0  .5000000000000 .5000000000000 .0000000000000       
          0 1 0 0 0-1-1 0 0  .0000000000000 .5000000000000 .5000000000000       
          0-1 0 0 0 1-1 0 0  .5000000000000 .0000000000000 .5000000000000       
          0 0 1 1 0 0 0 1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1 1 0 0 0-1 0  .5000000000000 .0000000000000 .5000000000000       
          0 0-1-1 0 0 0 1 0  .5000000000000 .5000000000000 .0000000000000       
          0 0 1-1 0 0 0-1 0  .0000000000000 .5000000000000 .5000000000000       
FM3                                                                             
         12                                                                     
          1 4 7                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 0 0 1 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 0 0-1 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 0 0-1-1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 0 0 1-1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 0 1 1 0 0 0 1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1 1 0 0 0-1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1-1 0 0 0 1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0 1-1 0 0 0-1 0  .0000000000000 .0000000000000 .0000000000000       
FD3                                                                             
         12                                                                     
          1 4 7                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .0000000000000 .2500000000000 .2500000000000       
         -1 0 0 0 1 0 0 0-1  .2500000000000 .0000000000000 .2500000000000       
         -1 0 0 0-1 0 0 0 1  .2500000000000 .2500000000000 .0000000000000       
          0 1 0 0 0 1 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 0 0-1 1 0 0  .2500000000000 .2500000000000 .0000000000000       
          0 1 0 0 0-1-1 0 0  .0000000000000 .2500000000000 .2500000000000       
          0-1 0 0 0 1-1 0 0  .2500000000000 .0000000000000 .2500000000000       
          0 0 1 1 0 0 0 1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1 1 0 0 0-1 0  .2500000000000 .0000000000000 .2500000000000       
          0 0-1-1 0 0 0 1 0  .2500000000000 .2500000000000 .0000000000000       
          0 0 1-1 0 0 0-1 0  .0000000000000 .2500000000000 .2500000000000       
IM3                                                                             
         12                                                                     
          1 2 7                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 0 0 1 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 0 0-1 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 0 0-1-1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 0 0 1-1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 0 1 1 0 0 0 1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1 1 0 0 0-1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1-1 0 0 0 1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0 1-1 0 0 0-1 0  .0000000000000 .0000000000000 .0000000000000       
PA3                                                                             
         12                                                                     
          1 1 7                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .5000000000000 .5000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .5000000000000 .5000000000000       
         -1 0 0 0-1 0 0 0 1  .5000000000000 .0000000000000 .5000000000000       
          0 1 0 0 0 1 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 0 0-1 1 0 0  .5000000000000 .0000000000000 .5000000000000       
          0 1 0 0 0-1-1 0 0  .5000000000000 .5000000000000 .0000000000000       
          0-1 0 0 0 1-1 0 0  .0000000000000 .5000000000000 .5000000000000       
          0 0 1 1 0 0 0 1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1 1 0 0 0-1 0  .0000000000000 .5000000000000 .5000000000000       
          0 0-1-1 0 0 0 1 0  .5000000000000 .0000000000000 .5000000000000       
          0 0 1-1 0 0 0-1 0  .5000000000000 .5000000000000 .0000000000000       
IA3                                                                             
         12                                                                     
          1 2 7                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .5000000000000 .5000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .5000000000000 .5000000000000       
         -1 0 0 0-1 0 0 0 1  .5000000000000 .0000000000000 .5000000000000       
          0 1 0 0 0 1 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 0 0-1 1 0 0  .5000000000000 .0000000000000 .5000000000000       
          0 1 0 0 0-1-1 0 0  .5000000000000 .5000000000000 .0000000000000       
          0-1 0 0 0 1-1 0 0  .0000000000000 .5000000000000 .5000000000000       
          0 0 1 1 0 0 0 1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1 1 0 0 0-1 0  .0000000000000 .5000000000000 .5000000000000       
          0 0-1-1 0 0 0 1 0  .5000000000000 .0000000000000 .5000000000000       
          0 0 1-1 0 0 0-1 0  .5000000000000 .5000000000000 .0000000000000       
P432                                                                            
         24                                                                     
          0 1 7                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 0 0 1 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 0 0-1 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 0 0-1-1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 0 0 1-1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 0 1 1 0 0 0 1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1 1 0 0 0-1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1-1 0 0 0 1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0 1-1 0 0 0-1 0  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0-1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          0 0 1 0 1 0-1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1 0-1 0-1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1 0 1 0 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 0 1 0-1 0 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0 0 1 0-1 0  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0 0-1 0 1 0  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 0-1 0-1 0  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 0 1 0 1 0  .0000000000000 .0000000000000 .0000000000000       
P4232                                                                           
         24                                                                     
          0 1 7                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 0 0 1 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 0 0-1 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 0 0-1-1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 0 0 1-1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 0 1 1 0 0 0 1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1 1 0 0 0-1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1-1 0 0 0 1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0 1-1 0 0 0-1 0  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 1 0 0 0 0-1  .5000000000000 .5000000000000 .5000000000000       
          0-1 0 1 0 0 0 0 1  .5000000000000 .5000000000000 .5000000000000       
          0 1 0-1 0 0 0 0 1  .5000000000000 .5000000000000 .5000000000000       
          0-1 0-1 0 0 0 0-1  .5000000000000 .5000000000000 .5000000000000       
          0 0 1 0 1 0-1 0 0  .5000000000000 .5000000000000 .5000000000000       
          0 0-1 0-1 0-1 0 0  .5000000000000 .5000000000000 .5000000000000       
          0 0-1 0 1 0 1 0 0  .5000000000000 .5000000000000 .5000000000000       
          0 0 1 0-1 0 1 0 0  .5000000000000 .5000000000000 .5000000000000       
          1 0 0 0 0 1 0-1 0  .5000000000000 .5000000000000 .5000000000000       
          1 0 0 0 0-1 0 1 0  .5000000000000 .5000000000000 .5000000000000       
         -1 0 0 0 0-1 0-1 0  .5000000000000 .5000000000000 .5000000000000       
         -1 0 0 0 0 1 0 1 0  .5000000000000 .5000000000000 .5000000000000       
F432                                                                            
         24                                                                     
          0 4 7                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 0 0 1 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 0 0-1 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 0 0-1-1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 0 0 1-1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 0 1 1 0 0 0 1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1 1 0 0 0-1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1-1 0 0 0 1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0 1-1 0 0 0-1 0  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0-1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          0 0 1 0 1 0-1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1 0-1 0-1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1 0 1 0 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 0 1 0-1 0 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0 0 1 0-1 0  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0 0-1 0 1 0  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 0-1 0-1 0  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 0 1 0 1 0  .0000000000000 .0000000000000 .0000000000000       
F4132                                                                           
         24                                                                     
          0 4 7                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 0 0 1 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 0 0-1 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 0 0-1-1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 0 0 1-1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 0 1 1 0 0 0 1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1 1 0 0 0-1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1-1 0 0 0 1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0 1-1 0 0 0-1 0  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 1 0 0 0 0-1  .2500000000000 .2500000000000 .2500000000000       
          0-1 0 1 0 0 0 0 1  .2500000000000 .2500000000000 .2500000000000       
          0 1 0-1 0 0 0 0 1  .2500000000000 .2500000000000 .2500000000000       
          0-1 0-1 0 0 0 0-1  .2500000000000 .2500000000000 .2500000000000       
          0 0 1 0 1 0-1 0 0  .2500000000000 .2500000000000 .2500000000000       
          0 0-1 0-1 0-1 0 0  .2500000000000 .2500000000000 .2500000000000       
          0 0-1 0 1 0 1 0 0  .2500000000000 .2500000000000 .2500000000000       
          0 0 1 0-1 0 1 0 0  .2500000000000 .2500000000000 .2500000000000       
          1 0 0 0 0 1 0-1 0  .2500000000000 .2500000000000 .2500000000000       
          1 0 0 0 0-1 0 1 0  .2500000000000 .2500000000000 .2500000000000       
         -1 0 0 0 0-1 0-1 0  .2500000000000 .2500000000000 .2500000000000       
         -1 0 0 0 0 1 0 1 0  .2500000000000 .2500000000000 .2500000000000       
I432                                                                            
         24                                                                     
          0 2 7                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 0 0 1 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 0 0-1 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 0 0-1-1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 0 0 1-1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 0 1 1 0 0 0 1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1 1 0 0 0-1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1-1 0 0 0 1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0 1-1 0 0 0-1 0  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0-1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          0 0 1 0 1 0-1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1 0-1 0-1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1 0 1 0 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 0 1 0-1 0 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0 0 1 0-1 0  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0 0-1 0 1 0  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 0-1 0-1 0  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 0 1 0 1 0  .0000000000000 .0000000000000 .0000000000000       
P4332                                                                           
         24                                                                     
          0 1 7                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .5000000000000 .5000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .5000000000000 .5000000000000       
         -1 0 0 0-1 0 0 0 1  .5000000000000 .0000000000000 .5000000000000       
          0 1 0 0 0 1 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 0 0-1 1 0 0  .5000000000000 .0000000000000 .5000000000000       
          0 1 0 0 0-1-1 0 0  .5000000000000 .5000000000000 .0000000000000       
          0-1 0 0 0 1-1 0 0  .0000000000000 .5000000000000 .5000000000000       
          0 0 1 1 0 0 0 1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1 1 0 0 0-1 0  .0000000000000 .5000000000000 .5000000000000       
          0 0-1-1 0 0 0 1 0  .5000000000000 .0000000000000 .5000000000000       
          0 0 1-1 0 0 0-1 0  .5000000000000 .5000000000000 .0000000000000       
          0 1 0 1 0 0 0 0-1  .2500000000000 .7500000000000 .7500000000000       
          0-1 0 1 0 0 0 0 1  .7500000000000 .2500000000000 .7500000000000       
          0 1 0-1 0 0 0 0 1  .7500000000000 .7500000000000 .2500000000000       
          0-1 0-1 0 0 0 0-1  .2500000000000 .2500000000000 .2500000000000       
          0 0 1 0 1 0-1 0 0  .2500000000000 .7500000000000 .7500000000000       
          0 0-1 0-1 0-1 0 0  .2500000000000 .2500000000000 .2500000000000       
          0 0-1 0 1 0 1 0 0  .7500000000000 .2500000000000 .7500000000000       
          0 0 1 0-1 0 1 0 0  .7500000000000 .7500000000000 .2500000000000       
          1 0 0 0 0 1 0-1 0  .2500000000000 .7500000000000 .7500000000000       
          1 0 0 0 0-1 0 1 0  .7500000000000 .7500000000000 .2500000000000       
         -1 0 0 0 0-1 0-1 0  .2500000000000 .2500000000000 .2500000000000       
         -1 0 0 0 0 1 0 1 0  .7500000000000 .2500000000000 .7500000000000       
P4132                                                                           
         24                                                                     
          0 1 7                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .5000000000000 .5000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .5000000000000 .5000000000000       
         -1 0 0 0-1 0 0 0 1  .5000000000000 .0000000000000 .5000000000000       
          0 1 0 0 0 1 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 0 0-1 1 0 0  .5000000000000 .0000000000000 .5000000000000       
          0 1 0 0 0-1-1 0 0  .5000000000000 .5000000000000 .0000000000000       
          0-1 0 0 0 1-1 0 0  .0000000000000 .5000000000000 .5000000000000       
          0 0 1 1 0 0 0 1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1 1 0 0 0-1 0  .0000000000000 .5000000000000 .5000000000000       
          0 0-1-1 0 0 0 1 0  .5000000000000 .0000000000000 .5000000000000       
          0 0 1-1 0 0 0-1 0  .5000000000000 .5000000000000 .0000000000000       
          0 1 0 1 0 0 0 0-1  .7500000000000 .2500000000000 .2500000000000       
          0-1 0 1 0 0 0 0 1  .2500000000000 .7500000000000 .2500000000000       
          0 1 0-1 0 0 0 0 1  .2500000000000 .2500000000000 .7500000000000       
          0-1 0-1 0 0 0 0-1  .7500000000000 .7500000000000 .7500000000000       
          0 0 1 0 1 0-1 0 0  .7500000000000 .2500000000000 .2500000000000       
          0 0-1 0-1 0-1 0 0  .7500000000000 .7500000000000 .7500000000000       
          0 0-1 0 1 0 1 0 0  .2500000000000 .7500000000000 .2500000000000       
          0 0 1 0-1 0 1 0 0  .2500000000000 .2500000000000 .7500000000000       
          1 0 0 0 0 1 0-1 0  .7500000000000 .2500000000000 .2500000000000       
          1 0 0 0 0-1 0 1 0  .2500000000000 .2500000000000 .7500000000000       
         -1 0 0 0 0-1 0-1 0  .7500000000000 .7500000000000 .7500000000000       
         -1 0 0 0 0 1 0 1 0  .2500000000000 .7500000000000 .2500000000000       
I4132                                                                           
         24                                                                     
          0 2 7                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .5000000000000 .5000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .5000000000000 .5000000000000       
         -1 0 0 0-1 0 0 0 1  .5000000000000 .0000000000000 .5000000000000       
          0 1 0 0 0 1 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 0 0-1 1 0 0  .5000000000000 .0000000000000 .5000000000000       
          0 1 0 0 0-1-1 0 0  .5000000000000 .5000000000000 .0000000000000       
          0-1 0 0 0 1-1 0 0  .0000000000000 .5000000000000 .5000000000000       
          0 0 1 1 0 0 0 1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1 1 0 0 0-1 0  .0000000000000 .5000000000000 .5000000000000       
          0 0-1-1 0 0 0 1 0  .5000000000000 .0000000000000 .5000000000000       
          0 0 1-1 0 0 0-1 0  .5000000000000 .5000000000000 .0000000000000       
          0 1 0 1 0 0 0 0-1  .2500000000000 .7500000000000 .7500000000000       
          0-1 0 1 0 0 0 0 1  .7500000000000 .2500000000000 .7500000000000       
          0 1 0-1 0 0 0 0 1  .7500000000000 .7500000000000 .2500000000000       
          0-1 0-1 0 0 0 0-1  .2500000000000 .2500000000000 .2500000000000       
          0 0 1 0 1 0-1 0 0  .2500000000000 .7500000000000 .7500000000000       
          0 0-1 0-1 0-1 0 0  .2500000000000 .2500000000000 .2500000000000       
          0 0-1 0 1 0 1 0 0  .7500000000000 .2500000000000 .7500000000000       
          0 0 1 0-1 0 1 0 0  .7500000000000 .7500000000000 .2500000000000       
          1 0 0 0 0 1 0-1 0  .2500000000000 .7500000000000 .7500000000000       
          1 0 0 0 0-1 0 1 0  .7500000000000 .7500000000000 .2500000000000       
         -1 0 0 0 0-1 0-1 0  .2500000000000 .2500000000000 .2500000000000       
         -1 0 0 0 0 1 0 1 0  .7500000000000 .2500000000000 .7500000000000       
P-43M                                                                           
         24                                                                     
          0 1 7                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 0 0 1 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 0 0-1 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 0 0-1-1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 0 0 1-1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 0 1 1 0 0 0 1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1 1 0 0 0-1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1-1 0 0 0 1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0 1-1 0 0 0-1 0  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 0 1 0 1 0 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1 0-1 0 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1 0 1 0-1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 0 1 0-1 0-1 0 0  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0 0 1 0 1 0  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0 0-1 0-1 0  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 0-1 0 1 0  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 0 1 0-1 0  .0000000000000 .0000000000000 .0000000000000       
F-43M                                                                           
         24                                                                     
          0 4 7                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 0 0 1 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 0 0-1 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 0 0-1-1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 0 0 1-1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 0 1 1 0 0 0 1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1 1 0 0 0-1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1-1 0 0 0 1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0 1-1 0 0 0-1 0  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 0 1 0 1 0 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1 0-1 0 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1 0 1 0-1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 0 1 0-1 0-1 0 0  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0 0 1 0 1 0  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0 0-1 0-1 0  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 0-1 0 1 0  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 0 1 0-1 0  .0000000000000 .0000000000000 .0000000000000       
I-43M                                                                           
         24                                                                     
          0 2 7                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 0 0 1 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 0 0-1 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 0 0-1-1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 0 0 1-1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 0 1 1 0 0 0 1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1 1 0 0 0-1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1-1 0 0 0 1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0 1-1 0 0 0-1 0  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 0 1 0 1 0 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1 0-1 0 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1 0 1 0-1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 0 1 0-1 0-1 0 0  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0 0 1 0 1 0  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0 0-1 0-1 0  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 0-1 0 1 0  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 0 1 0-1 0  .0000000000000 .0000000000000 .0000000000000       
P-43N                                                                           
         24                                                                     
          0 1 7                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 0 0 1 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 0 0-1 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 0 0-1-1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 0 0 1-1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 0 1 1 0 0 0 1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1 1 0 0 0-1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1-1 0 0 0 1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0 1-1 0 0 0-1 0  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 1 0 0 0 0 1  .5000000000000 .5000000000000 .5000000000000       
          0-1 0 1 0 0 0 0-1  .5000000000000 .5000000000000 .5000000000000       
          0 1 0-1 0 0 0 0-1  .5000000000000 .5000000000000 .5000000000000       
          0-1 0-1 0 0 0 0 1  .5000000000000 .5000000000000 .5000000000000       
          0 0 1 0 1 0 1 0 0  .5000000000000 .5000000000000 .5000000000000       
          0 0-1 0-1 0 1 0 0  .5000000000000 .5000000000000 .5000000000000       
          0 0-1 0 1 0-1 0 0  .5000000000000 .5000000000000 .5000000000000       
          0 0 1 0-1 0-1 0 0  .5000000000000 .5000000000000 .5000000000000       
          1 0 0 0 0 1 0 1 0  .5000000000000 .5000000000000 .5000000000000       
          1 0 0 0 0-1 0-1 0  .5000000000000 .5000000000000 .5000000000000       
         -1 0 0 0 0-1 0 1 0  .5000000000000 .5000000000000 .5000000000000       
         -1 0 0 0 0 1 0-1 0  .5000000000000 .5000000000000 .5000000000000       
F-43C                                                                           
         24                                                                     
          0 4 7                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 0 0 1 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 0 0-1 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 0 0-1-1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 0 0 1-1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 0 1 1 0 0 0 1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1 1 0 0 0-1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1-1 0 0 0 1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0 1-1 0 0 0-1 0  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          0-1 0 1 0 0 0 0-1  .0000000000000 .0000000000000 .5000000000000       
          0 1 0-1 0 0 0 0-1  .0000000000000 .0000000000000 .5000000000000       
          0-1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          0 0 1 0 1 0 1 0 0  .0000000000000 .0000000000000 .5000000000000       
          0 0-1 0-1 0 1 0 0  .0000000000000 .0000000000000 .5000000000000       
          0 0-1 0 1 0-1 0 0  .0000000000000 .0000000000000 .5000000000000       
          0 0 1 0-1 0-1 0 0  .0000000000000 .0000000000000 .5000000000000       
          1 0 0 0 0 1 0 1 0  .0000000000000 .0000000000000 .5000000000000       
          1 0 0 0 0-1 0-1 0  .0000000000000 .0000000000000 .5000000000000       
         -1 0 0 0 0-1 0 1 0  .0000000000000 .0000000000000 .5000000000000       
         -1 0 0 0 0 1 0-1 0  .0000000000000 .0000000000000 .5000000000000       
I-43D                                                                           
         24                                                                     
          0 2 7                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .5000000000000 .5000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .5000000000000 .5000000000000       
         -1 0 0 0-1 0 0 0 1  .5000000000000 .0000000000000 .5000000000000       
          0 1 0 0 0 1 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 0 0-1 1 0 0  .5000000000000 .0000000000000 .5000000000000       
          0 1 0 0 0-1-1 0 0  .5000000000000 .5000000000000 .0000000000000       
          0-1 0 0 0 1-1 0 0  .0000000000000 .5000000000000 .5000000000000       
          0 0 1 1 0 0 0 1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1 1 0 0 0-1 0  .0000000000000 .5000000000000 .5000000000000       
          0 0-1-1 0 0 0 1 0  .5000000000000 .0000000000000 .5000000000000       
          0 0 1-1 0 0 0-1 0  .5000000000000 .5000000000000 .0000000000000       
          0 1 0 1 0 0 0 0 1  .2500000000000 .2500000000000 .2500000000000       
          0-1 0 1 0 0 0 0-1  .7500000000000 .7500000000000 .2500000000000       
          0 1 0-1 0 0 0 0-1  .7500000000000 .2500000000000 .7500000000000       
          0-1 0-1 0 0 0 0 1  .2500000000000 .7500000000000 .7500000000000       
          0 0 1 0 1 0 1 0 0  .2500000000000 .2500000000000 .2500000000000       
          0 0-1 0-1 0 1 0 0  .2500000000000 .7500000000000 .7500000000000       
          0 0-1 0 1 0-1 0 0  .7500000000000 .7500000000000 .2500000000000       
          0 0 1 0-1 0-1 0 0  .7500000000000 .2500000000000 .7500000000000       
          1 0 0 0 0 1 0 1 0  .2500000000000 .2500000000000 .2500000000000       
          1 0 0 0 0-1 0-1 0  .7500000000000 .2500000000000 .7500000000000       
         -1 0 0 0 0-1 0 1 0  .2500000000000 .7500000000000 .7500000000000       
         -1 0 0 0 0 1 0-1 0  .7500000000000 .7500000000000 .2500000000000       
PM3M                                                                            
         24                                                                     
          1 1 7                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 0 0 1 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 0 0-1 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 0 0-1-1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 0 0 1-1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 0 1 1 0 0 0 1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1 1 0 0 0-1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1-1 0 0 0 1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0 1-1 0 0 0-1 0  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 0 1 0 1 0 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1 0-1 0 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1 0 1 0-1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 0 1 0-1 0-1 0 0  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0 0 1 0 1 0  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0 0-1 0-1 0  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 0-1 0 1 0  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 0 1 0-1 0  .0000000000000 .0000000000000 .0000000000000       
PN3N                                                                            
         24                                                                     
          1 1 7                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .0000000000000 .5000000000000 .5000000000000       
         -1 0 0 0 1 0 0 0-1  .5000000000000 .0000000000000 .5000000000000       
         -1 0 0 0-1 0 0 0 1  .5000000000000 .5000000000000 .0000000000000       
          0 1 0 0 0 1 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 0 0-1 1 0 0  .5000000000000 .5000000000000 .0000000000000       
          0 1 0 0 0-1-1 0 0  .0000000000000 .5000000000000 .5000000000000       
          0-1 0 0 0 1-1 0 0  .5000000000000 .0000000000000 .5000000000000       
          0 0 1 1 0 0 0 1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1 1 0 0 0-1 0  .5000000000000 .0000000000000 .5000000000000       
          0 0-1-1 0 0 0 1 0  .5000000000000 .5000000000000 .0000000000000       
          0 0 1-1 0 0 0-1 0  .0000000000000 .5000000000000 .5000000000000       
          0 1 0 1 0 0 0 0 1  .5000000000000 .5000000000000 .5000000000000       
          0-1 0 1 0 0 0 0-1  .0000000000000 .5000000000000 .0000000000000       
          0 1 0-1 0 0 0 0-1  .5000000000000 .0000000000000 .0000000000000       
          0-1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          0 0 1 0 1 0 1 0 0  .5000000000000 .5000000000000 .5000000000000       
          0 0-1 0-1 0 1 0 0  .0000000000000 .0000000000000 .5000000000000       
          0 0-1 0 1 0-1 0 0  .0000000000000 .5000000000000 .0000000000000       
          0 0 1 0-1 0-1 0 0  .5000000000000 .0000000000000 .0000000000000       
          1 0 0 0 0 1 0 1 0  .5000000000000 .5000000000000 .5000000000000       
          1 0 0 0 0-1 0-1 0  .5000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 0-1 0 1 0  .0000000000000 .0000000000000 .5000000000000       
         -1 0 0 0 0 1 0-1 0  .0000000000000 .5000000000000 .0000000000000       
PM3N                                                                            
         24                                                                     
          1 1 7                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 0 0 1 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 0 0-1 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 0 0-1-1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 0 0 1-1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 0 1 1 0 0 0 1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1 1 0 0 0-1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1-1 0 0 0 1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0 1-1 0 0 0-1 0  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 1 0 0 0 0 1  .5000000000000 .5000000000000 .5000000000000       
          0-1 0 1 0 0 0 0-1  .5000000000000 .5000000000000 .5000000000000       
          0 1 0-1 0 0 0 0-1  .5000000000000 .5000000000000 .5000000000000       
          0-1 0-1 0 0 0 0 1  .5000000000000 .5000000000000 .5000000000000       
          0 0 1 0 1 0 1 0 0  .5000000000000 .5000000000000 .5000000000000       
          0 0-1 0-1 0 1 0 0  .5000000000000 .5000000000000 .5000000000000       
          0 0-1 0 1 0-1 0 0  .5000000000000 .5000000000000 .5000000000000       
          0 0 1 0-1 0-1 0 0  .5000000000000 .5000000000000 .5000000000000       
          1 0 0 0 0 1 0 1 0  .5000000000000 .5000000000000 .5000000000000       
          1 0 0 0 0-1 0-1 0  .5000000000000 .5000000000000 .5000000000000       
         -1 0 0 0 0-1 0 1 0  .5000000000000 .5000000000000 .5000000000000       
         -1 0 0 0 0 1 0-1 0  .5000000000000 .5000000000000 .5000000000000       
PN3M                                                                            
         24                                                                     
          1 1 7                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .0000000000000 .5000000000000 .5000000000000       
         -1 0 0 0 1 0 0 0-1  .5000000000000 .0000000000000 .5000000000000       
         -1 0 0 0-1 0 0 0 1  .5000000000000 .5000000000000 .0000000000000       
          0 1 0 0 0 1 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 0 0-1 1 0 0  .5000000000000 .5000000000000 .0000000000000       
          0 1 0 0 0-1-1 0 0  .0000000000000 .5000000000000 .5000000000000       
          0-1 0 0 0 1-1 0 0  .5000000000000 .0000000000000 .5000000000000       
          0 0 1 1 0 0 0 1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1 1 0 0 0-1 0  .5000000000000 .0000000000000 .5000000000000       
          0 0-1-1 0 0 0 1 0  .5000000000000 .5000000000000 .0000000000000       
          0 0 1-1 0 0 0-1 0  .0000000000000 .5000000000000 .5000000000000       
          0 1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0-1  .5000000000000 .0000000000000 .5000000000000       
          0 1 0-1 0 0 0 0-1  .0000000000000 .5000000000000 .5000000000000       
          0-1 0-1 0 0 0 0 1  .5000000000000 .5000000000000 .0000000000000       
          0 0 1 0 1 0 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1 0-1 0 1 0 0  .5000000000000 .5000000000000 .0000000000000       
          0 0-1 0 1 0-1 0 0  .5000000000000 .0000000000000 .5000000000000       
          0 0 1 0-1 0-1 0 0  .0000000000000 .5000000000000 .5000000000000       
          1 0 0 0 0 1 0 1 0  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0 0-1 0-1 0  .0000000000000 .5000000000000 .5000000000000       
         -1 0 0 0 0-1 0 1 0  .5000000000000 .5000000000000 .0000000000000       
         -1 0 0 0 0 1 0-1 0  .5000000000000 .0000000000000 .5000000000000       
FM3M                                                                            
         24                                                                     
          1 4 7                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 0 0 1 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 0 0-1 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 0 0-1-1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 0 0 1-1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 0 1 1 0 0 0 1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1 1 0 0 0-1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1-1 0 0 0 1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0 1-1 0 0 0-1 0  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 0 1 0 1 0 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1 0-1 0 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1 0 1 0-1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 0 1 0-1 0-1 0 0  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0 0 1 0 1 0  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0 0-1 0-1 0  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 0-1 0 1 0  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 0 1 0-1 0  .0000000000000 .0000000000000 .0000000000000       
FM3C                                                                            
         24                                                                     
          1 4 7                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 0 0 1 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 0 0-1 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 0 0-1-1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 0 0 1-1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 0 1 1 0 0 0 1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1 1 0 0 0-1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1-1 0 0 0 1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0 1-1 0 0 0-1 0  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          0-1 0 1 0 0 0 0-1  .0000000000000 .0000000000000 .5000000000000       
          0 1 0-1 0 0 0 0-1  .0000000000000 .0000000000000 .5000000000000       
          0-1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          0 0 1 0 1 0 1 0 0  .0000000000000 .0000000000000 .5000000000000       
          0 0-1 0-1 0 1 0 0  .0000000000000 .0000000000000 .5000000000000       
          0 0-1 0 1 0-1 0 0  .0000000000000 .0000000000000 .5000000000000       
          0 0 1 0-1 0-1 0 0  .0000000000000 .0000000000000 .5000000000000       
          1 0 0 0 0 1 0 1 0  .0000000000000 .0000000000000 .5000000000000       
          1 0 0 0 0-1 0-1 0  .0000000000000 .0000000000000 .5000000000000       
         -1 0 0 0 0-1 0 1 0  .0000000000000 .0000000000000 .5000000000000       
         -1 0 0 0 0 1 0-1 0  .0000000000000 .0000000000000 .5000000000000       
FD3M                                                                            
         24                                                                     
          1 4 7                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .0000000000000 .2500000000000 .2500000000000       
         -1 0 0 0 1 0 0 0-1  .2500000000000 .0000000000000 .2500000000000       
         -1 0 0 0-1 0 0 0 1  .2500000000000 .2500000000000 .0000000000000       
          0 1 0 0 0 1 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 0 0-1 1 0 0  .2500000000000 .2500000000000 .0000000000000       
          0 1 0 0 0-1-1 0 0  .0000000000000 .2500000000000 .2500000000000       
          0-1 0 0 0 1-1 0 0  .2500000000000 .0000000000000 .2500000000000       
          0 0 1 1 0 0 0 1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1 1 0 0 0-1 0  .2500000000000 .0000000000000 .2500000000000       
          0 0-1-1 0 0 0 1 0  .2500000000000 .2500000000000 .0000000000000       
          0 0 1-1 0 0 0-1 0  .0000000000000 .2500000000000 .2500000000000       
          0 1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0-1  .2500000000000 .0000000000000 .2500000000000       
          0 1 0-1 0 0 0 0-1  .0000000000000 .2500000000000 .2500000000000       
          0-1 0-1 0 0 0 0 1  .2500000000000 .2500000000000 .0000000000000       
          0 0 1 0 1 0 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1 0-1 0 1 0 0  .2500000000000 .2500000000000 .0000000000000       
          0 0-1 0 1 0-1 0 0  .2500000000000 .0000000000000 .2500000000000       
          0 0 1 0-1 0-1 0 0  .0000000000000 .2500000000000 .2500000000000       
          1 0 0 0 0 1 0 1 0  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0 0-1 0-1 0  .0000000000000 .2500000000000 .2500000000000       
         -1 0 0 0 0-1 0 1 0  .2500000000000 .2500000000000 .0000000000000       
         -1 0 0 0 0 1 0-1 0  .2500000000000 .0000000000000 .2500000000000       
FD3C                                                                            
         24                                                                     
          1 4 7                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .0000000000000 .2500000000000 .2500000000000       
         -1 0 0 0 1 0 0 0-1  .2500000000000 .0000000000000 .2500000000000       
         -1 0 0 0-1 0 0 0 1  .2500000000000 .2500000000000 .0000000000000       
          0 1 0 0 0 1 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 0 0-1 1 0 0  .2500000000000 .2500000000000 .0000000000000       
          0 1 0 0 0-1-1 0 0  .0000000000000 .2500000000000 .2500000000000       
          0-1 0 0 0 1-1 0 0  .2500000000000 .0000000000000 .2500000000000       
          0 0 1 1 0 0 0 1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1 1 0 0 0-1 0  .2500000000000 .0000000000000 .2500000000000       
          0 0-1-1 0 0 0 1 0  .2500000000000 .2500000000000 .0000000000000       
          0 0 1-1 0 0 0-1 0  .0000000000000 .2500000000000 .2500000000000       
          0 1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .5000000000000       
          0-1 0 1 0 0 0 0-1  .2500000000000 .0000000000000 .7500000000000       
          0 1 0-1 0 0 0 0-1  .0000000000000 .2500000000000 .7500000000000       
          0-1 0-1 0 0 0 0 1  .2500000000000 .2500000000000 .5000000000000       
          0 0 1 0 1 0 1 0 0  .0000000000000 .0000000000000 .5000000000000       
          0 0-1 0-1 0 1 0 0  .2500000000000 .2500000000000 .5000000000000       
          0 0-1 0 1 0-1 0 0  .2500000000000 .0000000000000 .7500000000000       
          0 0 1 0-1 0-1 0 0  .0000000000000 .2500000000000 .7500000000000       
          1 0 0 0 0 1 0 1 0  .0000000000000 .0000000000000 .5000000000000       
          1 0 0 0 0-1 0-1 0  .0000000000000 .2500000000000 .7500000000000       
         -1 0 0 0 0-1 0 1 0  .2500000000000 .2500000000000 .5000000000000       
         -1 0 0 0 0 1 0-1 0  .2500000000000 .0000000000000 .7500000000000       
IM3M                                                                            
         24                                                                     
          1 2 7                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0-1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 0 0 1 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 0 0-1 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 0 0-1-1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 0 0 1-1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 0 1 1 0 0 0 1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1 1 0 0 0-1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1-1 0 0 0 1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0 1-1 0 0 0-1 0  .0000000000000 .0000000000000 .0000000000000       
          0 1 0 1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          0 1 0-1 0 0 0 0-1  .0000000000000 .0000000000000 .0000000000000       
          0-1 0-1 0 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          0 0 1 0 1 0 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1 0-1 0 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1 0 1 0-1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0 0 1 0-1 0-1 0 0  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0 0 1 0 1 0  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0 0-1 0-1 0  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 0-1 0 1 0  .0000000000000 .0000000000000 .0000000000000       
         -1 0 0 0 0 1 0-1 0  .0000000000000 .0000000000000 .0000000000000       
IA3D                                                                            
         24                                                                     
          1 2 7                                                                 
          1 0 0 0 1 0 0 0 1  .0000000000000 .0000000000000 .0000000000000       
          1 0 0 0-1 0 0 0-1  .5000000000000 .5000000000000 .0000000000000       
         -1 0 0 0 1 0 0 0-1  .0000000000000 .5000000000000 .5000000000000       
         -1 0 0 0-1 0 0 0 1  .5000000000000 .0000000000000 .5000000000000       
          0 1 0 0 0 1 1 0 0  .0000000000000 .0000000000000 .0000000000000       
          0-1 0 0 0-1 1 0 0  .5000000000000 .0000000000000 .5000000000000       
          0 1 0 0 0-1-1 0 0  .5000000000000 .5000000000000 .0000000000000       
          0-1 0 0 0 1-1 0 0  .0000000000000 .5000000000000 .5000000000000       
          0 0 1 1 0 0 0 1 0  .0000000000000 .0000000000000 .0000000000000       
          0 0-1 1 0 0 0-1 0  .0000000000000 .5000000000000 .5000000000000       
          0 0-1-1 0 0 0 1 0  .5000000000000 .0000000000000 .5000000000000       
          0 0 1-1 0 0 0-1 0  .5000000000000 .5000000000000 .0000000000000       
          0 1 0 1 0 0 0 0 1  .2500000000000 .2500000000000 .2500000000000       
          0-1 0 1 0 0 0 0-1  .7500000000000 .7500000000000 .2500000000000       
          0 1 0-1 0 0 0 0-1  .7500000000000 .2500000000000 .7500000000000       
          0-1 0-1 0 0 0 0 1  .2500000000000 .7500000000000 .7500000000000       
          0 0 1 0 1 0 1 0 0  .2500000000000 .2500000000000 .2500000000000       
          0 0-1 0-1 0 1 0 0  .2500000000000 .7500000000000 .7500000000000       
          0 0-1 0 1 0-1 0 0  .7500000000000 .7500000000000 .2500000000000       
          0 0 1 0-1 0-1 0 0  .7500000000000 .2500000000000 .7500000000000       
          1 0 0 0 0 1 0 1 0  .2500000000000 .2500000000000 .2500000000000       
          1 0 0 0 0-1 0-1 0  .7500000000000 .2500000000000 .7500000000000       
         -1 0 0 0 0-1 0 1 0  .2500000000000 .7500000000000 .7500000000000       
         -1 0 0 0 0 1 0-1 0  .7500000000000 .7500000000000 .2500000000000       
 

#}